@@ -1,29 +1,28 @@
 /* AbiSource Program Utilities
  * Copyright (C) 1998-2000 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
- 
+
 
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 #include "ut_types.h"
 #include "ut_assert.h"
-#include "ut_debugmsg.h"
 #include "ev_EditBinding.h"
 #include "ev_EditEventMapper.h"
 #include "ev_EditMethod.h"
@@ -31,14 +30,13 @@
 #include "ev_NamedVirtualKey.h"
 #include "ev_UnixKeyboard.h"
 #include "ev_UnixKeysym2ucs.cpp"
-#include "xap_EncodingManager.h"
-
-#include"ut_mbtowc.h"
+#include "ut_mbtowc.h"
+#include "ut_string_class.h"
 
 //////////////////////////////////////////////////////////////////
 
-static EV_EditBits s_mapVirtualKeyCodeToNVK(gint keyval);
-static bool s_isVirtualKeyCode(gint keyval);
+static EV_EditBits s_mapVirtualKeyCodeToNVK(guint keyval);
+static bool s_isVirtualKeyCode(guint keyval);
 static GdkModifierType s_getAltMask(void);
 
 //////////////////////////////////////////////////////////////////
@@ -73,7 +71,9 @@ bool ev_UnixKeyboard::keyPressEvent(AV_View* pView, GdkEventKey* e)
 	EV_EditBits state = 0;
 	EV_EditEventMapperResult result;
 	EV_EditMethod * pEM;
-	
+
+	UT_uint32 charData = e->keyval;
+
 	if (e->state & GDK_SHIFT_MASK)
 		state |= EV_EMS_SHIFT;
 	if (e->state & GDK_CONTROL_MASK)
@@ -81,11 +81,9 @@ bool ev_UnixKeyboard::keyPressEvent(AV_View* pView, GdkEventKey* e)
 	if (e->state & (s_alt_mask))
 		state |= EV_EMS_ALT;
 
-	//UT_DEBUGMSG(("KeyPressEvent: keyval=%x state=%x\n",e->keyval,state));
-	
-	if (s_isVirtualKeyCode(e->keyval))
+	if (s_isVirtualKeyCode(charData))
 	{
-		EV_EditBits nvk = s_mapVirtualKeyCodeToNVK(e->keyval);
+		EV_EditBits nvk = s_mapVirtualKeyCodeToNVK(charData);
 
 		switch (nvk)
 		{
@@ -93,7 +91,7 @@ bool ev_UnixKeyboard::keyPressEvent(AV_View* pView, GdkEventKey* e)
 			return false;
 		default:
 
-			result = m_pEEM->Keystroke((UT_uint32)EV_EKP_PRESS|state|nvk,&pEM);
+			result = m_pEEM->Keystroke(static_cast<UT_uint32>(EV_EKP_PRESS|state|nvk),&pEM);
 
 			switch (result)
 			{
@@ -102,7 +100,7 @@ bool ev_UnixKeyboard::keyPressEvent(AV_View* pView, GdkEventKey* e)
 				// progress, we should let the system handle it
 				// (this lets things like ALT-F4 work).
 				return false;
-				
+
 			case EV_EEMR_BOGUS_CONT:
 				// If it is a bogus key but in the middle of a sequence,
 				// we should silently eat it (this is to prevent things
@@ -110,115 +108,91 @@ bool ev_UnixKeyboard::keyPressEvent(AV_View* pView, GdkEventKey* e)
 				// to kill us, fine, but they shouldn't be in the middle
 				// of a sequence).
 				return true;
-				
+
 			case EV_EEMR_COMPLETE:
 				UT_ASSERT(pEM);
-				//UT_DEBUGMSG(("invokeKeyboardMethod (1)\n"));
 				invokeKeyboardMethod(pView,pEM,0,0); // no char data to offer
 				return true;
-				
+
 			case EV_EEMR_INCOMPLETE:
 				return true;
-				
+
 			default:
 				UT_ASSERT(0);
 				return true;
 			}
 		}
 	}
-	else
-	{
-		UT_uint16 charData = e->keyval;
-		//UT_DEBUGMSG(("UnixKeyboard::pressKeyEvent: key value %x\n", charData));
+	else 
+	  {
+	    // TODO: is this necessary?
+	    charData = gdk_keyval_to_unicode (charData);
+	    UT_UTF8String utf8 (static_cast<const UT_UCS4Char *>(&charData), 1);
+	    return charDataEvent (pView, state, utf8.utf8_str(), utf8.byteLength());
+	  }
 
-		if(charData>0xff || charData == 0)
-		  result = m_pEEM->Keystroke(EV_EKP_PRESS|state|'a',&pEM);
-		else
-		  result = m_pEEM->Keystroke(EV_EKP_PRESS|state|charData,&pEM);
+	return false;
+}
 
-		switch (result)
-		{
-		case EV_EEMR_BOGUS_START:
-			// If it is a bogus key and we don't have a sequence in
-			// progress, we should let the system handle it
-			// (this lets things like ALT-F4 work).
-			return false;
-			
-		case EV_EEMR_BOGUS_CONT:
-			// If it is a bogus key but in the middle of a sequence,
-			// we should silently eat it (this is to prevent things
-			// like Control-X ALT-F4 from killing us -- if they want
-			// to kill us, fine, but they shouldn't be in the middle
-			// of a sequence).
-			return true;
-			
-		case EV_EEMR_COMPLETE:
-		  {
-			UT_ASSERT(pEM);
+bool ev_UnixKeyboard::charDataEvent(AV_View* pView, EV_EditBits state, const char * text, size_t len)
+{
+	EV_EditEventMapperResult result;
+	EV_EditMethod * pEM;
 
-			UT_UCSChar *ucs = NULL;
-			char *mbs = e->string;
-			int mLength = strlen(mbs);
-			int uLength = 0;
+	/* delete_surrounding commits a 0 length string in some IMs */
+	if (!len)
+		return true;
 
-			/*
-				if gdk fails to translate, then we will try to do this
-				ourselves by calling kesym2ucs
-				
-				if the current locale is utf-8, we will also use keysym2ucs
-			*/
-			
-			if(XAP_EncodingManager::get_instance()->isUnicodeLocale() || mLength == 0)
-			{
-				UT_sint32 u = keysym2ucs(e->keyval);
-				
-				if(u == -1 || u > 0xFFFF) //conversion failed, or more than 16 bit requied
-				{
-					mLength = 0;
-				}
-				else
-				{
-					mLength = 1;
-					ucs = new UT_UCSChar[1];
-					ucs[0] = u;
-				}
-				
-				uLength = mLength;
-				//UT_DEBUGMSG(("#TF: keyval=%x, ucs=%x\n", ucs[0]));
-			}
-			else
-			{
-				UT_Mbtowc m;
-				ucs=new UT_UCSChar[mLength];
-				for(int i=0;i<mLength;++i)
-			  	{
-					wchar_t wc;
-					if(m.mbtowc(wc,mbs[i]))
-					  ucs[uLength++]=wc;
-					//UT_DEBUGMSG(("ucs[i] 0x%04x, ",ucs[i]));
-			  	}					
-			  	//UT_DEBUGMSG((" uLength %d\n",uLength));
-			 }
-			//UT_DEBUGMSG(("invokeKeyboardMethod (2)\n"));
-			invokeKeyboardMethod(pView,pEM,ucs,uLength); // no char data to offer
-			delete[] ucs;
- 			return true;
+	UT_UCS4String ucs (text, len);
+
+	UT_uint32 charData = static_cast<UT_uint32>(ucs[0]);
 	
-		  }
-		case EV_EEMR_INCOMPLETE:
-			return true;
-			
-		default:
-			UT_ASSERT(0);
-			return true;
-		}
-	}
+	xxx_UT_DEBUGMSG(("DOM: charData: %d | length: %d | string: '%s'\n", charData, len, text));
+	if (charData == 32)
+	  charData = 'a'; // HACK!!! for space bar not working. investigate more....
+
+	if(charData>0xff || charData == 0)
+	  result = m_pEEM->Keystroke(EV_EKP_PRESS|state|'a',&pEM);
+	else
+	  result = m_pEEM->Keystroke(EV_EKP_PRESS|state|charData,&pEM);
+	
+	switch (result)
+	  {
+	  case EV_EEMR_BOGUS_START:
+	    // If it is a bogus key and we don't have a sequence in
+	    // progress, we should let the system handle it
+	    // (this lets things like ALT-F4 work).
+	    return false;
+	    
+	  case EV_EEMR_BOGUS_CONT:
+	    // If it is a bogus key but in the middle of a sequence,
+	    // we should silently eat it (this is to prevent things
+	    // like Control-X ALT-F4 from killing us -- if they want
+	    // to kill us, fine, but they shouldn't be in the middle
+	    // of a sequence).
+	    return true;
+	    
+	  case EV_EEMR_COMPLETE:
+	    {
+	      UT_ASSERT(pEM);
+
+	      invokeKeyboardMethod(pView,pEM,
+				   const_cast<UT_UCS4Char *>(ucs.ucs4_str()), static_cast<UT_uint32>(ucs.size()));
+	      return true;	      
+	    }
+	  case EV_EEMR_INCOMPLETE:
+	    return true;
+	    
+	  default:
+	    UT_ASSERT(0);
+	    return true;
+	  }
 
 	return false;
 }
 
 // pulled in from gdk/gdkkeysyms.h
-static EV_EditBits s_Table_NVK_0xff[] =
+static const EV_EditBits s_Table_NVK_0xff[] =
 {	0, // 00
 	0, // 01
 	0, // 02
@@ -282,7 +256,7 @@ static EV_EditBits s_Table_NVK_0xff[] =
 	EV_NVK__IGNORE__,    // GDK_Print 0xFF61
 	EV_NVK__IGNORE__,    // GDK_Execute 0xFF62
 	EV_NVK_INSERT,       // GDK_Insert 0xFF63
-	0, 
+	0,
 	EV_NVK__IGNORE__,    // GDK_Undo 0xFF65
 	EV_NVK__IGNORE__,    // GDK_Redo 0xFF66
 	EV_NVK__IGNORE__,    // GDK_Menu 0xFF67
@@ -314,7 +288,7 @@ static EV_EditBits s_Table_NVK_0xff[] =
 	EV_NVK_HOME,         // GDK_KP_Begin 0xFF9D
 	EV_NVK_INSERT,       // GDK_KP_Insert 0xFF9E
 	EV_NVK_DELETE,       // GDK_KP_Delete 0xFF9F
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	0,                   // GDK_KP_Multiply 0xFFAA
 	0,                   // GDK_KP_Add 0xFFAB
 	0,                   // GDK_KP_Separator 0xFFAC
@@ -386,7 +360,7 @@ static EV_EditBits s_Table_NVK_0xff[] =
 	EV_NVK_DELETE,       // GDK_Delete 0xFFFF
 };
 
-static EV_EditBits s_Table_NVK_0xfe[] =		// ISO 9995 Function and Modifier Keys
+static const EV_EditBits s_Table_NVK_0xfe[] =		// ISO 9995 Function and Modifier Keys
 {											// see /usr/include/X11/keysymdef.h
 	0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,		// 0xfe00 - 0xfe0f
 	0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,		// 0xfe10 - 0xfe1f
@@ -422,7 +396,7 @@ static EV_EditBits s_Table_NVK_0xfe[] =		// ISO 9995 Function and Modifier Keys
 };
 
 
-static bool s_isVirtualKeyCode(gint keyval)
+static bool s_isVirtualKeyCode(guint keyval)
 {
 	// X11 has several segregated sets of keys
 	// 0xff00 - 0xffff has most function keys and non-letter keys
@@ -438,25 +412,25 @@ static bool s_isVirtualKeyCode(gint keyval)
 
 	if (keyval > 0x0000FFFF)
 		return false; //was true before CJK patch
-//
-// Causes immediate on keypress segfault??
+	//
+	// Causes immediate on keypress segfault??
 	if (keyval >= GDK_KP_Space && keyval <= GDK_KP_9 && keyval != GDK_KP_Enter) // number pad keys
 		return false;
-	
+
 	if (keyval > 0xFF00)				// see the above table
 		return true;
 	if (keyval > 0xFE00)				// see the above table
 		return true;
 
 	UT_ASSERT(keyval <= 0xFD00);		// we don't what to do with 3270 keys
-	
+
 	if (keyval == 0x0020)				// special handling for ASCII-Space
 		return true;
 
 	return false;
 }
 
-static EV_EditBits s_mapVirtualKeyCodeToNVK(gint keyval)
+static EV_EditBits s_mapVirtualKeyCodeToNVK(guint keyval)
 {
 	// map the given virtual key into a "named virtual key".
 	// these are referenced by NVK_ symbol so that the cross
@@ -468,12 +442,12 @@ static EV_EditBits s_mapVirtualKeyCodeToNVK(gint keyval)
 
 	if (keyval > 0x0000FFFF)
 		return EV_NVK__IGNORE__;
-	
+
 	if (keyval > 0xFF00)
 		return s_Table_NVK_0xff[keyval - 0xFF00];
 	if (keyval > 0xFE00)
 		return s_Table_NVK_0xfe[keyval - 0xFE00];
-	
+
 	if (keyval == 0x0020)
 		return EV_NVK_SPACE;
 
@@ -495,7 +469,7 @@ static GdkModifierType s_getAltMask(void)
 	//////////////////////////////////////////////////////////////////
 	// find out what modifier mask XL_Alt_{L,R} are bound to.
 	//////////////////////////////////////////////////////////////////
-	
+
 	int alt_mask = 0;
 
 	Display * display = GDK_DISPLAY();
@@ -508,7 +482,7 @@ static GdkModifierType s_getAltMask(void)
 	int k,m;
 	int mAltL=-1;
 	int mAltR=-1;
-	
+
 	for (m=0; m<8; m++)
 	{
 		for (k=0; k<mkpm; k++)
@@ -529,7 +503,7 @@ static GdkModifierType s_getAltMask(void)
 	case 1:								// Alt_L is mapped to (Caps)LOCK ??
 	case 2:								// Alt_L is mapped to CONTROL ??
 		break;							// ... ignore this key.
-		
+
 	case 3: alt_mask |= GDK_MOD1_MASK; break;
 	case 4: alt_mask |= GDK_MOD2_MASK; break;
 	case 5: alt_mask |= GDK_MOD3_MASK; break;
@@ -545,7 +519,7 @@ static GdkModifierType s_getAltMask(void)
 	case 1:								// Alt_R is mapped to (Caps)LOCK ??
 	case 2:								// Alt_R is mapped to CONTROL ??
 		break;							// ... ignore this key.
-		
+
 	case 3: alt_mask |= GDK_MOD1_MASK; break;
 	case 4: alt_mask |= GDK_MOD2_MASK; break;
 	case 5: alt_mask |= GDK_MOD3_MASK; break;
@@ -557,8 +531,6 @@ static GdkModifierType s_getAltMask(void)
 
 	if (!alt_mask)						// if nothing set, fall back to MOD1
 		alt_mask = GDK_MOD1_MASK;
-	
-	//UT_DEBUGMSG(("Keycodes for alt [l 0x%x][r 0x%x] using modifiers [%d %d] yields [0x%x]\n",kcAltL,kcAltR,mAltL-2,mAltR-2,alt_mask));
 
-	return (GdkModifierType)alt_mask;
+	return static_cast<GdkModifierType>(alt_mask);
 }
