@@ -27,7 +27,8 @@
 #include "ut_vector.h"
 #include "ev_Win32Toolbar.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Frame.h"
+#include "xap_Win32FrameImpl.h"
 #include "ev_Toolbar_Actions.h"
 #include "ev_Toolbar_Layouts.h"
 #include "ev_Toolbar_Labels.h"
@@ -41,7 +42,21 @@
 // The following are for the windows font colour dialog hack
 #include "ap_Dialog_Id.h"
 #include "ap_Dialog_Background.h"
-#include "fv_view.h"
+#include "fv_View.h"
+#include "pt_PieceTable.h"
+#include "ap_Win32Toolbar_FontCombo.h"
+
+#ifndef TBSTYLE_EX_DRAWDDARROWS
+#define TBSTYLE_EX_DRAWDDARROWS 0x00000001
+#endif
+
+#ifndef TB_SETEXTENDEDSTYLE
+#define TB_SETEXTENDEDSTYLE     (WM_USER + 84)  // For TBSTYLE_EX_*
+#endif
+
+#ifndef TB_GETEXTENDEDSTYLE
+#define TB_GETEXTENDEDSTYLE     (WM_USER + 85)  // For TBSTYLE_EX_*
+#endif
 
 #ifndef TBSTYLE_AUTOSIZE
 #define TBSTYLE_AUTOSIZE  0x10
@@ -95,16 +110,17 @@ foo_Bitmap_container::~foo_Bitmap_container()
 /*****************************************************************/
 
 
-EV_Win32Toolbar::EV_Win32Toolbar(XAP_Win32App * pWin32App, XAP_Win32Frame * pWin32Frame,
+EV_Win32Toolbar::EV_Win32Toolbar(XAP_Win32App * pWin32App, XAP_Frame * pFrame,
 								 const char * szToolbarLayoutName,
 								 const char * szToolbarLabelSetName)
 :	EV_Toolbar(pWin32App->getEditMethodContainer(),
 				szToolbarLayoutName,
 				szToolbarLabelSetName),
 	m_pWin32App(pWin32App),
-	m_pWin32Frame(pWin32Frame),
+	m_pWin32Frame(pFrame),
 	m_pViewListener(NULL),
 	m_lid(0),				// view listener id
+	m_pFontCtrl(NULL),
 	m_hwnd(0)
 {
 }
@@ -112,6 +128,9 @@ EV_Win32Toolbar::EV_Win32Toolbar(XAP_Win32App * pWin32App, XAP_Win32Frame * pWin
 EV_Win32Toolbar::~EV_Win32Toolbar(void)
 {
 	_releaseListener();
+
+	if (m_pFontCtrl)
+		delete m_pFontCtrl;
 }
 
 bool EV_Win32Toolbar::toolbarEvent(XAP_Toolbar_Id id,
@@ -122,8 +141,8 @@ bool EV_Win32Toolbar::toolbarEvent(XAP_Toolbar_Id id,
 	// invoke the appropriate function.
 	// return true iff handled.
 
-	// we use this for the color dialog hack
-	UT_String color;
+	// we use this for the color dialog hack	
+	UT_UCS4String ucs4color;
 
 	const EV_Toolbar_ActionSet * pToolbarActionSet = m_pWin32App->getToolbarActionSet();
 	UT_ASSERT(pToolbarActionSet);
@@ -178,16 +197,32 @@ bool EV_Win32Toolbar::toolbarEvent(XAP_Toolbar_Id id,
 			//const XML_Char * pszBackground = UT_getAttribute("background-color",propsSection);
 			//pDialog->setColor(pszBackground);
 
+			XAP_Win32FrameImpl* fimpl = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl());
+
+			if (!fimpl->m_sColorBack.empty() && pAction->getItemType() == EV_TBIT_ColorBack)
+				pDialog->setColor(fimpl->m_sColorBack.utf8_str());
+
+			if (!fimpl->m_sColorFore.empty() && pAction->getItemType() == EV_TBIT_ColorFore)
+				pDialog->setColor(fimpl->m_sColorFore.utf8_str());
+
 			pDialog->runModal (pFrame);
 
 			AP_Dialog_Background::tAnswer ans = pDialog->getAnswer();
-			bool bOK = (ans == AP_Dialog_Background::a_OK);
+			bool bOK = (ans == AP_Dialog_Background::a_OK);			
 
 			if (bOK)
 			{
-				color = reinterpret_cast<const char *>(pDialog->getColor());
-				pData = (UT_UCSChar *)(color.c_str());
-				dataLength = color.size();
+				UT_UTF8String strColor;
+				strColor = (reinterpret_cast<const char *>(pDialog->getColor()));								
+				ucs4color = strColor.ucs4_str();
+				pData = (UT_UCSChar *) ucs4color.ucs4_str();
+				dataLength =  strColor.size();
+
+				if (pAction->getItemType() == EV_TBIT_ColorBack)
+					fimpl->m_sColorBack = strColor;
+
+				if (pAction->getItemType() == EV_TBIT_ColorFore)
+					fimpl->m_sColorFore = strColor;
 			}
 
 			pDialogFactory->releaseDialog(pDialog);
@@ -220,12 +255,105 @@ bool EV_Win32Toolbar::toolbarEvent(XAP_Toolbar_Id id,
 WHICHPROC s_lpfnDefCombo; 
 WHICHPROC s_lpfnDefComboEdit; 
 
+
 #define COMBO_BUF_LEN 256
 
+
 LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
 	switch (uMessage)
 	{
+		/* Draws the font preview in the font selection combobox*/
+		case WM_DRAWITEM:
+		{						
+			EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);			
+			UINT u = GetDlgCtrlID(hWnd);
+			XAP_Toolbar_Id id = t->ItemIdFromWmCommand(u);
+						
+			if (id!=AP_TOOLBAR_ID_FMT_FONT)	/* Only owner draw the font selection*/
+			{
+				UT_ASSERT(0);
+				break;
+			}
+								
+			DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lParam;			
+			XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();			
+
+			if (!t->m_pFontCtrl)
+			{	
+				EV_Toolbar_Control * pControl = pFactory->getControl(t, AP_TOOLBAR_ID_FMT_FONT);                                                                         						
+				t->m_pFontCtrl = static_cast<AP_Win32Toolbar_FontCombo *>(pControl);
+				t->m_pFontCtrl->populate();
+			}
+
+			HFONT hFont, hUIFont;
+			LOGFONT logfont;
+			SIZE size;
+			HFONT hfontSave;
+			const UT_Vector * v = t->m_pFontCtrl->getContents();
+			const UT_Vector * vcharSet = t->m_pFontCtrl->getFontsCharset();						
+			const char * sz  = (const char *)v->getNthItem(dis->itemData);			
+			const int nCharset = (int)vcharSet->getNthItem(dis->itemData);						
+			
+			if(dis->itemState & ODS_COMBOBOXEDIT)
+			{
+				HFONT hUIFont = (HFONT) GetStockObject(DEFAULT_GUI_FONT);
+				hfontSave = (HFONT) SelectObject (dis->hDC, hUIFont);
+				ExtTextOut(dis->hDC, dis->rcItem.left, dis->rcItem.top, ETO_OPAQUE | ETO_CLIPPED, 0, sz, lstrlen(sz), 0);
+				SelectObject (dis->hDC, hfontSave);				
+			}
+			else
+			{	
+				memset (&logfont, 0, sizeof (logfont));
+
+				/* Create current font */
+				logfont.lfHeight = 18;
+				logfont.lfWidth = 0;
+				logfont.lfEscapement = 0;
+				logfont.lfOrientation = 0;
+				logfont.lfWeight = FW_BOLD;
+				logfont.lfItalic = FALSE;
+				logfont.lfUnderline = FALSE;
+				logfont.lfStrikeOut = FALSE;
+				logfont.lfCharSet = DEFAULT_CHARSET;
+				logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
+				logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
+				logfont.lfQuality = DEFAULT_QUALITY;
+				logfont.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;			  
+				strcpy (logfont.lfFaceName, sz);   
+				hFont = CreateFontIndirect (&logfont);			
+
+				if(dis->itemState & ODS_SELECTED) /* HighLight the selected text*/
+				{
+					SetTextColor(dis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
+					SetBkColor(dis->hDC, GetSysColor(COLOR_HIGHLIGHT));
+					FillRect(dis->hDC, &dis->rcItem, GetSysColorBrush(COLOR_HIGHLIGHT));
+				}
+				else
+				{
+					SetTextColor(dis->hDC, GetSysColor(COLOR_WINDOWTEXT));
+					SetBkColor(dis->hDC, GetSysColor(COLOR_WINDOW));
+					FillRect(dis->hDC, &dis->rcItem, GetSysColorBrush(COLOR_WINDOW) );				
+				}
+
+				hUIFont = (HFONT) GetStockObject(DEFAULT_GUI_FONT);
+
+				/*Fontname in regular font*/
+				hfontSave = (HFONT) SelectObject (dis->hDC, hUIFont);
+				ExtTextOut(dis->hDC, dis->rcItem.left, dis->rcItem.top, ETO_OPAQUE | ETO_CLIPPED, 0, sz, lstrlen(sz), 0);
+				
+				/*Font example after the name*/
+				const char* szSample="AbCdEfGhIj";
+				GetTextExtentPoint32(dis->hDC, sz, lstrlen(sz), &size);
+				SelectObject (dis->hDC, hFont);
+				ExtTextOut(dis->hDC, dis->rcItem.left+size.cx+5, dis->rcItem.top, ETO_OPAQUE | ETO_CLIPPED, 0, szSample, lstrlen(szSample), 0);					
+				SelectObject (dis->hDC, hfontSave);									
+				DeleteObject(hFont); 
+			}
+
+			return TRUE;
+		}
+		
 		case WM_MOUSEMOVE:
 		case WM_LBUTTONDOWN:
 		case WM_LBUTTONUP:
@@ -250,7 +378,7 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 		case WM_COMMAND:
 		{
 			switch (HIWORD(wParam))
-			{
+			{								
 				case CBN_SELCHANGE:
 				{
 					// are we currently dropped down?
@@ -260,23 +388,66 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 					// yep, we're done 
 					SendMessage(hWnd, CB_SHOWDROPDOWN, (WPARAM) FALSE, 0);
 
-					UT_sint32 iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);
-					
+					UT_sint32 iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);		
+									
 					if(iSelected != -1)
-						{
-						// now that we know dropdown is gone, this should be ok
-						PostMessage(hWnd, WM_KEYDOWN, VK_RETURN, 0);
-						}
-					else
-						{
-						PostMessage(hWnd, WM_KEYDOWN, VK_ESCAPE, 0);
-						}
-					break;
-				}
-			}
-			break;
-		}
+					{							
+						EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);
+						UT_ASSERT(t);
 
+						
+						UINT u = GetDlgCtrlID(hWnd);
+						XAP_Toolbar_Id id = t->ItemIdFromWmCommand(u);
+
+						static UT_UCSChar ucs_buf[COMBO_BUF_LEN];
+						char buf[COMBO_BUF_LEN];
+
+						UT_uint32 dataLength = GetWindowText(hWnd, buf, COMBO_BUF_LEN);
+
+						UT_UCS4_strcpy_char(ucs_buf, buf);
+						UT_UCSChar * pData = (UT_UCSChar *) ucs_buf;	// HACK: should be void *
+						
+						if(dataLength)
+						{	
+							// If is a STYLE_NAME we should pass the internal one, not the localised				
+							if (id==AP_TOOLBAR_ID_FMT_STYLE)
+							{									
+								UT_sint32 iSelected;					
+								int nData;
+								
+								iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);										
+
+								// Find the proper non-localised text                                                                             					
+								XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();
+								EV_Toolbar_Control * pControl = pFactory->getControl(t, AP_TOOLBAR_ID_FMT_STYLE);                                                                         
+								const UT_Vector * v = pControl->getContents();                                                                          
+								AP_Win32Toolbar_StyleCombo * pStyleC = static_cast<AP_Win32Toolbar_StyleCombo *>(pControl);
+								pStyleC->repopulate();                                                                                                
+
+								nData  = SendMessage(hWnd, CB_GETITEMDATA, iSelected, 0);                                                         					
+								strcpy (buf, (char *)v->getNthItem(nData));				
+								DELETEP(pControl);
+							}
+							
+							t->toolbarEvent(id, pData, dataLength);
+						}
+
+						SetFocus(static_cast<XAP_Win32FrameImpl*>(t->m_pWin32Frame->getFrameImpl())->getTopLevelWindow());
+					}	
+					else
+					{
+						PostMessage(hWnd, WM_KEYDOWN, VK_ESCAPE, 0);
+					}
+						
+					break;
+				}					
+				
+							
+				break;
+				
+			} // swich
+		} // case command
+			
 		case WM_KEYDOWN:
 		{
 			EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);
@@ -297,8 +468,7 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 				case VK_TAB:
 				case VK_RETURN:
 				{
-					HWND hwndFrame = t->m_pWin32Frame->getTopLevelWindow();
-					SetFocus(hwndFrame);
+					SetFocus(static_cast<XAP_Win32FrameImpl*>(t->m_pWin32Frame->getFrameImpl())->getTopLevelWindow());
 					return 0;
 				}
 			}
@@ -307,82 +477,6 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 	return (CallWindowProc(s_lpfnDefCombo, hWnd, uMessage, wParam, lParam));
 }
 
-LRESULT CALLBACK EV_Win32Toolbar::_ComboEditWndProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
-{
-	switch (uMessage)
-	{
-		case WM_MOUSEMOVE:
-		case WM_LBUTTONDOWN:
-		case WM_LBUTTONUP:
-		{
-			// forward these to parent (ie, the combo)
-			// TODO: figure out why tooltips still don't work over the edit
-			// HYP:  may need to tweak coord system for wParam/lParam
-			HWND hwndParent = GetParent(hWnd);
-			SendMessage(hwndParent, uMessage, wParam, lParam);
-			break;
-		}
-
-
-		case WM_KILLFOCUS:
-		{
-			// for now, we fire an event any time we lose focus
-			// TODO: confirm that this gives the desired behavior
-			EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);
-			UT_ASSERT(t);
-
-			HWND hwndParent = GetParent(hWnd);
-			UINT u = GetDlgCtrlID(hwndParent);
-			XAP_Toolbar_Id id = t->ItemIdFromWmCommand(u);
-
-			static char buf[COMBO_BUF_LEN];
-
-			UT_UCSChar * pData = (UT_UCSChar *) buf;	// HACK: should be void *
-			UT_uint32 dataLength = GetWindowText(hWnd, buf, COMBO_BUF_LEN);
-			if(dataLength)
-				{
-				t->toolbarEvent(id, pData, dataLength);
-				}
-			else
-				{
-				SendMessage(hWnd, WM_KEYDOWN, VK_ESCAPE, 0);
-				}
-			break;
-		}
-
-		case WM_KEYDOWN:
-		{
-			switch (wParam)
-			{
-				case VK_ESCAPE:
-				case VK_RETURN:
-				case VK_TAB:
-				{
-					// forward these to parent (ie, the combo)
-					HWND hwndParent = GetParent(hWnd);
-					SendMessage(hwndParent, uMessage, wParam, lParam);
-					return 0;
-				}
-			}
-			break;
-		}
-
-		case WM_KEYUP:
-		case WM_CHAR:
-		{
-			switch (wParam) 
-			{ 
-                case VK_TAB:
-				case VK_ESCAPE: 
-                case VK_RETURN:
-					// swallow these
-					return 0;
-			}
-			break;
-		}
-	}
-	return (CallWindowProc(s_lpfnDefComboEdit, hWnd, uMessage, wParam, lParam));
-}
 
 /*****************************************************************/
 
@@ -425,7 +519,7 @@ bool EV_Win32Toolbar::synthesize(void)
 	XAP_Toolbar_ControlFactory * pFactory = m_pWin32App->getControlFactory();
 	UT_ASSERT(pFactory);
 
-	HWND hwndParent = m_pWin32Frame->getToolbarWindow();
+	HWND hwndParent = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl())->getToolbarWindow();
 	if(hwndParent == NULL)
 		return false;
 
@@ -453,6 +547,10 @@ bool EV_Win32Toolbar::synthesize(void)
 
 	UT_ASSERT(m_hwnd);
 
+	// override the window procedure 	
+	SetWindowLong(m_hwnd, GWL_USERDATA, (LONG)this);
+
+
 	SendMessage(m_hwnd, TB_BUTTONSTRUCTSIZE, (WPARAM) sizeof(TBBUTTON), 0);  
 
 	// the Windows Common Control Toolbar requires that we set
@@ -556,13 +654,17 @@ bool EV_Win32Toolbar::synthesize(void)
 
 						// create a matching child control
 						DWORD dwStyle = WS_CHILD | WS_BORDER | WS_VISIBLE | WS_VSCROLL |
-								CBS_HASSTRINGS | CBS_DROPDOWN;
+								CBS_HASSTRINGS | CBS_DROPDOWNLIST;
 
 						if ((pControl) && (pControl->shouldSort()))
 						{
 							dwStyle |= CBS_SORT;
 						}
-
+						
+						if (id==AP_TOOLBAR_ID_FMT_FONT)
+							dwStyle |= CBS_OWNERDRAWFIXED;
+							
+							
 						HWND hwndCombo = CreateWindowEx ( 0L,   // No extended styles.
 							"COMBOBOX",						// Class name.
 							"",								// Default text.
@@ -576,6 +678,7 @@ bool EV_Win32Toolbar::synthesize(void)
 						UT_ASSERT(hwndCombo);
 						
 						SendMessage(hwndCombo, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));
+						SendMessage(hwndCombo, CB_SETDROPPEDWIDTH,(WPARAM)pControl->getDroppedWidth(), 0);
 
 						// populate it
 						if (pControl)
@@ -585,15 +688,21 @@ bool EV_Win32Toolbar::synthesize(void)
 							const UT_Vector * v = pControl->getContents();
 							UT_ASSERT(v);
 
+							SendMessage(hwndCombo, WM_SETREDRAW, FALSE,0);
+
 							if (v)
 							{
 								UT_uint32 items = v->getItemCount();
+								int nIndex;
 								for (UT_uint32 k=0; k < items; k++)
 								{
 									char * sz = (char *)v->getNthItem(k);
-									SendMessage(hwndCombo, CB_ADDSTRING,(WPARAM)0, (LPARAM)sz);
+									nIndex = SendMessage(hwndCombo, CB_ADDSTRING,(WPARAM)0, (LPARAM)sz);
+									SendMessage(hwndCombo,CB_SETITEMDATA, nIndex, k);
 								}
 							}
+
+							SendMessage(hwndCombo, WM_SETREDRAW, TRUE,0);
 						}
 
 						// override the window procedure for the combo box
@@ -601,17 +710,6 @@ bool EV_Win32Toolbar::synthesize(void)
 						SetWindowLong(hwndCombo, GWL_WNDPROC, (LONG)_ComboWndProc);
 						SetWindowLong(hwndCombo, GWL_USERDATA, (LONG)this);
 
-						// override the window procedure for its edit control, too
-						POINT pt;
-						pt.x = 4;
-						pt.y = 4; 
-						HWND hwndComboEdit = ChildWindowFromPoint(hwndCombo, pt); 
-						UT_ASSERT(hwndComboEdit);
-						UT_ASSERT(hwndComboEdit != hwndCombo);
-						s_lpfnDefComboEdit = (WHICHPROC)GetWindowLong(hwndComboEdit, GWL_WNDPROC);
-						SetWindowLong(hwndComboEdit, GWL_WNDPROC, (LONG)_ComboEditWndProc);
-						SetWindowLong(hwndComboEdit, GWL_USERDATA, (LONG)this);
-
 						// Get the handle to the tooltip window.
 						HWND hwndTT = (HWND)SendMessage(m_hwnd, TB_GETTOOLTIPS, 0, 0);
 
@@ -690,7 +788,8 @@ bool EV_Win32Toolbar::synthesize(void)
 
 						TBADDBITMAP ab;
 						ab.hInst = 0;
-						ab.nID = (LPARAM)hBitmap;
+						ab.nID = (LPARAM)hBitmap;						
+						
 						LRESULT iAddedAt = SendMessage(m_hwnd,TB_ADDBITMAP,1,(LPARAM)&ab);
 						UT_ASSERT(iAddedAt != -1);
 
@@ -705,6 +804,17 @@ bool EV_Win32Toolbar::synthesize(void)
 						tbb.iString = SendMessage(m_hwnd, TB_ADDSTRING, (WPARAM) 0, (LPARAM) (LPSTR) szLabel);
 					}
 					
+					if (pAction->getItemType() == EV_TBIT_ColorFore || pAction->getItemType() == EV_TBIT_ColorBack)
+					{
+						tbb.fsStyle |=TBSTYLE_DROPDOWN;
+
+						//tbb.fsStyle |=BTNS_WHOLEDROPDOWN;
+
+						DWORD dwStyle = SendMessage(m_hwnd, TB_GETEXTENDEDSTYLE, (WPARAM) 0, (LPARAM) 0);						
+						dwStyle |= TBSTYLE_EX_DRAWDDARROWS;																		
+						SendMessage(m_hwnd, TB_SETEXTENDEDSTYLE, (WPARAM) 0, (LPARAM) dwStyle);						
+					}
+					
 				}
 			}
 			break;
@@ -938,14 +1048,64 @@ bool EV_Win32Toolbar::_refreshItem(AV_View * pView, const EV_Toolbar_Action * pA
 				bool bGrayed = EV_TIS_ShouldBeGray(tis);
 				bool bString = EV_TIS_ShouldUseString(tis);
 
-				HWND hwndCombo = _getControlWindow(id);
-				UT_ASSERT(hwndCombo);
+				UT_return_val_if_fail(szState, true);
 
+				HWND hwndCombo = _getControlWindow(id);
+				UT_return_val_if_fail(hwndCombo, true);
+
+				
+				/*
+				{
 				// NOTE: we always update the control even if !bString
 				int idx = SendMessage(hwndCombo, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)szState);
 				if (idx==CB_ERR)
 					SetWindowText(hwndCombo, szState);
+				}	
+				break;
+				*/
+				
+				// are we currently dropped down?
+			
+				// Find the proper non-localised text
+		
+				EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hwndCombo, GWL_USERDATA);
+				XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();			
 
+				EV_Toolbar_Control * pControl = pFactory->getControl(t, AP_TOOLBAR_ID_FMT_STYLE);			
+				const UT_Vector * v = pControl->getContents();				
+					
+				AP_Win32Toolbar_StyleCombo * pStyleC = static_cast<AP_Win32Toolbar_StyleCombo *>(pControl);
+				pStyleC->repopulate();	
+				
+				//
+				// Is this a valid text?
+				//
+				UT_uint32 items = v->getItemCount();
+				bool bFound = false;
+				UT_uint32 k=0;
+				
+				for (k=0; k < items; k++)
+				{
+					if (strcmp((char *)v->getNthItem(k), szState)==0)
+					{
+						bFound = true;
+						break;
+					}	
+				}				
+				
+				char* pLocalised = (char *)szState;
+				
+				if (bFound)
+				{
+				 	pLocalised = (char *)v->getNthItem(k);					
+					pLocalised = (char *) pt_PieceTable::s_getLocalisedStyleName(pLocalised);
+				}								
+												
+				int idx = SendMessage(hwndCombo, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)pLocalised);
+				if (idx==CB_ERR)
+					SetWindowText(hwndCombo, pLocalised);							
+	
+				DELETEP(pControl);	
 				//UT_DEBUGMSG(("refreshToolbar: ComboBox [%s] is %s and %s\n",
 				//			 m_pToolbarLabelSet->getLabel(id)->getToolbarLabel(),
 				//			 ((bGrayed) ? "disabled" : "enabled"),
@@ -963,7 +1123,7 @@ bool EV_Win32Toolbar::_refreshItem(AV_View * pView, const EV_Toolbar_Action * pA
 
 		case EV_TBIT_BOGUS:
 		default:
-			UT_ASSERT(0);
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 			break;
 	}
 
@@ -1009,10 +1169,10 @@ bool EV_Win32Toolbar::getToolTip(LPARAM lParam)
 }
 
 
-void EV_Win32Toolbar::show()
+void EV_Win32Toolbar::show(void)
 {
 	UT_ASSERT(m_pWin32Frame);
-	HWND hRebar = m_pWin32Frame->getToolbarWindow();
+	HWND hRebar = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl())->getToolbarWindow();
 	UT_ASSERT(hRebar);
 	const int iBand = _getBandForHwnd(m_hwnd);
 	UT_ASSERT(iBand < 0);	// It can't already be displayed!
@@ -1021,10 +1181,10 @@ void EV_Win32Toolbar::show()
 	m_bVisible = true;
 }
 
-void EV_Win32Toolbar::hide()
+void EV_Win32Toolbar::hide(void)
 {
 	UT_ASSERT(m_pWin32Frame);
-	HWND hRebar = m_pWin32Frame->getToolbarWindow();
+	HWND hRebar = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl())->getToolbarWindow();
 	UT_ASSERT(hRebar);
 	const int iBand = _getBandForHwnd(m_hwnd);
 	if (iBand >= 0)
@@ -1041,7 +1201,7 @@ void EV_Win32Toolbar::hide()
 //
 int EV_Win32Toolbar::_getBandForHwnd(HWND hToolbar) const
 {
-	HWND hRebar = m_pWin32Frame->getToolbarWindow();
+	HWND hRebar = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl())->getToolbarWindow();
 
 	// If we ever get more than 20 toolbars I don't wanna be around.
 	REBARBANDINFO rbi = { 0 };
@@ -1091,13 +1251,13 @@ void EV_Win32Toolbar::_addToRebar()
 	rbbi.cyMinChild = HIWORD(dwBtnSize);
 	rbbi.cx = iWidth;
 
-	HWND hRebar = m_pWin32Frame->getToolbarWindow();
+	HWND hRebar = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl())->getToolbarWindow();
 	// Add it at the the end
 	SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbbi);
 }
 
 
-XAP_Win32Frame * EV_Win32Toolbar::getFrame(void)
+XAP_Frame * EV_Win32Toolbar::getFrame(void)
 {
 	return m_pWin32Frame;
 }
@@ -1127,6 +1287,9 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 	}
 	if(i>=count)
 		return false;
+	
+
+
 //
 // GOT IT!
 //
@@ -1136,7 +1299,8 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 	EV_Toolbar_Control * pControl = pFactory->getControl(this, id);
 	AP_Win32Toolbar_StyleCombo * pStyleC = static_cast<AP_Win32Toolbar_StyleCombo *>(pControl);
 	pStyleC->repopulate();
-
+	
+	
 	HWND hwndCombo = _getControlWindow(id);
 	UT_ASSERT(hwndCombo);
 	// GtkCombo * item = GTK_COMBO(wd->m_widget);
@@ -1145,6 +1309,7 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 //						
 	const UT_Vector * v = pControl->getContents();
 	UT_ASSERT(v);
+	
 //
 // Now  we must remove and delete the old data so we add the new
 // list of styles to the combo box.
@@ -1162,10 +1327,20 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 // Now make a new one.
 //
 	UT_uint32 items = v->getItemCount();
+	int	nItem;													    
+	
 	for (UT_uint32 k=0; k < items; k++)
 	{
-		char * sz = (char *)v->getNthItem(k);
-		SendMessage(hwndCombo, CB_ADDSTRING,(WPARAM)0, (LPARAM)sz);
+		char*	sz = (char *)v->getNthItem(k);
+		char*	pLocalised = sz;
+
+		pLocalised = (char *) pt_PieceTable::s_getLocalisedStyleName(sz);
+		
+		if (pLocalised!=sz)
+			int n=1;			
+
+		nItem = SendMessage(hwndCombo, CB_ADDSTRING,(WPARAM)0, (LPARAM)pLocalised);
+		SendMessage(hwndCombo, CB_SETITEMDATA,(WPARAM)nItem, (LPARAM)k);
 	}
 //
 // Don't need this anymore and we don't like memory leaks in abi
@@ -1177,3 +1352,47 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 	return true;
 }
 
+/*
+	This method is called when the user clicks on the arrow next to the
+	foreground and background colour seletion buttons on the toolbar
+*/
+void	EV_Win32Toolbar::onDropArrow(UINT cmd)
+{
+	UT_UCS4String ucs4color;
+	UT_uint32 dataLength;
+	XAP_Win32FrameImpl* fimpl = static_cast<XAP_Win32FrameImpl*>(m_pWin32Frame->getFrameImpl());
+	int id = ItemIdFromWmCommand(cmd);
+
+	UT_ASSERT(id== AP_TOOLBAR_ID_COLOR_BACK || id== AP_TOOLBAR_ID_COLOR_FORE);
+	
+	const EV_Toolbar_ActionSet * pToolbarActionSet = m_pWin32App->getToolbarActionSet();
+	UT_ASSERT(pToolbarActionSet);
+
+	const EV_Toolbar_Action * pAction = pToolbarActionSet->getAction(id);
+
+	if (!pAction) 
+		return;
+
+	AV_View * pView = m_pWin32Frame->getCurrentView();
+	
+	const char * szMethodName = pAction->getMethodName();
+	if (!szMethodName)
+		return;
+	
+	const EV_EditMethodContainer * pEMC = m_pWin32App->getEditMethodContainer();
+	EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
+
+	if (id==AP_TOOLBAR_ID_COLOR_BACK)
+	{
+		ucs4color = fimpl->m_sColorBack.ucs4_str();
+		dataLength =  fimpl->m_sColorBack.size();
+	}
+	else
+	{
+		ucs4color = fimpl->m_sColorFore.ucs4_str();
+		dataLength =  fimpl->m_sColorFore.size();
+	}	
+
+	invokeToolbarMethod(pView,pEM, ucs4color.ucs4_str(),dataLength);
+}
+
