@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* Abiword
  * Copyright (C) 1998 AbiSource, Inc.
  * 
@@ -26,42 +28,64 @@
 #include "xap_UnixFont.h"
 #include "xap_Frame.h"
 #include "gr_Graphics.h"
+#include "ut_vector.h"
+
+#include <X11/Xft/Xft.h>
 
 class UT_ByteBuf;
+class UT_String;
+class UT_Wctomb;
+
+UT_uint32 adobeToUnicode(UT_uint32 iAdobe);
+
+UT_uint32 adobeDingbatsToUnicode(UT_uint32 iAdobe);
 
 class GR_UnixGraphics : public GR_Graphics
 {
  public:
-	GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App *app);
-	~GR_UnixGraphics();
+#ifndef WITH_PANGO
+ 	GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App *app);
+#else
+ 	GR_UnixGraphics(GdkWindow * win, XAP_App *app);
+#endif
+	virtual ~GR_UnixGraphics();
 
-    // HACK: I need more speed
-	virtual void        drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff);
+#ifndef WITH_PANGO 
+	virtual void        drawGlyph(UT_uint32 glyph_idx, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void		drawChars(const UT_UCSChar* pChars, int iCharOffset,
-								  int iLength, UT_sint32 xoff, UT_sint32 yoff);
+								  int iLength, UT_sint32 xoff, UT_sint32 yoff,
+								  int * pCharWidths = NULL);
 	virtual void		setFont(GR_Font* pFont);
 	virtual UT_uint32	getFontHeight();
 	// virtual UT_uint32	measureString(const UT_UCSChar*s, int iOffset, int num, unsigned short* pWidths);
 	virtual UT_uint32 measureUnRemappedChar(const UT_UCSChar c);
+#endif	
 	virtual void		setColor(const UT_RGBColor& clr);
+	virtual void        getColor(UT_RGBColor &clr);
 
 	virtual GR_Font*	getGUIFont();
+
+#ifndef WITH_PANGO 	
 	virtual GR_Font*	findFont(const char* pszFontFamily, 
 								 const char* pszFontStyle, 
 								 const char* pszFontVariant, 
 								 const char* pszFontWeight, 
 								 const char* pszFontStretch, 
 								 const char* pszFontSize);
+	virtual GR_Font*	getDefaultFont(UT_String& fontFamily);
+
 	virtual UT_uint32	getFontAscent();
 	virtual UT_uint32	getFontDescent();
+
+	virtual void		getCoverage(UT_Vector& coverage);
+#endif	
 	virtual void		drawLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void		setLineWidth(UT_sint32);
 	virtual void		xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void		polyLine(UT_Point * pts, UT_uint32 nPoints);
-	virtual void		fillRect(UT_RGBColor& c,
+	virtual void		fillRect(const UT_RGBColor& c,
 								 UT_sint32 x, UT_sint32 y,
 								 UT_sint32 w, UT_sint32 h);
-	virtual void		fillRect(UT_RGBColor& c, UT_Rect &r);
 	virtual void		invertRect(const UT_Rect* pRect);
 	virtual void		setClipRect(const UT_Rect* pRect);
 	virtual void		scroll(UT_sint32, UT_sint32);
@@ -79,6 +103,8 @@ class GR_UnixGraphics : public GR_Graphics
 	virtual bool		startPrint(void);
 	virtual bool		startPage(const char * szPageLabel, UT_uint32 pageNumber,
 								  bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight);
+
+	virtual void      setZoomPercentage(UT_uint32 iZoom);
 	virtual bool		endPrint(void);
 
 	virtual void		flush(void);
@@ -97,31 +123,51 @@ class GR_UnixGraphics : public GR_Graphics
 	virtual void		fillRect(GR_Color3D c, UT_Rect &r);
 	
 	virtual void		polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints);
-	
+
+	void                createPixmapFromXPM( char ** pXPM,GdkPixmap *source,
+											 GdkBitmap * mask);
+
+	virtual void setLineProperties ( double inWidth, 
+					 GR_Graphics::JoinStyle inJoinStyle = JOIN_MITER,
+					 GR_Graphics::CapStyle inCapStyle   = CAP_BUTT,
+					 GR_Graphics::LineStyle inLineStyle = LINE_SOLID ) ;
+
+#ifndef WITH_PANGO 	
 	/* GR_Font versions of the above -- TODO: should I add drawChar* methods too? */
 	virtual UT_uint32 getFontAscent(GR_Font *);
 	virtual UT_uint32 getFontDescent(GR_Font *);
 	virtual UT_uint32 getFontHeight(GR_Font *);
+#else
+ private:
+	virtual void _drawFT2Bitmap(UT_sint32 x, UT_sint32 y, FT_Bitmap * pBitmap) const;
+#endif
+	virtual void	  saveRectangle(UT_Rect & r, UT_uint32 iIndx);
+	virtual void	  restoreRectangle(UT_uint32 iIndx);
+	
+	static UT_uint32  s_getDeviceResolution(void);
+	virtual UT_uint32 	getDeviceResolution(void) const;
 
  protected:
-	virtual UT_uint32 	_getResolution(void) const;
 	void				_setColor(GdkColor & c);
-
+#ifndef WITH_PANGO
 	XAP_UnixFontManager * 	m_pFontManager;
+#endif	
+
 	GdkGC*       			m_pGC;
 	GdkGC*  	      		m_pXORGC;
 	GdkWindow*  	  		m_pWin;
 
+#ifndef WITH_PANGO	
 	// our currently requested font by handle
 	XAP_UnixFontHandle *	m_pFont;
-
+#endif
 	// Current GDK fonts corresponding to this. Calling m_pFont->explodeGdkFont
 	// causes gdk_font_load to be called and memory to be allocated. This should
 	// not happen on every draw
 	GdkFont * m_pSingleByteFont, * m_pMultiByteFont;
 
 	// our "OEM" system font, like a 10 point Helvetica for GUI items
-	static XAP_UnixFontHandle *	s_pFontGUI;
+	XAP_UnixFontHandle *	m_pFontGUI;
 	static UT_uint32		s_iInstanceCount;
   
 	GdkColormap* 	 		m_pColormap;
@@ -133,7 +179,29 @@ class GR_UnixGraphics : public GR_Graphics
 	
 	GdkColor				m_3dColors[COUNT_3D_COLORS];
 private:
+#ifndef WITH_PANGO 	
 	XAP_UnixFontHandle *	m_pFallBackFontHandle;
+#endif
+
+	XftDraw*				m_pXftDraw;
+	XftColor				m_XftColor;
+	XftFont*				m_pXftFontL;
+	XftFont*				m_pXftFontD;
+	
+	Drawable				m_Drawable;
+	Visual*					m_pVisual;
+	Colormap				m_Colormap;
+
+	FcChar32				m_aMap[FC_CHARSET_MAP_SIZE];
+	UT_sint32               m_iXoff;
+	UT_sint32               m_iYoff;
+	UT_RGBColor				m_curColor;
+
+	UT_Vector				m_vSaveRect;
+	UT_Vector 				m_vSaveRectBuf;
+
+	bool                    m_bIsSymbol;       
+	bool                    m_bIsDingbat;       
 };
 
 #endif /* GR_UNIXGRAPHICS_H */
