@@ -22,6 +22,9 @@
 
 #include "gr_CharWidths.h"
 
+class UT_NumberVector;
+class GR_Graphics;
+
 //////////////////////////////////////////////////////////////////
 // the only reason that we subclass is because Win32 provides a
 // routine to fetch the widths of a whole font in one call.
@@ -29,7 +32,13 @@
 class GR_Win32CharWidths : public GR_CharWidths
 {
 public:
-	void			setCharWidthsOfRange(HDC hdc, UT_UCSChar c0, UT_UCSChar c1);
+	void  setCharWidthsOfRange(HDC hdc, UT_UCSChar c0, UT_UCSChar c1, GR_Graphics * pGr);
+
+private:
+	bool _doesGlyphExist(UT_UCS4Char g);
+	void _retrieveFontInfo(HDC hdc);
+	
+	UT_NumberVector m_vRanges;
 };
 
 #endif /* GR_WIN32CHARWIDTHS_H */
