@@ -21,10 +21,10 @@
 
 #include <string.h>
 #include "ut_hash.h"
-
+#include "ut_string_class.h"
+#include "ut_vector.h"
 #include "ut_debugmsg.h"
 
-
 // fwd. decls.
 static  UT_uint32 _Recommended_hash_size(UT_uint32	size);
 
@@ -133,7 +133,8 @@ UT_StringPtrMap::UT_StringPtrMap(size_t expected_cardinality)
 	n_deleted(0),
 	m_nSlots(_Recommended_hash_size(expected_cardinality)),
 	reorg_threshold(compute_reorg_threshold(m_nSlots)),
-	flags(0)
+	flags(0),
+	m_list(0)
 {
 	m_pMapping = new hash_slot[m_nSlots];
 }
@@ -142,8 +143,40 @@ UT_StringPtrMap::UT_StringPtrMap(size_t expected_cardinality)
 UT_StringPtrMap::~UT_StringPtrMap()
 {
 	DELETEPV(m_pMapping);
+	FREEP(m_list);
 }
 
+/* IMPORTANT: for use only with <char*> -> <char*> maps
+ */
+const XML_Char ** UT_StringPtrMap::list()
+{
+	if (!m_list)
+	{
+		m_list = reinterpret_cast<XML_Char **>(malloc (2 * (n_keys + 1) * sizeof (XML_Char *)));
+		if (m_list == 0)
+			return 0;
+
+		UT_uint32 index;
+
+		UT_Cursor c(this);
+
+		for (const char * value = reinterpret_cast<const char *>(c.first ());
+		     c.is_valid ();
+		     value = reinterpret_cast<const char *>(c.next ()))
+		{
+			const char * key = c.key().c_str ();
+
+			if (!key || !value)
+				continue;
+
+			m_list[index++] = static_cast<XML_Char *>(const_cast<char *>(key));
+			m_list[index++] = static_cast<XML_Char *>(const_cast<char *>(value));
+		}
+		m_list[index++] = NULL;
+		m_list[index  ] = NULL;
+	}
+	return const_cast<const XML_Char **>(m_list);
+}
 
 /*!
  * Find the value associated with the key \k
@@ -185,17 +218,10 @@ bool UT_StringPtrMap::contains(const UT_String& k, const void* v) const
 	size_t slot    = 0;
 	size_t hashval = 0;
 
-#if 1
 	// DOM: TODO: make this call work
 	sl = find_slot (k, SM_LOOKUP, slot, key_found,
 			hashval, v, &v_found, 0, 0);
 	return v_found;
-#else
-	sl = find_slot(k, SM_LOOKUP, slot, key_found, hashval, 0, 0, 0, 0);
-
-	// this is obviously not correct
-	return key_found ? true : false;
-#endif
 }
 
 
@@ -210,6 +236,8 @@ bool UT_StringPtrMap::insert(const char* key, const void* value)
 
 bool UT_StringPtrMap::insert(const UT_String& key, const void* value)
 {
+	FREEP(m_list);
+
 	size_t		slot = 0;
 	bool		key_found = false;
 	size_t		hashval = 0;
@@ -235,15 +263,6 @@ bool UT_StringPtrMap::insert(const UT_String& key, const void* value)
 		}
 	}
 
-#if 0
-	UT_DEBUGMSG(("DOM: inserted (%s, %p)\n", key.c_str(), value));
-
-	const void * v = pick (key);
-	bool b = contains (key, 0);
-
-	UT_DEBUGMSG(("DOM: %d (%s, %p, %d)\n", b, key.c_str(), v, v==value));
-
-#endif
 	return true;
 }
 
@@ -259,6 +278,8 @@ void UT_StringPtrMap::set(const char* key, const void* value)
 
 void UT_StringPtrMap::set(const UT_String& key, const void* value)
 {
+	FREEP(m_list);
+
 	size_t		slot = 0;
 	bool		key_found = false;
 	size_t		hashval = 0;
@@ -268,9 +289,7 @@ void UT_StringPtrMap::set(const UT_String& key, const void* value)
 	
 	if (!sl || !key_found) // TODO: should we insert or just return?
 	{
-#if 1
 		insert(key, value);
-#endif
 		return;
 	}
 	
@@ -281,7 +300,7 @@ void UT_StringPtrMap::set(const UT_String& key, const void* value)
  * Return a UT_Vector of elements in the HashTable that you must
  * Later free with a call to delete
  */
-UT_Vector * UT_StringPtrMap::enumerate (void) const
+UT_Vector * UT_StringPtrMap::enumerate (bool strip_null_values) const
 {
 	UT_Vector * pVec = new UT_Vector (size());
 
@@ -293,12 +312,10 @@ UT_Vector * UT_StringPtrMap::enumerate (void) const
 	{
 		// we don't allow nulls since so much of our code depends on this
 		// behavior
-#if 1
-		if (val)
+		if (!strip_null_values || val)
 		{
-			pVec->addItem ((void*)val);
+			pVec->addItem (static_cast<const void*>(val));
 		}
-#endif
 	}
 
 	return pVec;
@@ -308,7 +325,7 @@ UT_Vector * UT_StringPtrMap::enumerate (void) const
  * Return a UT_Vector of pointers to our UT_String keys in the Hashtable
  * You must FREEP the UT_Vector* but not the keys
  */
-UT_Vector * UT_StringPtrMap::keys (void) const
+UT_Vector * UT_StringPtrMap::keys (bool strip_null_values) const
 {
 	UT_Vector * pVec = new UT_Vector (size());
 
@@ -318,7 +335,12 @@ UT_Vector * UT_StringPtrMap::keys (void) const
 
 	for (val = cursor.first(); cursor.is_valid(); val = cursor.next ())
 	{
-	  pVec->addItem ((void*)&cursor.key());
+		// we don't allow nulls since so much of our code depends on this
+		// behavior
+		if (!strip_null_values || val)
+		{
+			pVec->addItem (static_cast<const void*>(&cursor.key()));
+		}
 	}
 
 	return pVec;
@@ -336,6 +358,8 @@ void UT_StringPtrMap::remove(const char* key, const void*)
 
 void UT_StringPtrMap::remove(const UT_String& key, const void*)
 {
+	FREEP(m_list);
+
 	size_t slot = 0, hashval;
 	bool bFound = false;
 	hash_slot* sl = find_slot(key, SM_LOOKUP, slot, bFound,
@@ -358,6 +382,8 @@ void UT_StringPtrMap::remove(const UT_String& key, const void*)
  */
 void UT_StringPtrMap::clear()
 {
+	FREEP(m_list);
+
 	hash_slot* slots = m_pMapping;
 	for (size_t x=0; x < m_nSlots; x++)
 	{
@@ -422,8 +448,12 @@ UT_StringPtrMap::find_slot(const UT_String& k,
 							void*			vi,
 							size_t			hashval_in) const
 {
-	hashval = (hashval_in ? hashval_in : key_wrapper::compute_hash(k));
+	if ( m_nSlots == 0 )
+	  {
+	    key_found = false ; return NULL ;
+	  }
 
+	hashval = (hashval_in ? hashval_in : key_wrapper::compute_hash(k));
 	int nSlot = hashval % m_nSlots;
 
 	xxx_UT_DEBUGMSG(("DOM: hashval for \"%s\" is %d (#%dth slot)\n", k.c_str(), hashval, nSlot));
@@ -590,7 +620,7 @@ const UT_String& UT_StringPtrMap::_key(UT_Cursor& c) const
 	}
 	// should never happen
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-        static UT_String bad;
+	static UT_String bad;
 	return bad;
 }
 
@@ -619,7 +649,7 @@ const void* UT_StringPtrMap::_next(UT_Cursor& c) const
 const void* UT_StringPtrMap::_prev(UT_Cursor& c) const
 {
 	const hash_slot* map = m_pMapping;
-	int x;
+	size_t x;
 	for (x = c._get_index() - 1; x >= 0; --x)
 	{
 		if (!map[x].empty() && !map[x].deleted())
