@@ -30,6 +30,11 @@
 #include "ut_assert.h"
 #include "ut_string.h"
 #include "ut_debugmsg.h"
+
+//#include "fl_AutoLists.h" // need definition of AUTO_LIST_RESERVED
+#define AUTO_LIST_RESERVED 1000
+#include <limits.h>
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -297,8 +302,8 @@ extern "C" {
 
 static int color_compare (const void * a, const void * b)
 {
-  const char * name = (const char *) a;
-  const colorToRGBMapping * id = (const colorToRGBMapping *) b;
+  const char * name = static_cast<const char *>(a);
+  const colorToRGBMapping * id = static_cast<const colorToRGBMapping *>(b);
 		
   return UT_stricmp (name, id->m_name);
 }
@@ -362,7 +367,7 @@ const char * UT_HashColor::setColor (unsigned char r, unsigned char g, unsigned
 	m_colorBuffer[6] = hexval[ b       & 0x0f];
 	m_colorBuffer[7] = 0;
 
-	return (const char *) m_colorBuffer;
+	return static_cast<const char *>(m_colorBuffer);
 }
 
 const char * UT_HashColor::lookupNamedColor (const char * color_name)
@@ -373,7 +378,7 @@ const char * UT_HashColor::lookupNamedColor (const char * color_name)
 	size_t length = sizeof (s_Colors) / sizeof (s_Colors[0]);
 
 	colorToRGBMapping * id = 0;
-	id = (colorToRGBMapping *) bsearch (color_name, s_Colors, (int) length, sizeof (colorToRGBMapping), color_compare);
+	id = static_cast<colorToRGBMapping *>(bsearch (color_name, s_Colors, static_cast<int>(length), sizeof (colorToRGBMapping), color_compare));
 
 	if (id == 0) return 0;
 
@@ -411,7 +416,7 @@ const char * UT_HashColor::setHashIfValid (const char * color_hash)
 	m_colorBuffer[0] = '#';
 	m_colorBuffer[7] = 0;
 
-	return (const char *) m_colorBuffer;
+	return static_cast<const char *>(m_colorBuffer);
 }
 
 const UT_RGBColor UT_HashColor::rgb ()
@@ -422,9 +427,9 @@ const UT_RGBColor UT_HashColor::rgb ()
 
 	if (m_colorBuffer[0])
 	{
-		r = (unsigned char) (x_hexDigit (m_colorBuffer[1]) << 4) | (unsigned char) x_hexDigit (m_colorBuffer[2]);
-		g = (unsigned char) (x_hexDigit (m_colorBuffer[3]) << 4) | (unsigned char) x_hexDigit (m_colorBuffer[4]);
-		b = (unsigned char) (x_hexDigit (m_colorBuffer[5]) << 4) | (unsigned char) x_hexDigit (m_colorBuffer[6]);
+		r = static_cast<unsigned char>(x_hexDigit (m_colorBuffer[1]) << 4) | static_cast<unsigned char>(x_hexDigit (m_colorBuffer[2]));
+		g = static_cast<unsigned char>(x_hexDigit (m_colorBuffer[3]) << 4) | static_cast<unsigned char>(x_hexDigit (m_colorBuffer[4]));
+		b = static_cast<unsigned char>(x_hexDigit (m_colorBuffer[5]) << 4) | static_cast<unsigned char>(x_hexDigit (m_colorBuffer[6]));
 	}
 
 	return UT_RGBColor (r, g, b);
@@ -509,20 +514,50 @@ UT_RGBColor::UT_RGBColor()
 	m_red = 0;
 	m_grn = 0;
 	m_blu = 0;
+	m_bIsTransparent = false;
 }
 
-UT_RGBColor::UT_RGBColor(unsigned char red, unsigned char grn, unsigned char blu)
+UT_RGBColor::UT_RGBColor(unsigned char red, unsigned char grn, unsigned char blu, bool bTransparent)
 {
 	m_red = red;
 	m_grn = grn;
 	m_blu = blu;
+	m_bIsTransparent = bTransparent;
 }
 
-void UT_setColor(UT_RGBColor & col, unsigned char r, unsigned char g, unsigned char b)
+UT_RGBColor::UT_RGBColor(const UT_RGBColor &c)
+{
+	m_red = c.m_red;
+	m_grn = c.m_grn;
+	m_blu = c.m_blu;
+	m_bIsTransparent = c.m_bIsTransparent;
+}
+
+bool UT_RGBColor::setColor(const char * pszColor)
+{
+	unsigned char r = m_red, g = m_grn, b = m_blu;
+	
+	if(!pszColor || !strcmp(pszColor,"transparent") /* || !strcmp(pszColor,"ffffff") */)
+	{
+		m_red = m_grn = m_blu = 255;
+		m_bIsTransparent = true;
+	}
+	else
+	{
+		UT_parseColor(pszColor, *this);
+		m_bIsTransparent = false;
+	}
+	
+	return (r != m_red || g != m_grn || b != m_blu);
+}
+
+
+void UT_setColor(UT_RGBColor & col, unsigned char r, unsigned char g, unsigned char b, bool bTransparent)
 {
 	col.m_red = r;
 	col.m_grn = g;
 	col.m_blu = b;
+	col.m_bIsTransparent = bTransparent;
 }
 
 void UT_parseColor(const char *p, UT_RGBColor& c)
@@ -543,6 +578,14 @@ void UT_parseColor(const char *p, UT_RGBColor& c)
 	    return ;
 	  }
 
+
+	if(!strcmp(p,"transparent") /* || !strcmp(p,"ffffff") */)
+	{
+		c.m_red = c.m_grn = c.m_blu = 255;
+		c.m_bIsTransparent = true;
+		return;
+	}
+	
 	UT_HashColor hash;
 
 	if (hash.setColor (p))
@@ -558,6 +601,7 @@ void UT_parseColor(const char *p, UT_RGBColor& c)
 	    UT_DEBUGMSG(("String = %s \n",p));
 	    UT_ASSERT(UT_NOT_IMPLEMENTED);
 	  }
+
 }
 
 #ifdef WIN32
@@ -627,11 +671,24 @@ static ucs_range s_word_delim[] =
   {0x00a9, 0x00b3},
   {0x00b5, 0x00b7},
   {0x00b9, 0x00bf},
-  {0x00f7, 0x00f7}
+  {0x00f7, 0x00f7},
+  {0x060c, 0x060c}, //Arabic punctuation
+  {0x061b, 0x061b},
+  {0x061f, 0x061f},
+  {0xfeff, 0xfeff}
 };
 
 static bool s_find_delim(UT_UCSChar c)
 {
+    // This function is called for every character in the document
+    // during spell-checking. So make an effort to return in a hurry,
+    // by checking for the common cases (a-z and the biggest area)
+    // first. (not A-Z since there are more word seperators than
+    // capitals in a sentence!)
+    if ('a' <= c && c <= 'z') return false;
+    if (0x00f8 <= c && c <= 0xfefe) return false;
+
+    // Everything else via the table above...
     for(UT_uint32 i = 0; i < NrElements(s_word_delim); i++)
 	{
 		if(c < s_word_delim[i].low)
@@ -654,7 +711,7 @@ bool UT_isWordDelimiter(UT_UCSChar currentChar, UT_UCSChar followChar, UT_UCSCha
 		case UCS_RDBLQUOTE:    // smart quote, close double /* wjc */
 		case UCS_LQUOTE:    // smart quote, open single  /* wjc */
 		case UCS_RQUOTE:	// we want quotes inside words for contractions
-			if (UT_UCS_isalpha(followChar) && UT_UCS_isalpha(prevChar))
+			if (UT_UCS4_isalpha(followChar) && UT_UCS4_isalpha(prevChar))
 			  {
 				  return false;
 			  }
@@ -751,7 +808,7 @@ const XML_Char* UT_getAttribute(const XML_Char* name, const XML_Char** atts)
 
 	while (*p)
 	{
-		if (0 == strcmp((const char*)p[0], (const char*)name))
+		if (0 == strcmp(static_cast<const char*>(p[0]), static_cast<const char*>(name)))
 			break;
 		p += 2;
 	}
@@ -768,8 +825,8 @@ UT_sint32 signedLoWord(UT_uint32 dw)
 {
 	// return low word as a signed quantity
 
-	unsigned short u16 = (unsigned short)(dw & 0xffff);
-	signed short   s16 = *(signed short *)&u16;
+	unsigned short u16 = static_cast<unsigned short>(dw & 0xffff);
+	signed short   s16 = *reinterpret_cast<signed short *>(&u16);
 	UT_sint32      s32 = s16;
 
 	return s32;
@@ -779,8 +836,8 @@ UT_sint32 signedHiWord(UT_uint32 dw)
 {
 	// return high word as a signed quantity
 
-	unsigned short u16 = (unsigned short)((dw >> 16) & 0xffff);
-	signed short   s16 = *(signed short *)&u16;
+	unsigned short u16 = static_cast<unsigned short>((dw >> 16) & 0xffff);
+	signed short   s16 = *reinterpret_cast<signed short *>(&u16);
 	UT_sint32      s32 = s16;
 
 	return s32;
@@ -901,7 +958,7 @@ UT_uint32 UT_HeadingDepth(const char * szHeadingName)
 			break;
 		}
 	}
-	i = (UT_uint32) atoi(sz.c_str());
+	i = static_cast<UT_uint32>(atoi(sz.c_str()));
 	return i;
 }
 
@@ -923,3 +980,90 @@ void * UT_calloc ( UT_uint32 nmemb, UT_uint32 size )
   return ::calloc ( nmemb, size ) ;
 #endif
 }
+
+
+//////////////////////////////////////////////////
+UT_UniqueId::UT_UniqueId()
+{
+	memset(m_iID,0,sizeof(m_iID));
+
+	UT_uint32 i = (UT_uint32) List;
+	m_iID[i] = AUTO_LIST_RESERVED;
+}
+
+
+/*!
+    returns unique id (0 <= id < UT_UID_INVALID) of given type or
+    UT_UID_INVALID on failure
+*/
+UT_uint32 UT_UniqueId::getUID(idType t)
+{
+	UT_return_val_if_fail(t < _Last, UT_UID_INVALID);
+	UT_uint32 i = (UT_uint32)t;
+	UT_uint32 r = m_iID[i]++;
+	return r;
+}
+
+/*!
+    sets the minimum id to be returned by subsequent calls to getUID()
+    to iMin and returns true on success; returns false if it fails
+    (either because unknown type is specified or because the value of
+    iMin is too high)
+*/
+bool UT_UniqueId::setMinId(idType t, UT_uint32 iMin)
+{
+	UT_return_val_if_fail(t < _Last, false);
+
+	// we really want some space left to generate future id's
+	UT_return_val_if_fail(iMin < UINT_MAX - 1000, false);
+	
+	UT_uint32 i = (UT_uint32) t;
+
+	if(m_iID[i] > iMin)
+		return false;
+	
+	m_iID[i] = iMin;
+	return true;
+}
+
+/*!
+    returns true of iId can be used as a unique identifier of a type
+    t; before using iId, THE CALLER MUST CALL setMinId(t, iId+1) !!!
+*/
+bool UT_UniqueId::isIdUnique(idType t, UT_uint32 iId)
+{
+	UT_return_val_if_fail(t < _Last, false);
+
+	// we really want some space left to generate future id's
+	UT_return_val_if_fail(iId < UINT_MAX - 1000, false);
+	
+	UT_uint32 i = (UT_uint32) t;
+
+	if(m_iID[i] > iId)
+		return false;
+	
+	return true;
+}
+
+/**
+ * UT_parseBool 
+ *
+ * Returns true if param is [true, 1, yes, allow, enable, on]
+ * Returns false if param is [false, 0, no, disallow, disable, off]
+ * Returns dfl otherwise, including if param is null
+ */
+bool UT_parseBool (const char * param, bool dfl)
+{
+	UT_return_val_if_fail (param && strlen(param), dfl);
+
+	if (!UT_strnicmp(param, "true", 4) || !UT_strnicmp(param, "1", 1) ||
+		!UT_strnicmp(param, "yes", 3) || !UT_strnicmp(param, "allow", 5) ||
+		!UT_strnicmp(param, "enable", 6) || !UT_strnicmp(param, "on", 2))
+		return true;
+	else if (!UT_strnicmp(param, "false", 5) || !UT_strnicmp(param, "0", 1) ||
+			 !UT_strnicmp(param, "no", 2) || !UT_strnicmp(param, "disallow", 8) ||
+			 !UT_strnicmp(param, "disable", 7) || !UT_strnicmp(param, "off", 3))
+		return false;
+	
+	return dfl;
+}
