@@ -43,11 +43,15 @@
 #include "xap_Menu_Layouts.h"
 #include "xap_Toolbar_Layouts.h"
 #include "ut_rand.h"
+#include "ut_contextGlyph.h"
+#include "ut_map.h"
+#include "gr_CharWidthsCache.h"
+
+UT_Map * abi_ut_map_instance = 0;
 
 /*****************************************************************/
 
 XAP_App * XAP_App::m_pApp = NULL;
-bool XAP_App::s_bShowDisplay = true;
 
 XAP_App * XAP_App::getApp() {return m_pApp;}
 
@@ -65,14 +69,33 @@ XAP_App::XAP_App(XAP_Args * pArgs, const char * szAppName)
 	  m_lastFocussedFrame(NULL),
 	  m_pMenuFactory(NULL),
 	  m_pToolbarFactory(NULL),
-	  m_bAllowCustomizing(true)
+	  m_bAllowCustomizing(true),
+	  m_bAreCustomized(true),
+	  m_bDebugBool(false),
+	  m_bBonoboRunning(false),
+	  m_bEnableSmoothScrolling(true)
 {
+#ifdef DEBUG
+	_fundamentalAsserts(); // see the comments in the function itself
+#endif
+	
 	UT_ASSERT(szAppName && *szAppName);
 	m_pApp = this;
+
+	// TODO don't build GUI when only doing commandline
+	//      file conversion
 	UT_DEBUGMSG(("SEVIOR: Building menus and toolbars \n"));
 	m_pMenuFactory = new XAP_Menu_Factory(this);
 	m_pToolbarFactory = new XAP_Toolbar_Factory(this);
 	clearIdTable();
+
+	/* hack to force the linker to link in UT_Map functions
+	 */
+	if (abi_ut_map_instance)
+	{
+	    delete abi_ut_map_instance;
+	    abi_ut_map_instance = new UT_Map;
+	}
 }
 
 XAP_App::~XAP_App()
@@ -97,42 +120,54 @@ XAP_App::~XAP_App()
 
 	// Delete the instance of the Encoding Manager.
 	XAP_EncodingManager::get_instance()->Delete_instance();
+
+	// This is to delete static data allocated by UT_contextGlyph; it
+	// is strictly speaking not necessary -- this is really to shut up
+	// the debugger complaining about memory leaks
+	UT_contextGlyph cg(true);
+	cg.static_destructor();
+	GR_CharWidthsCache::destroyCharWidthsCache();
 }
 
 const char* XAP_App::getBuildId ()
 {
-  return s_szBuild_ID;
+	return s_szBuild_ID;
 }
 
 const char* XAP_App::getBuildVersion ()
 {
-  return s_szBuild_Version;
+	return s_szBuild_Version;
 }
 
 const char* XAP_App::getBuildOptions ()
 {
-  return s_szBuild_Options;
+	return s_szBuild_Options;
 }
 
 const char* XAP_App::getBuildTarget ()
 {
-  return s_szBuild_Target;
+	return s_szBuild_Target;
 }
 
 const char* XAP_App::getBuildCompileTime ()
 {
-  return s_szBuild_CompileTime;
+	return s_szBuild_CompileTime;
 }
 
 const char* XAP_App::getBuildCompileDate ()
 {
-  return s_szBuild_ID;
+	return s_szBuild_CompileDate;
+}
+
+const char* XAP_App::getAbiSuiteHome ()
+{
+	return s_szAbiSuite_Home;
 }
 
 /*! this function is silly */
 const XAP_EncodingManager* XAP_App::getEncodingManager() const 
 { 
-    return XAP_EncodingManager::get_instance();
+	return XAP_EncodingManager::get_instance();
 };
 
 EV_Menu_ActionSet *XAP_App::getMenuActionSet()
@@ -179,56 +214,83 @@ bool XAP_App::initialize()
 	bool bAllowCustom = true;
 	getPrefsValueBool(XAP_PREF_KEY_AllowCustomToolbars, &bAllowCustom);
 	if(bAllowCustom)
-	{
-		m_pToolbarFactory->restoreToolbarsFromCurrentScheme();
 		setToolbarsCustomizable(true);
-	}
 	else
 	{
 		setToolbarsCustomizable(false);
+		setToolbarsCustomized(false);
 	}
+	m_pToolbarFactory->restoreToolbarsFromCurrentScheme();
+	if(!bAllowCustom)
+	{
+		m_pToolbarFactory->resetAllToolbarsToDefault();
+	}
+//
+// Set Smooth Scrolling
+//
+	bool bEnableSmooth = true;
+	getPrefsValueBool(XAP_PREF_KEY_EnableSmoothScrolling, &bEnableSmooth);
+	if(bEnableSmooth)
+		setEnableSmoothScrolling(true);
+	else
+		setEnableSmoothScrolling(false);
+
 	// TODO use m_pArgs->{argc,argv} to process any command-line
 	// TODO options that we need.
 	//
 	// Need to initialize the random number generator. 
 	//
-	UT_uint32 t = (UT_uint32) time( NULL);
+	UT_uint32 t = static_cast<UT_uint32>(time(NULL));
 	UT_srandom(t);
+
 	return true;
 }
 
+void XAP_App::resetToolbarsToDefault(void)
+{
+	//
+	// Set all the frames to default toolbars
+	//
+	m_pToolbarFactory->resetAllToolbarsToDefault();
+	UT_uint32 count = m_vecFrames.getItemCount();
+	UT_Vector vClones;
+	UT_uint32 i = 0;
+	for(i=0; i< count; i++)
+	{
+		XAP_Frame * pFrame = static_cast<XAP_Frame *>(m_vecFrames.getNthItem(i));
+		if(pFrame->getViewNumber() > 0)
+		{
+			getClones(&vClones,pFrame);
+			UT_uint32 j=0;
+			for(j=0; j < vClones.getItemCount(); j++)
+			{
+				XAP_Frame * f = static_cast<XAP_Frame *>(vClones.getNthItem(j));
+				f->rebuildAllToolbars();
+			}
+		}
+		else
+		{
+			pFrame->rebuildAllToolbars();
+		}
+	}
+	setToolbarsCustomized (true);
+}
+
+void XAP_App::setEnableSmoothScrolling(bool b)
+{
+	m_bEnableSmoothScrolling = b;
+}
+
 void XAP_App::setToolbarsCustomizable(bool b)
 {
-	if(m_bAllowCustomizing && !b)
-	{
-//
-// Set all the frames to default toolbars
-//
+	if(m_bAllowCustomizing != b)
 		m_bAllowCustomizing = b;
-		m_pToolbarFactory->resetAllToolbarsToDefault();
-		UT_uint32 count = m_vecFrames.getItemCount();
-		UT_Vector vClones;
-		UT_uint32 i = 0;
-		for(i=0; i< count; i++)
-		{
-			XAP_Frame * pFrame = (XAP_Frame *) m_vecFrames.getNthItem(i);
-			if(pFrame->getViewNumber() > 0)
-			{
-				getClones(&vClones,pFrame);
-				UT_uint32 j=0;
-				for(j=0; j < vClones.getItemCount(); j++)
-				{
-					XAP_Frame * f = (XAP_Frame *) vClones.getNthItem(j);
-					f->rebuildAllToolbars();
-				}
-			}
-			else
-			{
-				pFrame->rebuildAllToolbars();
-			}
-		}
-	}
-	m_bAllowCustomizing = b;
+}
+
+void XAP_App::setToolbarsCustomized(bool b)
+{
+	if(m_bAreCustomized != b)
+		m_bAreCustomized = b;
 }
 
 const char * XAP_App::getApplicationTitleForTitleBar() const
@@ -287,13 +349,13 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 	if (pCloneOf)
 	{
 		// locate vector of this frame's clones
-		const void * pEntry = m_hashClones.pick(pCloneOf->getViewKey());
+		void * pEntry = const_cast<void *>(m_hashClones.pick(pCloneOf->getViewKey()));
 		UT_Vector * pvClones = NULL;
 
 		if (pEntry)
 		{
 			// hash table entry already exists
-			pvClones = (UT_Vector *) pEntry;
+			pvClones = static_cast<UT_Vector *>(pEntry);
 
 			if (!pvClones)
 			{
@@ -305,7 +367,7 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 
 				// reuse this slot
 				m_hashClones.set(pCloneOf->getViewKey(), 
-						 (void *)pvClones);
+						 static_cast<void *>(pvClones));
 			}
 		}
 		else
@@ -317,7 +379,7 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 			pvClones->addItem(pCloneOf);
 
 			// add it to the hash table
-			m_hashClones.insert(pCloneOf->getViewKey(), (void *)pvClones);
+			m_hashClones.insert(pCloneOf->getViewKey(), static_cast<void *>(pvClones));
 		}
 
 		pvClones->addItem(pFrame);
@@ -325,7 +387,7 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 		// notify all clones of their new view numbers
 		for (UT_uint32 j=0; j<pvClones->getItemCount(); j++)
 		{
-			XAP_Frame * f = (XAP_Frame *) pvClones->getNthItem(j);
+			XAP_Frame * f = static_cast<XAP_Frame *>(pvClones->getNthItem(j));
 			UT_ASSERT(f);
 
 			f->setViewNumber(j+1);
@@ -348,18 +410,18 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 
 	if(pFrame == m_lastFocussedFrame )
 	{
-		m_lastFocussedFrame = (XAP_Frame *) NULL;
+		m_lastFocussedFrame = static_cast<XAP_Frame *>(NULL);
 	}
 
 	if (pFrame->getViewNumber() > 0)
 	{
 		// locate vector of this frame's clones
-		const void * pEntry = m_hashClones.pick(pFrame->getViewKey());
+		void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
 		UT_ASSERT(pEntry);
 
 		if (pEntry)
 		{
-			UT_Vector * pvClones = (UT_Vector *) pEntry;
+			UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
 			UT_ASSERT(pvClones);
 
 			// remove this frame from the vector
@@ -379,7 +441,7 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 			if (count == 1)
 			{
 				// remaining clone is now a singleton
-				f = (XAP_Frame *) pvClones->getNthItem(count-1);
+				f = static_cast<XAP_Frame *>(pvClones->getNthItem(count-1));
 				UT_ASSERT(f);
 
 				f->setViewNumber(0);
@@ -395,7 +457,7 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 				// notify remaining clones of their new view numbers
 				for (UT_uint32 j=0; j<count; j++)
 				{
-					f = (XAP_Frame *) pvClones->getNthItem(j);
+					f = static_cast<XAP_Frame *>(pvClones->getNthItem(j));
 					UT_ASSERT(f);
 
 					f->setViewNumber(j+1);
@@ -435,7 +497,7 @@ bool XAP_App::forgetClones(XAP_Frame * pFrame)
 	
 	for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
 	{
-		XAP_Frame * f = (XAP_Frame *) vClones.getNthItem(i);
+		XAP_Frame * f = static_cast<XAP_Frame *>(vClones.getNthItem(i));
 		forgetFrame(f);
 	}
 
@@ -449,10 +511,10 @@ bool XAP_App::getClones(UT_Vector *pvClonesCopy, XAP_Frame * pFrame)
 	UT_ASSERT(pFrame->getViewNumber() > 0);
 
 	// locate vector of this frame's clones
-	const void * pEntry = m_hashClones.pick(pFrame->getViewKey());
+	void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
 	UT_ASSERT(pEntry);
 
-	UT_Vector * pvClones = (UT_Vector *) pEntry;
+	UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
 	UT_ASSERT(pvClones);
 
 	return pvClonesCopy->copy(pvClones);
@@ -464,12 +526,12 @@ bool XAP_App::updateClones(XAP_Frame * pFrame)
 	UT_ASSERT(pFrame->getViewNumber() > 0);
 
 	// locate vector of this frame's clones
-	const void * pEntry = m_hashClones.pick(pFrame->getViewKey());
+	void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
 	UT_ASSERT(pEntry);
 
 	if (pEntry)
 	{
-		UT_Vector * pvClones = (UT_Vector *) pEntry;
+		UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
 		UT_ASSERT(pvClones);
 
 		UT_uint32 count = pvClones->getItemCount();
@@ -478,7 +540,7 @@ bool XAP_App::updateClones(XAP_Frame * pFrame)
 
 		for (UT_uint32 j=0; j<count; j++)
 		{
-			f = (XAP_Frame *) pvClones->getNthItem(j);
+			f = static_cast<XAP_Frame *>(pvClones->getNthItem(j));
 			UT_ASSERT(f);
 
 			f->updateTitle();
@@ -499,7 +561,7 @@ XAP_Frame * XAP_App::getFrame(UT_uint32 ndx) const
 	
 	if (ndx < m_vecFrames.getItemCount())
 	{
-		pFrame = (XAP_Frame *) m_vecFrames.getNthItem(ndx);
+		pFrame = static_cast<XAP_Frame *>(m_vecFrames.getNthItem(ndx));
 		UT_ASSERT(pFrame);
 	}
 	return pFrame;
@@ -523,7 +585,7 @@ UT_sint32 XAP_App::findFrame(const char * szFilename)
 
 		if (s && *s && (0 == UT_stricmp(szFilename, s)))
 		{
-			return (UT_sint32) i;
+			return static_cast<UT_sint32>(i);
 		}
 	}
 
@@ -560,12 +622,12 @@ bool XAP_App::isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const
 /*!
  * Look up the custom dictionary for suggested words
  */
-void    XAP_App::suggestWord(UT_Vector * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord)
+void XAP_App::suggestWord(UT_Vector * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord)
 {
-  if(m_pDict)
-  {
-       m_pDict->suggestWord(pVecSuggestions, pWord, lenWord);
-  }
+	if(m_pDict)
+	{
+		m_pDict->suggestWord(pVecSuggestions, pWord, lenWord);
+	}
 }
 
 XAP_Prefs * XAP_App::getPrefs() const
@@ -599,12 +661,12 @@ bool XAP_App::getPrefsValueBool(const XML_Char * szKey, bool * pbValue) const
 
 void XAP_App::rememberFocussedFrame( void * pJustFocussedFrame)
 {
-	m_lastFocussedFrame = (XAP_Frame *) pJustFocussedFrame;
+	m_lastFocussedFrame = static_cast<XAP_Frame *>(pJustFocussedFrame);
 
 	UT_sint32 i = safefindFrame( m_lastFocussedFrame);
 	if(i < 0 ) 
 	{   
-		m_lastFocussedFrame = (XAP_Frame *) NULL;
+		m_lastFocussedFrame = static_cast<XAP_Frame *>(NULL);
 	}
 	notifyModelessDlgsOfActiveFrame(m_lastFocussedFrame);
 }
@@ -625,23 +687,23 @@ UT_sint32 XAP_App::safefindFrame( XAP_Frame * f)
 
 void XAP_App::clearLastFocussedFrame()
 {
-	m_lastFocussedFrame = (XAP_Frame *) NULL;
+	m_lastFocussedFrame = static_cast<XAP_Frame *>(NULL);
 }
 
 XAP_Frame* XAP_App::getLastFocussedFrame() 
 {
-	if(m_lastFocussedFrame == (XAP_Frame *) NULL)
-		return (XAP_Frame *) NULL;
+		if(m_lastFocussedFrame == static_cast<XAP_Frame *>(NULL))
+			return static_cast<XAP_Frame *>(NULL);
 	UT_sint32 i = safefindFrame(m_lastFocussedFrame);
 	if( i>= 0)
 		return m_lastFocussedFrame;
-	return (XAP_Frame *) NULL;
+	return static_cast<XAP_Frame *>(NULL);
 }
 
 XAP_Frame * XAP_App::findValidFrame()
 {
-        XAP_Frame * validFrame =  getFrame(0);
-        return validFrame;
+	XAP_Frame * validFrame =  getFrame(0);
+	return validFrame;
 }
 
 void XAP_App::clearIdTable()
@@ -649,7 +711,7 @@ void XAP_App::clearIdTable()
 	for(UT_sint32 i =0; i <= NUM_MODELESSID; i++)
 	{
 		m_IdTable[i].id =  -1;
-		m_IdTable[i].pDialog = (XAP_Dialog_Modeless *) NULL;
+		m_IdTable[i].pDialog = static_cast<XAP_Dialog_Modeless *>(NULL);
 	}
 }
 
@@ -672,10 +734,15 @@ void XAP_App::forgetModelessId( UT_sint32 id )
 
 	UT_sint32 i;
 	for(i=0; i <= NUM_MODELESSID && m_IdTable[i].id != id; i++) ;
-	UT_ASSERT( i <= NUM_MODELESSID );
-	UT_ASSERT( m_IdTable[i].id == id );
+	if(i >  NUM_MODELESSID)
+	{
+//
+// forgotten already
+//
+		return;
+	}
 	m_IdTable[i].id =  -1;
-	m_IdTable[i].pDialog = (XAP_Dialog_Modeless *) NULL;
+	m_IdTable[i].pDialog = static_cast<XAP_Dialog_Modeless *>(NULL);
 }
 
 bool XAP_App::isModelessRunning(UT_sint32 id)
@@ -704,9 +771,14 @@ void XAP_App::closeModelessDlgs()
 
 	for(UT_sint32 i=0; i <= NUM_MODELESSID; i++)
 	{
-		if(getModelessDialog(i) != (XAP_Dialog_Modeless *) NULL)
+		if(m_IdTable[i].id >= 0)
 		{
-			getModelessDialog(i)->destroy();
+			if(getModelessDialog(i) != static_cast<XAP_Dialog_Modeless *>(NULL))
+			{
+				getModelessDialog(i)->destroy();
+			}
+			m_IdTable[i].id = -1;
+			m_IdTable[i].pDialog = NULL;
 		}
 	}
 }
@@ -716,7 +788,7 @@ void XAP_App::notifyModelessDlgsOfActiveFrame(XAP_Frame *p_Frame)
 {
 	for(UT_sint32 i=0; i <= NUM_MODELESSID; i++)
 	{
-		if(getModelessDialog(i) != (XAP_Dialog_Modeless *) NULL)
+		if(getModelessDialog(i) != static_cast<XAP_Dialog_Modeless *>(NULL))
 		{
 			getModelessDialog(i)->setActiveFrame(p_Frame);
 		}
@@ -727,7 +799,7 @@ void XAP_App::notifyModelessDlgsCloseFrame(XAP_Frame *p_Frame)
 {
 	for(UT_sint32 i=0; i <= NUM_MODELESSID; i++)
 	{
-		if(getModelessDialog(i) != (XAP_Dialog_Modeless *) NULL)
+		if(getModelessDialog(i) != static_cast<XAP_Dialog_Modeless *>(NULL))
 		{
 			getModelessDialog(i)->notifyCloseFrame(p_Frame);
 		}
@@ -747,7 +819,8 @@ bool XAP_App::getGeometry(UT_sint32 *x, UT_sint32 *y, UT_uint32 *width, UT_uint3
 	return prefs->getGeometry(x, y, width, height, flags);
 }
 
-void XAP_App::parseAndSetGeometry(const char *string) {
+void XAP_App::parseAndSetGeometry(const char *string)
+{
 	UT_uint32 nw, nh, nflags;
     UT_sint32 nx, ny;
     char *next;
@@ -755,64 +828,51 @@ void XAP_App::parseAndSetGeometry(const char *string) {
 	nw = nh = nflags = 0;
 	nx = ny = 0;
 
-    next = (char *)string;
-    if (*next != '+' && *next != '-') {
+    next = const_cast<char *>(string);
+    if (*next != '+' && *next != '-')
+	{
         nw = strtoul(next, &next, 10);
-        if(*next == 'x' || *next == 'X') {
+        if(*next == 'x' || *next == 'X')
+		{
             nh = strtoul(++next, &next, 10);
             nflags |= PREF_FLAG_GEOMETRY_SIZE;
         }
     }
-    if (*next == '+' || *next == '-') {
+    if (*next == '+' || *next == '-')
+	{
         nx = strtoul(next, &next, 10);
-        if(*next == '+' || *next == '-') {
+        if(*next == '+' || *next == '-')
+		{
             ny = strtoul(next, &next, 10);
             nflags |= PREF_FLAG_GEOMETRY_POS;
         }
     }
 
 	//Don't update the geometry from the file
-	if(nflags) {
+	if(nflags)
+	{
 		nflags |= PREF_FLAG_GEOMETRY_NOUPDATE;
 		setGeometry(nx, ny, nw, nh, nflags);
 	}
 } 
 
-void XAP_App::_printUsage()
-{
-	// just print to stdout, not stderr
-	printf("\nUsage: %s [option]... [file]...\n\n", m_pArgs->m_argv[0]);
-	printf("  -to               The target format of the file\n");
-        printf("                    (abw, zabw, rtf, txt, utf8, html, latex)\n");
-	printf("  -verbose          The verbosity level (0, 1, 2)\n");
-	printf("  -show             If you really want to start the GUI\n");
-        printf("                    (even if you use the -to option)\n");
 #ifdef DEBUG
-	printf("  -dumpstrings      dump strings strings to file\n");
+/*!
+    This function contains some of the most fundamental assumptions
+    about the compiler without which the code will not work; this is
+    for the benefit of future ports, etc.
+    Tomas, May 15, 2003
+
+    If you get assert in this function, then you have a real problem
+    -- the code will not work without substantial adjustments.
+*/
+void XAP_App::_fundamentalAsserts() const
+{
+	// our UT_Vector class is frequently used to store integers
+	// directly inside of a void *
+	UT_ASSERT(sizeof(void*) >= sizeof(UT_uint32));
+
+	// we frequently assume that XML_Char * is only one byte in size
+	UT_ASSERT(sizeof(XML_Char) == sizeof(char));
+}
 #endif
-	printf("  -geometry geom    set initial frame geometry\n");
-	printf("  -lib dir          use dir for application components\n");
-	printf("  -nosplash         do not show splash screen\n");
-
-	printf("\n");
-}
-
-
-bool XAP_App::setDisplayStatus(bool b)
-{
-    s_bShowDisplay = b;
-    return true;
-}
-
-bool XAP_App::getDisplayStatus(void) const
-{
-	return s_bShowDisplay;
-}
-
-
-
-
-
-
-
-
