@@ -36,15 +36,16 @@ struct xmlToIdMapping {
 
 static  UT_sint32 compareStrings(const void * ppS1, const void * ppS2)
 {
-	const char ** sz1 = (const char **) (ppS1);
-	const char ** sz2 = (const char **) (ppS2);
-	return UT_stricmp(*sz1, *sz2);
+	const char * sz1 = static_cast<const char *>(ppS1);
+	const char * sz2 = static_cast<const char *>(ppS2);
+	return UT_stricmp(sz1, sz2);
 }
 
 enum
 {
 	TT_ABIPREFERENCES,
 	TT_GEOMETRY,
+	TT_PLUGIN,
 	TT_RECENT,
 	TT_SCHEME,
 	TT_SELECT
@@ -55,6 +56,7 @@ static struct xmlToIdMapping s_Tokens[] =
 {
 	{ "AbiPreferences",		TT_ABIPREFERENCES },
 	{ "Geometry",			TT_GEOMETRY },
+	{ "Plugin",				TT_PLUGIN },
 	{ "Recent",				TT_RECENT },
 	{ "Scheme",				TT_SCHEME },
 	{ "Select",				TT_SELECT }
@@ -85,7 +87,7 @@ XAP_PrefsScheme::~XAP_PrefsScheme(void)
 
 	for (UT_uint32 i = 0 ; i < cnt; i++)
 	  {
-	    char * val = (char *)pVec->getNthItem (i);
+	    char * val = static_cast<char *>(pVec->getNthItem (i));
 	    FREEP(val);
 	  }
 
@@ -109,18 +111,18 @@ bool XAP_PrefsScheme::setValue(const XML_Char * szKey, const XML_Char * szValue)
 	const void * pEntry = m_hash.pick(szKey);
 	if (pEntry)
 	{
-		if (strcmp((const char*)szValue,(const char *)pEntry) == 0)
+		if (strcmp(static_cast<const char*>(szValue),static_cast<const char *>(pEntry)) == 0)
 			return true;				// equal values, no changes required
 		
 		FREEP(pEntry);
 		m_hash.set (szKey, 
-			    (void *)UT_strdup (szValue));
+			    static_cast<void *>(UT_strdup (szValue)));
 	}
 	else
 	{
 		// otherwise, need to add a new entry
 		m_hash.insert(szKey,
-			      (void *)UT_strdup(szValue));
+			      static_cast<void *>(UT_strdup(szValue)));
 	}
 
 	m_pPrefs->_markPrefChange( szKey );
@@ -130,7 +132,7 @@ bool XAP_PrefsScheme::setValue(const XML_Char * szKey, const XML_Char * szValue)
 
 bool XAP_PrefsScheme::setValueBool(const XML_Char * szKey, bool bValue)
 {
-	return setValue(szKey, (XML_Char*) ((bValue) ? "1" : "0"));
+	return setValue(szKey, reinterpret_cast<const XML_Char*>((bValue) ? "1" : "0"));
 }
 
 bool XAP_PrefsScheme::getValue(const XML_Char * szKey, const XML_Char ** pszValue) const
@@ -140,7 +142,7 @@ bool XAP_PrefsScheme::getValue(const XML_Char * szKey, const XML_Char ** pszValu
 		return false;
 
 	if (pszValue)
-		*pszValue = (const XML_Char*)pEntry;
+		*pszValue = static_cast<const XML_Char*>(pEntry);
 	return true;
 }
 
@@ -150,7 +152,7 @@ bool XAP_PrefsScheme::getValue(const UT_String &stKey, UT_String &stValue) const
 	if (!pEntry)
 		return false;
 
-	stValue = (const char *)pEntry;
+	stValue = static_cast<const char *>(pEntry);
 	return true;
 }
 
@@ -183,7 +185,7 @@ bool XAP_PrefsScheme::getValueBool(const XML_Char * szKey, bool * pbValue) const
 
 bool XAP_PrefsScheme::getNthValue(UT_uint32 k, const XML_Char ** pszKey, const XML_Char ** pszValue) const
 {
-	if (k >= (UT_uint32)m_hash.size())
+	if (k >= static_cast<UT_uint32>(m_hash.size()))
 		return false;
 //
 // Output prefs in alphabetic Order.
@@ -194,14 +196,14 @@ bool XAP_PrefsScheme::getNthValue(UT_uint32 k, const XML_Char ** pszKey, const X
 	vecKeys.clear();
 	for(i=0; i< vecD->getItemCount(); i++)
 	{
-		vecKeys.addItem((void *) static_cast<UT_String *>(vecD->getNthItem(i))->c_str());
+		vecKeys.addItem(static_cast<const void *>(static_cast<UT_String *>(vecD->getNthItem(i))->c_str()));
 	}
 	vecKeys.qsort(compareStrings);
 	delete vecD;
 	const char * szKey = NULL;
 	const char * szValue = NULL;
-	szKey = (const XML_Char *) vecKeys.getNthItem(k);
-	szValue = (const XML_Char *)  m_hash.pick(szKey);
+	szKey = reinterpret_cast<XML_Char *>(vecKeys.getNthItem(k));
+	szValue = static_cast<const XML_Char *>(m_hash.pick(szKey));
 	if(szValue && *szValue)
 	{
 		*pszKey = szKey;
@@ -274,7 +276,7 @@ const char * XAP_Prefs::getRecent(UT_uint32 k) const
 	
 	if (k <= m_vecRecent.getItemCount())
 	{
-		pRecent = (const char *) m_vecRecent.getNthItem(k - 1);
+		pRecent = reinterpret_cast<const char *>(m_vecRecent.getNthItem(k - 1));
 	}
 
 	return pRecent;
@@ -291,7 +293,7 @@ void XAP_Prefs::addRecent(const char * szRecent)
 	// was it already here? 
 	for (UT_uint32 i=0; i<m_vecRecent.getItemCount(); i++)
 	{
-		sz = (const char *)m_vecRecent.getNthItem(i);
+		sz = reinterpret_cast<const char *>(m_vecRecent.getNthItem(i));
 		if ((sz==szRecent) || !strcmp(sz, szRecent))
 		{
 			// yep, we're gonna move it up
@@ -307,7 +309,7 @@ void XAP_Prefs::addRecent(const char * szRecent)
 		UT_cloneString((char *&)sz, szRecent);
 	}
 
-	m_vecRecent.insertItemAt((void *)sz, 0);
+	m_vecRecent.insertItemAt(const_cast<void *>(static_cast<const void *>(sz)), 0);
 	_pruneRecent();
 }
 
@@ -316,7 +318,7 @@ void XAP_Prefs::removeRecent(UT_uint32 k)
 	UT_ASSERT(k>0);
 	UT_ASSERT(k<=getRecentCount());
 
-	char * sz = (char *) m_vecRecent.getNthItem(k-1);
+	char * sz = static_cast<char *>(m_vecRecent.getNthItem(k-1));
 	FREEP(sz);
 
 	m_vecRecent.deleteNthItem(k-1);
@@ -330,9 +332,9 @@ void XAP_Prefs::_pruneRecent(void)
 	if (m_iMaxRecent == 0)
 	{
 		// nuke the whole thing
-		for (i = (signed) count; i > 0 ; i--)
+		for (i = static_cast<signed>(count); i > 0 ; i--)
 		{
-			char * sz = (char *) m_vecRecent.getNthItem(i-1);
+			char * sz = static_cast<char *>(m_vecRecent.getNthItem(i-1));
 			FREEP(sz);
 		}
 
@@ -341,7 +343,7 @@ void XAP_Prefs::_pruneRecent(void)
 	else if (count > m_iMaxRecent)
 	{
 		// prune entries past m_iMaxRecent
-		for (i = (signed) count; i > (signed) m_iMaxRecent; i--)
+		for (i = static_cast<signed>(count); i > static_cast<signed>(m_iMaxRecent); i--)
 			removeRecent(i);
 	}
 }
@@ -414,21 +416,32 @@ XAP_Prefs::XAP_Prefs(XAP_App * pApp)
 XAP_Prefs::~XAP_Prefs(void)
 {
 	UT_VECTOR_PURGEALL(XAP_PrefsScheme *, m_vecSchemes);
+	UT_VECTOR_PURGEALL(XAP_PrefsScheme *, m_vecPluginSchemes);
 	UT_VECTOR_FREEALL(char *, m_vecRecent);
 	UT_VECTOR_PURGEALL(tPrefsListenersPair *, m_vecPrefsListeners);
 }
 
 /*****************************************************************/
 
-XAP_PrefsScheme * XAP_Prefs::getNthScheme(UT_uint32 k) const
+XAP_PrefsScheme * XAP_Prefs::_getNthScheme(UT_uint32 k, const UT_Vector &vecSchemes) const
 {
-	UT_uint32 kLimit = m_vecSchemes.getItemCount();
+	UT_uint32 kLimit = vecSchemes.getItemCount();
 	if (k < kLimit)
-		return (XAP_PrefsScheme *)m_vecSchemes.getNthItem(k);
+		return static_cast<XAP_PrefsScheme *>(vecSchemes.getNthItem(k));
 	else
 		return NULL;
 }
 
+XAP_PrefsScheme * XAP_Prefs::getNthScheme(UT_uint32 k) const
+{
+	return _getNthScheme(k, m_vecSchemes);
+}
+
+XAP_PrefsScheme * XAP_Prefs::getNthPluginScheme(UT_uint32 k) const
+{
+	return _getNthScheme(k, m_vecPluginSchemes);
+}
+
 XAP_PrefsScheme * XAP_Prefs::getScheme(const XML_Char * szSchemeName) const
 {
 	UT_uint32 kLimit = m_vecSchemes.getItemCount();
@@ -438,7 +451,23 @@ XAP_PrefsScheme * XAP_Prefs::getScheme(const XML_Char * szSchemeName) const
 	{
 		XAP_PrefsScheme * p = getNthScheme(k);
 		UT_ASSERT(p);
-		if (strcmp((const char*)szSchemeName,(const char*)p->getSchemeName()) == 0)
+		if (strcmp(static_cast<const char*>(szSchemeName),static_cast<const char*>(p->getSchemeName())) == 0)
+			return p;
+	}
+
+	return NULL;
+}
+
+XAP_PrefsScheme * XAP_Prefs::getPluginScheme(const XML_Char * szSchemeName) const
+{
+	UT_uint32 kLimit = m_vecPluginSchemes.getItemCount();
+	UT_uint32 k;
+
+	for (k=0; k<kLimit; k++)
+	{
+		XAP_PrefsScheme * p = getNthPluginScheme(k);
+		UT_ASSERT(p);
+		if (strcmp(static_cast<const char*>(szSchemeName),static_cast<const char*>(p->getSchemeName())) == 0)
 			return p;
 	}
 
@@ -450,7 +479,7 @@ bool XAP_Prefs::addScheme(XAP_PrefsScheme * pNewScheme)
 	const XML_Char * szBuiltinSchemeName = getBuiltinSchemeName();
 	const XML_Char * szThisSchemeName = pNewScheme->getSchemeName();
 	
-	if (strcmp((const char*)szThisSchemeName, (const char*)szBuiltinSchemeName) == 0)
+	if (strcmp(static_cast<const char*>(szThisSchemeName), static_cast<const char*>(szBuiltinSchemeName)) == 0)
 	{
 		UT_ASSERT(m_builtinScheme == NULL);
 		m_builtinScheme = pNewScheme;
@@ -459,6 +488,11 @@ bool XAP_Prefs::addScheme(XAP_PrefsScheme * pNewScheme)
 	return (m_vecSchemes.addItem(pNewScheme) == 0);
 }
 
+bool XAP_Prefs::addPluginScheme(XAP_PrefsScheme * pNewScheme)
+{
+	return (m_vecPluginSchemes.addItem(pNewScheme) == 0);
+}
+
 
 XAP_PrefsScheme * XAP_Prefs::getCurrentScheme(bool bCreate)
 {
@@ -466,9 +500,11 @@ XAP_PrefsScheme * XAP_Prefs::getCurrentScheme(bool bCreate)
 	{
 		// the builtin scheme is not updatable, 
 		// so we may need to create one that is
-		if ( !strcmp((const char*)m_currentScheme->getSchemeName(), "_builtin_") ) 
+		if ( !strcmp(static_cast<const char*>(m_currentScheme->getSchemeName()), "_builtin_") ) 
 		{
-			const XML_Char new_name[] = "_custom_";
+	
+
+		const XML_Char new_name[] = "_custom_";
 
 			if (setCurrentScheme(new_name))
 			{
@@ -590,7 +626,7 @@ static int n_compare (const char *name, const xmlToIdMapping *id)
 #else
 extern "C" static int n_compare (const void *name, const void *id)
 {
-	return strcmp ((const char *)name, ((const xmlToIdMapping *)id)->m_name);
+	return strcmp (static_cast<const char *>(name), (static_cast<const xmlToIdMapping *>(id))->m_name);
 }
 #endif
 
@@ -611,13 +647,13 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		return;
 
 	xmlToIdMapping * id = NULL;
-	id = (xmlToIdMapping *)bsearch ((const void*)name, (const void*)s_Tokens,
+	id = static_cast<xmlToIdMapping *>(bsearch (static_cast<const void*>(name), static_cast<const void*>(s_Tokens),
 									sizeof(s_Tokens)/sizeof(xmlToIdMapping),
 									sizeof (xmlToIdMapping),
 #ifndef __MRC__
 									(int (*)(const void*, const void*))
 #endif
-									n_compare);
+									n_compare));
 	if (!id)
 		return;
 
@@ -635,7 +671,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		{
 			UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
 
-			if (strcmp((const char*)a[0], "app") == 0)
+			if (strcmp(static_cast<const char*>(a[0]), "app") == 0)
 			{
 				// TODO the following test will fail if you are running
 				// TODO both an AbiWord (release) build and an AbiWord
@@ -646,13 +682,13 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 				const char * szThisApp = m_pApp->getApplicationName();
 				UT_DEBUGMSG(("Found preferences for application [%s] (this is [%s]).\n",
 							a[1],szThisApp));
-				if (strcmp((const char*)a[1],szThisApp) != 0)
+				if (strcmp(static_cast<const char*>(a[1]),szThisApp) != 0)
 				{
 					UT_DEBUGMSG(("Preferences file does not match this application.\n"));
 					goto InvalidFileError;
 				}
 			}
-			else if (strcmp((const char*)a[0], "ver") == 0)
+			else if (strcmp(static_cast<const char*>(a[0]), "ver") == 0)
 			{
 				// TODO test version number
 			}
@@ -677,13 +713,13 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		{
 			UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
 
-			if (strcmp((const char*)a[0], "scheme") == 0)
+			if (strcmp(static_cast<const char*>(a[0]), "scheme") == 0)
 			{
 				FREEP(m_parserState.m_szSelectedSchemeName);
-				if (!UT_cloneString((char *&)m_parserState.m_szSelectedSchemeName,(const char*)a[1]))
+				if (!UT_cloneString((char *&)m_parserState.m_szSelectedSchemeName,static_cast<const char*>(a[1])))
 					goto MemoryError;
 			}
-			else if (strcmp((char*)a[0], "autosaveprefs") == 0)
+			else if (strcmp(static_cast<const char*>(a[0]), "autosaveprefs") == 0)
 			{
 				// m_bAutoSavePrefs controls whether we automatically
 				// save any changes in the preferences during
@@ -695,7 +731,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 				
 				m_bAutoSavePrefs = (*a[1] == '1');
 			}
-			else if (strcmp((const char*)a[0], "useenvlocale") == 0)
+			else if (strcmp(static_cast<const char*>(a[0]), "useenvlocale") == 0)
 			{
 				m_bUseEnvLocale = (*a[1] == '1');
 			}
@@ -732,13 +768,13 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		{
 			UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
 
-			if (strcmp((const char*)a[0], "name") == 0)
+			if (strcmp(static_cast<const char*>(a[0]), "name") == 0)
 			{
 				bIsNamed = true;
 				
 				const XML_Char * szBuiltinSchemeName = getBuiltinSchemeName();
 
-				if (strcmp((const char*)a[1], (const char*)szBuiltinSchemeName) == 0)
+				if (strcmp(static_cast<const char*>(a[1]), static_cast<const char*>(szBuiltinSchemeName)) == 0)
 				{
 					UT_DEBUGMSG(("Reserved scheme name [%s] found in file; ignoring.\n",a[1]));
 					goto IgnoreThisScheme;
@@ -769,6 +805,51 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		pNewScheme = NULL;				// we don't own it anymore
 		break;
 		}
+		case TT_PLUGIN:
+		{
+		// Almost the same as TT_SCHEME, except is denoted by <Plugin ... /> 
+		// instead of <Scheme ... /> and has no builtin to deal with
+
+		bool bIsNamed = false;
+		
+		pNewScheme = new XAP_PrefsScheme(this, NULL);
+		if (!pNewScheme)
+			goto MemoryError;
+		
+		const XML_Char ** a = atts;
+		while (*a)
+		{
+			UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
+
+			if (strcmp(static_cast<const char*>(a[0]), "name") == 0)
+			{
+				bIsNamed = true;
+				
+				if (getPluginScheme(a[1]))
+				{
+					UT_DEBUGMSG(("Duplicate Plugin scheme [%s]; ignoring latter instance.\n",a[1]));
+					goto IgnoreThisScheme;
+				}
+
+				if (!pNewScheme->setSchemeName(a[1]))
+					goto MemoryError;
+
+				UT_DEBUGMSG(("Found Preferences Plugin scheme [%s].\n",a[1]));
+			}
+			else
+			{
+				if (!pNewScheme->setValue(a[0],a[1]))
+					goto MemoryError;
+			}
+
+			a += 2;
+		}
+
+		if (!addPluginScheme(pNewScheme))
+			goto MemoryError;
+		pNewScheme = NULL;				// we don't own it anymore
+		break;
+		}
 		case TT_RECENT:
 		{
 		m_parserState.m_bFoundRecent = true;
@@ -781,11 +862,11 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		{
 			UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
 
-			if (strcmp((const char*)a[0], "max") == 0)
+			if (strcmp(static_cast<const char*>(a[0]), "max") == 0)
 			{
-				m_iMaxRecent = atoi((const char*)a[1]);
+				m_iMaxRecent = atoi(static_cast<const char*>(a[1]));
 			}
-			else if (strncmp((const char*)a[0], "name", 4) == 0)
+			else if (strncmp(static_cast<const char*>(a[0]), "name", 4) == 0)
 			{
 				// NOTE: taking advantage of the fact that XML_Char == char
 				UT_ASSERT((sizeof(XML_Char) == sizeof(char)));
@@ -793,7 +874,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 				UT_XML_cloneString((XML_Char *&)sz, a[1]);
 
 				// NOTE: we keep the copied string in the vector
-				m_vecRecent.addItem((void *)sz);
+				m_vecRecent.addItem(static_cast<void *>(sz));
 			}
 
 			a += 2;
@@ -818,7 +899,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 			// (Both m_geom and the temporary variables are
 			// initialized so partial/invalid preference data from the
 			// file will still have sensible fall-back defaults).
-			UT_uint32 width = 600, height = 800, flags = PREF_FLAG_GEOMETRY_SIZE;
+			UT_uint32 width = 800, height = 600, flags = PREF_FLAG_GEOMETRY_SIZE;
 			UT_sint32 posx = 0, posy = 0;
 
 			m_geom.m_width = width;
@@ -832,25 +913,25 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 			{
 				UT_ASSERT(a[1] && *a[1]);	// require a value for each attribute keyword
 				
-				if (strcmp((const char*)a[0], "width") == 0)
+				if (strcmp(static_cast<const char*>(a[0]), "width") == 0)
 				{
-					width = atoi((const char*)a[1]);
+					width = atoi(static_cast<const char*>(a[1]));
 				}
-				else if (strcmp((const char*)a[0], "height") == 0)
+				else if (strcmp(static_cast<const char*>(a[0]), "height") == 0)
 				{
-					height = atoi((const char*)a[1]);
+					height = atoi(static_cast<const char*>(a[1]));
 				}
-				else if (strcmp((const char*)a[0], "posx") == 0)
+				else if (strcmp(static_cast<const char*>(a[0]), "posx") == 0)
 				{
-					posx = atoi((const char*)a[1]);
+					posx = atoi(static_cast<const char*>(a[1]));
 				}
-				else if (strcmp((const char*)a[0], "posy") == 0)
+				else if (strcmp(static_cast<const char*>(a[0]), "posy") == 0)
 				{
-					posy = atoi((const char*)a[1]);
+					posy = atoi(static_cast<const char*>(a[1]));
 				}
-				else if (strcmp((const char*)a[0], "flags") == 0)
+				else if (strcmp(static_cast<const char*>(a[0]), "flags") == 0)
 				{
-					flags = atoi((const char*)a[1]) & ~PREF_FLAG_GEOMETRY_NOUPDATE;
+					flags = atoi(static_cast<const char*>(a[1])) & ~PREF_FLAG_GEOMETRY_NOUPDATE;
 				}
 				
 				a += 2;
@@ -1057,8 +1138,8 @@ bool XAP_Prefs::savePrefsFile(void)
 					"\t    useenvlocale=\"%d\"\n"
 					"\t/>\n"),
 				m_currentScheme->getSchemeName(),
-				(UT_uint32)m_bAutoSavePrefs,
-				(UT_uint32)m_bUseEnvLocale);
+				static_cast<UT_uint32>(m_bAutoSavePrefs),
+				static_cast<UT_uint32>(m_bUseEnvLocale));
 
 		UT_uint32 kLimit = m_vecSchemes.getItemCount();
 		UT_uint32 k;
@@ -1107,9 +1188,9 @@ bool XAP_Prefs::savePrefsFile(void)
 					// from the builtin set.
 					const XML_Char * szBuiltinValue = NO_PREF_VALUE;
 					m_builtinScheme->getValue(szKey,&szBuiltinValue);
-					if (strcmp((const char*)szValue,(const char*)szBuiltinValue) != 0 ||
+					if (strcmp(static_cast<const char*>(szValue),static_cast<const char*>(szBuiltinValue)) != 0 ||
 						// Always print debug values
-					strncmp((const char*)szKey, (const char*)DEBUG_PREFIX,
+					strncmp(static_cast<const char*>(szKey), static_cast<const char*>(DEBUG_PREFIX),
 						sizeof(DEBUG_PREFIX) - 1) == 0)
 					{
 						need_print = true;
@@ -1117,11 +1198,11 @@ bool XAP_Prefs::savePrefsFile(void)
 				}
 				if (need_print == true)
 				{
-					// szValue is UTF8.  Convert to Unicode and then
+					// szValue is UTF-8.  Convert to Unicode and then
 					// do XML-encoding of XML-special characters and
 					// non-ASCII characters.  The printed value
 					// strings will get XML parsing and conversion to
-					// UTF8 the next time the application reads the
+					// UTF-8 the next time the application reads the
 					// prefs file.
 					UT_GrowBuf gb;
 					UT_decodeUTF8string(szValue, UT_XML_strlen(szValue), &gb);
@@ -1154,8 +1235,61 @@ bool XAP_Prefs::savePrefsFile(void)
 			fprintf(fp,"\t\t/>\n");
 		}
 
+		// add Plugin Scheme (plugin specific preferences) if they exist
+		kLimit = m_vecPluginSchemes.getItemCount();
+		for (k=0; k<kLimit; k++)
+		{
+			XAP_PrefsScheme * p = getNthPluginScheme(k);
+			UT_ASSERT(p);
+
+			const XML_Char * szThisSchemeName = p->getSchemeName();
+			fprintf(fp,"\n\t<Plugin\n\t\tname=\"%s\"\n",szThisSchemeName);
+
+			const XML_Char * szKey;
+			const XML_Char * szValue;
+			UT_uint32 j;
+
+			for (j=0;(p->getNthValue(j, &szKey, &szValue)) ; j++)
+			{
+					// szValue is UTF-8.  Convert to Unicode and then
+					// do XML-encoding of XML-special characters and
+					// non-ASCII characters.  The printed value
+					// strings will get XML parsing and conversion to
+					// UTF-8 the next time the application reads the
+					// prefs file.
+					UT_GrowBuf gb;
+					UT_decodeUTF8string(szValue, UT_XML_strlen(szValue), &gb);
+					UT_uint32 length = gb.getLength();
+					fprintf(fp,"\t\t%s=\"",szKey);
+					for (UT_uint32 udex=0; udex<length; ++udex)
+					{
+						UT_UCSChar ch = *(gb.getPointer(udex));
+						switch (ch)
+						{
+						case '&':   fputs("&amp;", fp);  break;
+						case '<':   fputs("&lt;", fp);  break;
+						case '>':   fputs("&gt;", fp);  break;
+						case '"':   fputs("&quot;", fp);  break;
+						default:
+							if (ch < ' ' || ch >= 128)
+							{
+								fprintf(fp, "&#x%x;", ch);
+							}
+							else
+							{
+								putc(ch, fp);
+							}
+						}
+					}
+					fputs("\"\n", fp);
+			}
+				
+			fprintf(fp,"\t\t/>\n");
+		}
+		// end Plugin preferences
+
 		fprintf(fp,"\n\t<Recent\n\t\tmax=\"%d\"\n",
-				(UT_uint32)m_iMaxRecent);
+				static_cast<UT_uint32>(m_iMaxRecent));
 
 		kLimit = m_vecRecent.getItemCount();
 
@@ -1196,7 +1330,7 @@ void XAP_Prefs::_startElement_SystemDefaultFile(const XML_Char *name, const XML_
 	if (!m_parserState.m_parserStatus)		// eat if already had an error
 		return;
 
-	if (strcmp((const char*)name, "SystemDefaults") == 0)
+	if (strcmp(static_cast<const char*>(name), "SystemDefaults") == 0)
 	{
 		// we found the system default preferences scheme.
 		//
@@ -1217,7 +1351,7 @@ void XAP_Prefs::_startElement_SystemDefaultFile(const XML_Char *name, const XML_
 			// we ignore "name=<schemename>" just incase they copied and
 			// pasted a user-profile into the system file.
 			
-			if (strcmp((const char*)a[0], "name") != 0)
+			if (strcmp(static_cast<const char*>(a[0]), "name") != 0)
 				if (!m_builtinScheme->setValue(a[0],a[1]))
 					goto MemoryError;
 
@@ -1275,7 +1409,7 @@ void XAP_Prefs::addListener	  ( PrefsListener pFunc, void *data )
 	pPair->m_pFunc = pFunc;
 	pPair->m_pData = data;
 
-	m_vecPrefsListeners.addItem( (void *)pPair );
+	m_vecPrefsListeners.addItem( static_cast<void *>(pPair) );
 }
 
 // optional parameter, data.  If given (i.e., != 0), will try to match data,
@@ -1287,7 +1421,7 @@ void XAP_Prefs::removeListener ( PrefsListener pFunc, void *data )
 
 	for ( index = 0; index < m_vecPrefsListeners.getItemCount(); index++ )
 	{
-		pPair = (tPrefsListenersPair *)m_vecPrefsListeners.getNthItem(index);
+		pPair = static_cast<tPrefsListenersPair *>(m_vecPrefsListeners.getNthItem(index));
 		UT_ASSERT(pPair);
 		if ( pPair ) {
 			if ( pPair->m_pFunc == pFunc && (!data || pPair->m_pData == data) ) {
@@ -1305,16 +1439,16 @@ void XAP_Prefs::_markPrefChange( const XML_Char *szKey )
 		const void * uth_e = m_ahashChanges.pick(szKey );
 
 		if ( uth_e ) 
-			uth_e = (const void *)1;
+			uth_e = reinterpret_cast<const void *>(1);
 		else
-			m_ahashChanges.insert(szKey, (void *)1);	
+			m_ahashChanges.insert(szKey, reinterpret_cast<void *>(1));	
 
 		// notify later
 	}
 	else
 	{
 		UT_StringPtrMap	changes(3);
-		changes.insert(szKey, (void *)1);	
+		changes.insert(szKey, reinterpret_cast<void *>(1));	
 
 		_sendPrefsSignal(&changes);
 	}
