@@ -1,20 +1,20 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (c) 2001,2002 Tomas Frydrych
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -22,13 +22,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
+#include "ut_string_class.h"
 #include "ut_vector.h"
 #include "ut_pair.h"
+
 #include "pt_Types.h"
+
 #include "pp_AttrProp.h"
 
 /****************************************************************/
@@ -38,15 +42,17 @@ PP_AttrProp::PP_AttrProp()
 	m_pProperties = NULL;
 	m_bIsReadOnly = false;
 	m_checkSum = 0;
+	xxx_UT_DEBUGMSG(("creating pp_AttrProp %x \n",this));
 }
 
 PP_AttrProp::~PP_AttrProp()
 {
+	xxx_UT_DEBUGMSG(("deleting pp_AttrProp %x \n",this));
 	if (m_pAttributes)
 	{
 		UT_StringPtrMap::UT_Cursor c1(m_pAttributes);
 
-		XML_Char * s = (XML_Char *)c1.first();
+		const XML_Char * s = static_cast<const XML_Char *>(c1.first());
 
 		while (true) {
 			FREEP(s);
@@ -54,7 +60,7 @@ PP_AttrProp::~PP_AttrProp()
 			if (!c1.is_valid())
 				break;
 
-			s = (XML_Char *)c1.next();
+			s = static_cast<const XML_Char *>(c1.next());
 		}
 
 		delete m_pAttributes;
@@ -66,9 +72,9 @@ PP_AttrProp::~PP_AttrProp()
 	if(m_pProperties)
 	{
 		UT_StringPtrMap::UT_Cursor c(m_pProperties);
-		UT_Pair * entry = NULL;
-		for (entry = (UT_Pair*)c.first(); c.is_valid(); 
-		     entry = (UT_Pair*)c.next())
+		const UT_Pair * entry = NULL;
+		for (entry = static_cast<const UT_Pair*>(c.first()); c.is_valid();
+		     entry = static_cast<const UT_Pair*>(c.next()))
 		{
 			if(entry)
 			{
@@ -76,8 +82,7 @@ PP_AttrProp::~PP_AttrProp()
 				void* tmp = const_cast<void*> (entry->first());
 				FREEP(tmp);
 				if (entry->second())
-					delete (PP_PropertyType *)entry->second();
-
+					delete static_cast<PP_PropertyType *const>(entry->second());
 				delete entry;
 			}
 		}
@@ -85,6 +90,7 @@ PP_AttrProp::~PP_AttrProp()
 		delete m_pProperties;
 		m_pProperties = NULL;
 	}
+
 }
 
 /*!
@@ -142,8 +148,8 @@ bool PP_AttrProp::setAttributes(const UT_Vector * pVector)
 	UT_uint32 kLimit = pVector->getItemCount();
 	for (UT_uint32 k=0; k+1<kLimit; k+=2)
 	{
-		const XML_Char * pName = (const XML_Char *)pVector->getNthItem(k);
-		const XML_Char * pValue = (const XML_Char *)pVector->getNthItem(k+1);
+		const XML_Char * pName = static_cast<XML_Char *>(pVector->getNthItem(k));
+		const XML_Char * pValue = static_cast<XML_Char *>(pVector->getNthItem(k+1));
 		if (!setAttribute(pName,pValue))
 			return false;
 	}
@@ -169,10 +175,30 @@ bool	PP_AttrProp::setProperties(const XML_Char ** properties)
 	return true;
 }
 
+/*!
+ * Sets properties as given in the UT_Vector of strings, read as
+ * (property, value) pairs.
+ *
+ * \param pVector A UT_Vector of strings, read in (properties, value) form.
+ */
+bool PP_AttrProp::setProperties(const UT_Vector * pVector)
+{
+	UT_uint32 kLimit = pVector->getItemCount();
+	for (UT_uint32 k=0; k+1<kLimit; k+=2)
+	{
+		const XML_Char * pName = static_cast<XML_Char *>(pVector->getNthItem(k));
+		const XML_Char * pValue = static_cast<XML_Char *>(pVector->getNthItem(k+1));
+		if (!setProperty(pName,pValue))
+			return false;
+	}
+	return true;
+}
+
+
 /*!
  * Sets given attribute in this PP_AttrProp bundle.
  * Deals correctly with setting the PT_PROPS_ATTRIBUTE_NAME property:
- * intercepts this call and appends properties instead. 
+ * intercepts this call and appends properties instead.
  *
  * Because all mutations of attributes go through here, it is always the
  * case that the props attribute is correctly handled.
@@ -181,12 +207,11 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 {
 	// TODO when this assert fails, switch this file to use UT_XML_ version of str*() functions.
 	UT_ASSERT(sizeof(char)==sizeof(XML_Char));
-	
-	if (0 == UT_strcmp(szName, PT_PROPS_ATTRIBUTE_NAME))	// PROPS -- cut value up into properties
-	{
-		if (*szValue == 0) return true; // empty props="" string
 
+	if (0 == UT_strcmp(szName, PT_PROPS_ATTRIBUTE_NAME) && *szValue)	// PROPS -- cut value up into properties
+	{
 		char * pOrig = NULL;
+		
 		if (!UT_cloneString(pOrig,szValue))
 		{
 			UT_DEBUGMSG(("setAttribute: UT_cloneString failed on [%s]\n",szValue));
@@ -220,7 +245,7 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 			// zero-out the colon, thus separating into two strings.
 			*q = 0;
 			q++;
-			
+
 			// now, search ahead for the next semicolon, and separate this property from the next
 			z = q;
 			while (*z && (*z != ';'))
@@ -240,7 +265,7 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 			while (isspace(*q))
 				q++;
 
-			setProperty((const XML_Char*)p, (const XML_Char*)q);
+			setProperty(static_cast<const XML_Char*>(p), static_cast<const XML_Char*>(q));
 		}
 
 		free(pOrig);
@@ -260,6 +285,7 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 
 		// make sure we store attribute names in lowercase
 		UT_ASSERT(sizeof(char) == sizeof(XML_Char));
+
 		char * copy;
 		if (!UT_cloneString(copy, szName))
 		{
@@ -268,12 +294,11 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 		}
 
 		UT_lowerString(copy);
-
 		char * szDupValue = UT_strdup(szValue);
-		
-		if(!m_pAttributes->insert(copy, (void *)szDupValue))
+
+		if(!m_pAttributes->insert(copy, static_cast<void *>(szDupValue)))
 			FREEP(szDupValue);
-		
+
 		FREEP(copy);
 
 		return true;
@@ -292,26 +317,46 @@ bool	PP_AttrProp::setProperty(const XML_Char * szName, const XML_Char * szValue)
 		}
 	}
 
+	// if szValue == NULL or *szValue == 0, we want this property
+	// removed
+	bool bRemove = (!szValue || !*szValue);
+	
 	const void * pEntry = m_pProperties->pick(szName);
 	if (pEntry)
 	{
-		UT_Pair* p = (UT_Pair*) pEntry;
+		const UT_Pair* p = static_cast<const UT_Pair*>(pEntry);
 
 		// hack. don't do it.
 		void* tmp = const_cast<void*> (p->first());
+		UT_ASSERT(!m_bIsReadOnly);
+		if(UT_strcmp(szName,"line-height") == 0)
+		{
+			UT_DEBUGMSG(("Found line-height, Old value %s new value is %s \n",tmp,szValue));
+		}
+
 		FREEP(tmp);
 		if (p->second())
-			delete (PP_PropertyType *)p->second();
-
+			delete static_cast<PP_PropertyType *const>(p->second());
 		delete p;
 
-		m_pProperties->set(szName, 
-				   (void *)new UT_Pair(UT_strdup(szValue), (void *)NULL));
+		if(bRemove)
+		{
+			m_pProperties->remove(szName,NULL);
+		}
+		else
+		{
+			m_pProperties->set(szName,
+							   static_cast<void *>(new UT_Pair(UT_strdup(szValue), static_cast<void *>(NULL))));
+		}
+		
 	}
 	else
 	{
-		m_pProperties->insert(szName, 
-				      (void *)new UT_Pair(UT_strdup(szValue), (void *)NULL));
+		if(!bRemove)
+		{
+			m_pProperties->insert(szName,
+								  static_cast<void *>(new UT_Pair(UT_strdup(szValue), static_cast<void *>(NULL))));
+		}
 	}
 	return true;
 }
@@ -320,7 +365,7 @@ bool	PP_AttrProp::getNthAttribute(int ndx, const XML_Char *& szName, const XML_C
 {
 	if (!m_pAttributes)
 		return false;
-	if ((UT_uint32)ndx >= m_pAttributes->size())
+	if (static_cast<UT_uint32>(ndx) >= m_pAttributes->size())
 		return false;
 
 	int i = 0;
@@ -334,8 +379,8 @@ bool	PP_AttrProp::getNthAttribute(int ndx, const XML_Char *& szName, const XML_C
 
 	if ((i == ndx) && c.is_valid())
 	  {
-	    szName = (XML_Char*) c.key().c_str();
-	    szValue = (XML_Char*) val;
+	    szName = static_cast<const XML_Char*>(c.key().c_str());
+	    szValue = static_cast<const XML_Char*>(val);
 	    return true;
 	  }
 	return false;
@@ -346,9 +391,9 @@ bool	PP_AttrProp::getNthProperty(int ndx, const XML_Char *& szName, const XML_Ch
 	if (!m_pProperties)
 		return false;
 
- 	if ((UT_uint32)ndx >= m_pProperties->size())
+ 	if (static_cast<UT_uint32>(ndx) >= m_pProperties->size())
   		return false;
- 
+
  	int i = 0;
  	UT_StringPtrMap::UT_Cursor c(m_pProperties);
  	const void * val = NULL;
@@ -357,11 +402,11 @@ bool	PP_AttrProp::getNthProperty(int ndx, const XML_Char *& szName, const XML_Ch
  	{
 	  // noop
  	}
- 
+
 	if ( (i == ndx) && c.is_valid())
  		{
-		  szName = (XML_Char*) c.key().c_str();
-		  szValue = (XML_Char*) ((UT_Pair*)val)->first();
+		  szName = static_cast<const XML_Char*>(c.key().c_str());
+		  szValue = static_cast<XML_Char* const>(static_cast<const UT_Pair*>(val)->first());
 		  return true;
  		}
 	return false;
@@ -371,12 +416,12 @@ bool PP_AttrProp::getProperty(const XML_Char * szName, const XML_Char *& szValue
 {
 	if (!m_pProperties)
 		return false;
-	
+
 	const void * pEntry = m_pProperties->pick(szName);
 	if (!pEntry)
 		return false;
 
-	szValue = (XML_Char *) ((UT_Pair*)pEntry)->first();
+	szValue = static_cast<XML_Char *const>(static_cast<const UT_Pair*>(pEntry)->first());
 
 	return true;
 }
@@ -385,36 +430,36 @@ const PP_PropertyType *PP_AttrProp::getPropertyType(const XML_Char * szName, tPr
 {
 	if (!m_pProperties)
 		return NULL;
-	
-	UT_Pair * pEntry = (UT_Pair *)m_pProperties->pick(szName);
+
+	const UT_Pair * pEntry = static_cast<const UT_Pair *>(m_pProperties->pick(szName));
 	if (!pEntry)
 		return NULL;
 
 	if(!pEntry->second())
 	{
 		m_pProperties->set(szName, new UT_Pair
-				   (pEntry->first(), 
-				    PP_PropertyType::createPropertyType(Type, 
-									(XML_Char *)pEntry->first())));
+				   (pEntry->first(),
+				    PP_PropertyType::createPropertyType(Type,
+									static_cast<XML_Char *const>(pEntry->first()))));
 		delete pEntry;
-		pEntry = (UT_Pair *)m_pProperties->pick(szName);
+		pEntry = static_cast<const UT_Pair *>(m_pProperties->pick(szName));
 	}
 
-	return (PP_PropertyType *)pEntry->second();
+	return static_cast<PP_PropertyType *const>(pEntry->second());
 }
 bool PP_AttrProp::getAttribute(const XML_Char * szName, const XML_Char *& szValue) const
 {
 	if (!m_pAttributes)
 		return false;
-	
+
 	const void * pEntry = m_pAttributes->pick(szName);
 	if (!pEntry)
 		return false;
 
-	szValue = (XML_Char *)pEntry;
+	szValue = static_cast<const XML_Char *>(pEntry);
 
 
-	xxx_UT_DEBUGMSG(("SEVIOR: getAttribute Found value %s \n",szValue)); 
+	xxx_UT_DEBUGMSG(("SEVIOR: getAttribute Found value %s \n",szValue));
 
 	return true;
 }
@@ -427,21 +472,30 @@ bool PP_AttrProp::hasProperties(void) const
 	return (m_pProperties->size() > 0);
 }
 
+bool PP_AttrProp::hasAttributes(void) const
+{
+	if (!m_pAttributes)
+		return false;
+
+	return (m_pAttributes->size() > 0);
+}
+
 bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char ** properties) const
 {
 	// return TRUE if each attribute and property is already present
 	// and has the same value as what we have.
 
-	// TODO consider using the fact that we are now (Dec 12 1998) using
-	// TODO alpha-hash-table rather than just a hash-table to optimize
-	// TODO these loops somewhat.
-	
 	if (attributes && *attributes)
 	{
 		const XML_Char ** p = attributes;
 		while (*p)
 		{
-			UT_ASSERT(p[1] && *p[1]);	// require value for each name
+			/*
+			    It seems that we also want empty strings for attributes,
+				at least for the 'param' attribute which goes with fields.
+				Is this bogus too? -PL
+			*/
+			UT_ASSERT(p[1] /* && *p[1]*/);	// require value for each name
 			const XML_Char * szValue = NULL;
 			if (!getAttribute(p[0],szValue))
 				return false;		// item not present
@@ -457,7 +511,7 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 		while (*p)
 		{
 			/*
-				Jeff, I weakened the following assert because we 
+				Jeff, I weakened the following assert because we
 				*want* to represent no tabstops as an empty string.
 				If this isn't safe, let me know.   -- PCR
 			*/
@@ -551,21 +605,21 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 
 		const void * v1 = ca1.first();
 		const void * v2 = ca2.first();
-	
+
 		do
 		{
-			XML_Char *l1 = (XML_Char *)ca1.key().c_str();
-			XML_Char *l2 = (XML_Char *)ca2.key().c_str();
-	
+			const XML_Char *l1 = static_cast<const XML_Char *>(ca1.key().c_str());
+			const XML_Char *l2 = static_cast<const XML_Char *>(ca2.key().c_str());
+
 			if (UT_XML_stricmp(l1, l2) != 0)
 				return false;
-			
-			l1 = (XML_Char *)v1;
-			l2 = (XML_Char *)v2;
-	
+
+			l1 = static_cast<const XML_Char *>(v1);
+			l2 = static_cast<const XML_Char *>(v2);
+
 			if (UT_XML_stricmp(l1,l2) != 0)
 				return false;
-	
+
 			v1 = ca1.next();
 			v2 = ca2.next();
 		} while (ca1.is_valid());
@@ -575,28 +629,28 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 	{
 		UT_StringPtrMap::UT_Cursor cp1(m_pProperties);
 		UT_StringPtrMap::UT_Cursor cp2(pMatch->m_pProperties);
-	
+
 		const void * v1 = cp1.first();
 		const void * v2 = cp2.first();
-	
+
 		do
 		{
-			XML_Char *l1 = (XML_Char *)cp1.key().c_str();
-			XML_Char *l2 = (XML_Char *)cp2.key().c_str();
-	
+			const XML_Char *l1 = static_cast<const XML_Char *>(cp1.key().c_str());
+			const XML_Char *l2 = static_cast<const XML_Char *>(cp2.key().c_str());
+
 			if (UT_XML_stricmp(l1, l2) != 0)
 				return false;
-			
-			l1 = (XML_Char *) ((UT_Pair*)v1)->first();;
-			l2 = (XML_Char *) ((UT_Pair*)v2)->first();;
-	
+
+			l1 = (XML_Char *) (static_cast<const UT_Pair*>(v1))->first();;
+			l2 = (XML_Char *) (static_cast<const UT_Pair*>(v2))->first();;
+
 			if (UT_XML_stricmp(l1,l2) != 0)
 				return false;
-	
+
 			v1 = cp1.next();
 			v2 = cp2.next();
 		} while (cp1.is_valid());
-	
+
 	#ifdef PT_TEST
 		s_Matches++;
 	#endif
@@ -609,6 +663,8 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 												 const XML_Char ** properties,
 												 bool bClearProps) const
 {
+	bool bIgnoreProps = false; // see below
+	
 	// create a new AttrProp based upon the given one
 	// and adding or replacing the items given.
 	// return NULL on failure.
@@ -620,7 +676,7 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 		goto Failed;
 	if (!papNew->setAttributes(attributes) || !papNew->setProperties(properties))
 		goto Failed;
-	
+
 	// next, add any items that we have that are not present
 	// (have not been overridden) in the new one.
 
@@ -628,7 +684,7 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 	const XML_Char * n;
 	const XML_Char * v;
 	const XML_Char * vNew;
-	
+
 	k = 0;
 	while (getNthAttribute(k++,n,v))
 	{
@@ -638,7 +694,7 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 		// TODO properties.  if we allow it to be given here, should
 		// TODO we blowaway all of the existing properties and create
 		// TODO them from this?  or should we expand it and override
-		// TODO individual properties?  
+		// TODO individual properties?
 		// TODO for now, we just barf on it.
 		UT_ASSERT(UT_XML_stricmp(n,PT_PROPS_ATTRIBUTE_NAME)!=0); // cannot handle PROPS here
 		if (!papNew->getAttribute(n,vNew))
@@ -646,7 +702,15 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 				goto Failed;
 	}
 
-	if (!bClearProps)
+	// we want to be able to remove all properties by setting the
+	// props attribute to ""; in order for that to work, we need to
+	// skip the following loop if props is set to ""
+	const XML_Char * szValue;
+
+	if(papNew->getAttribute("props", szValue) && !*szValue)
+		bIgnoreProps = true;
+
+	if (!bClearProps && !bIgnoreProps)
 	{
 		k = 0;
 		while (getNthProperty(k++,n,v))
@@ -658,9 +722,10 @@ PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 	}
 
 	// the following will remove all properties set to ""; this allows us
-	// to remove properties with the by setting them to ""
+	// to remove properties by setting them to ""
 	papNew->_clearEmptyProperties();
-	
+	papNew->_clearEmptyAttributes();
+
 	return papNew;
 
 Failed:
@@ -677,22 +742,22 @@ void PP_AttrProp::_clearEmptyProperties()
 		return;
 
 	UT_StringPtrMap::UT_Cursor _hc1(m_pProperties);
-	void * pEntry;
-	
-	for ( pEntry  = (UT_Pair*) _hc1.first(); _hc1.is_valid(); pEntry = (UT_Pair*) _hc1.next() )
+	const void * pEntry;
+
+	for ( pEntry  = static_cast<const UT_Pair*>(_hc1.first()); _hc1.is_valid(); pEntry = static_cast<const UT_Pair*>(_hc1.next()) )
 	{
 		if (pEntry)
 		{
-			UT_Pair* p = (UT_Pair*) pEntry;
+			const UT_Pair* p = static_cast<const UT_Pair*>(pEntry);
 
-			if(*((XML_Char *)p->first()) == 0)
+			if(*(static_cast<XML_Char *const>(p->first())) == 0)
 			{
-				
+
 				void* tmp = const_cast<void*> (p->first());
+				UT_ASSERT(!m_bIsReadOnly);
 				FREEP(tmp);
 				if (p->second())
-					delete (PP_PropertyType *)p->second();
-
+					delete static_cast<PP_PropertyType *const>(p->second());
 				delete p;
 
 				m_pProperties->remove(_hc1.key(),pEntry);
@@ -701,6 +766,25 @@ void PP_AttrProp::_clearEmptyProperties()
 	}
 }
 
+void PP_AttrProp::_clearEmptyAttributes()
+{
+	if(!m_pAttributes)
+		return;
+
+	UT_StringPtrMap::UT_Cursor _hc1(m_pAttributes);
+	const XML_Char * pEntry;
+
+	for ( pEntry  = static_cast<const XML_Char*>(_hc1.first()); _hc1.is_valid(); pEntry = static_cast<const XML_Char*>(_hc1.next()) )
+	{
+		if (pEntry && !*pEntry)
+		{
+			UT_ASSERT(!m_bIsReadOnly);
+			FREEP(pEntry);
+			m_pAttributes->remove(_hc1.key(),pEntry);
+		}
+	}
+}
+
 PP_AttrProp * PP_AttrProp::cloneWithElimination(const XML_Char ** attributes,
 												const XML_Char ** properties) const
 {
@@ -713,11 +797,11 @@ PP_AttrProp * PP_AttrProp::cloneWithElimination(const XML_Char ** attributes,
 	PP_AttrProp * papNew = new PP_AttrProp();
 	if (!papNew)
 		goto Failed;
-	
+
 	UT_uint32 k;
 	const XML_Char * n;
 	const XML_Char * v;
-	
+
 	k = 0;
 	while (getNthAttribute(k++,n,v))
 	{
@@ -737,7 +821,7 @@ PP_AttrProp * PP_AttrProp::cloneWithElimination(const XML_Char ** attributes,
 		}
 
 		// we didn't find it in the given array, add it to the new set.
-		
+
 		if (!papNew->setAttribute(n,v))
 			goto Failed;
 
@@ -763,7 +847,7 @@ PP_AttrProp * PP_AttrProp::cloneWithElimination(const XML_Char ** attributes,
 		}
 
 		// we didn't find it in the given array, add it to the new set.
-		
+
 		if (!papNew->setProperty(n,v))
 			goto Failed;
 
@@ -797,11 +881,11 @@ PP_AttrProp * PP_AttrProp::cloneWithEliminationIfEqual(const XML_Char ** attribu
 	PP_AttrProp * papNew = new PP_AttrProp();
 	if (!papNew)
 		goto Failed;
-	
+
 	UT_uint32 k;
 	const XML_Char * n;
 	const XML_Char * v;
-	
+
 	k = 0;
 	while (getNthAttribute(k++,n,v))
 	{
@@ -821,7 +905,7 @@ PP_AttrProp * PP_AttrProp::cloneWithEliminationIfEqual(const XML_Char ** attribu
 		}
 
 		// we didn't find it in the given array, add it to the new set.
-		
+
 		if (!papNew->setAttribute(n,v))
 			goto Failed;
 
@@ -847,7 +931,7 @@ PP_AttrProp * PP_AttrProp::cloneWithEliminationIfEqual(const XML_Char ** attribu
 		}
 
 		// we didn't find it in the given array, add it to the new set.
-		
+
 		if (!papNew->setProperty(n,v))
 			goto Failed;
 
@@ -876,7 +960,7 @@ void PP_AttrProp::_computeCheckSum(void)
 	if (!m_pAttributes || !m_pProperties)
 		return;
 
-	XML_Char * s1, *s2;
+	const XML_Char * s1, *s2;
 
 	UT_StringPtrMap::UT_Cursor c1(m_pAttributes);
 	UT_StringPtrMap::UT_Cursor c2(m_pProperties);
@@ -885,8 +969,8 @@ void PP_AttrProp::_computeCheckSum(void)
 
 	while (val != NULL)
 	{
-		s1 = (XML_Char *)c1.key().c_str();
-		s2 = (XML_Char *)val;
+		s1 = static_cast<const XML_Char *>(c1.key().c_str());
+		s2 = static_cast<const XML_Char *>(val);
 
 		m_checkSum += UT_XML_strlen(s1);
 		m_checkSum += UT_XML_strlen(s2);
@@ -900,8 +984,8 @@ void PP_AttrProp::_computeCheckSum(void)
 	val = c2.first();
 	while (val != NULL)
 	{
-		s1 = (XML_Char *)c2.key().c_str();
-		s2 = (XML_Char *) ((UT_Pair*)val)->first();
+		s1 = static_cast<const XML_Char *>(c2.key().c_str());
+		s2 = static_cast<XML_Char *const>(static_cast<const UT_Pair*>(val)->first());
 
 		m_checkSum += UT_XML_strlen(s1);
 		m_checkSum += UT_XML_strlen(s2);
@@ -946,6 +1030,6 @@ void PP_AttrProp::operator = (const PP_AttrProp &Other)
 			setProperty(szName, szValue);
 		}
 	}
-		
+
 }
 
