@@ -9,7 +9,6 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -26,18 +25,26 @@
 #endif /* ABI_OPT_JS */
 
 #include <Pt.h>
+#include <Ap.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/dir.h>
+#include <popt.h>
+
 
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "ut_misc.h"
 
+#include "ut_Script.h"
+#include "ut_PerlBindings.h"
+
 #include "xap_Args.h"
+#include "ap_Args.h"
 #include "ap_Convert.h"
 #include "ap_QNXFrame.h"
 #include "ap_QNXApp.h"
@@ -49,9 +56,13 @@
 #include "xap_Dlg_MessageBox.h"
 #include "xap_Dialog_Id.h"
 #include "xap_Menu_ActionSet.h"
+#include "xap_Menu_Layouts.h"
 #include "xap_Toolbar_ActionSet.h"
+#include "xap_ModuleManager.h"
+#include "xap_Module.h"
 #include "xav_View.h"
 
+
 #include "gr_Graphics.h"
 #include "gr_QNXGraphics.h"
 #include "gr_Image.h"
@@ -81,24 +92,15 @@ void signalWrapper(int sig_num);
 /*****************************************************************/
 
 AP_QNXApp::AP_QNXApp(XAP_Args * pArgs, const char * szAppName)
-	: XAP_QNXApp(pArgs,szAppName)
+	: AP_App(pArgs,szAppName)
 {
 	m_prefs = NULL;
 	m_pStringSet = NULL;
 	m_pClipboard = NULL;
-
-	m_bHasSelection = false;
-	m_bSelectionInFlux = false;
-	m_cacheDeferClear = false;
-	m_pViewSelection = NULL;
-	m_pFrameSelection = NULL;
-	m_cacheSelectionView = NULL;
 }
 
 AP_QNXApp::~AP_QNXApp(void)
 {
-	UT_DEBUGMSG(("TODO: Clean up the spell check stuff \n"));
-//	SpellCheckCleanup();
 
 	DELETEP(m_prefs);
 	DELETEP(m_pStringSet);
@@ -165,37 +167,6 @@ bool AP_QNXApp::initialize(void)
 	//////////////////////////////////////////////////////////////////
 	IE_ImpExp_RegisterXP();
 
-#if 0
-	//////////////////////////////////////////////////////////////////
-	// initializes the spell checker.
-	//////////////////////////////////////////////////////////////////
-	
-	{
-		const char * szISpellDirectory = NULL;
-		getPrefsValueDirectory(false,AP_PREF_KEY_SpellDirectory,&szISpellDirectory);
-		UT_ASSERT((szISpellDirectory) && (*szISpellDirectory));
-
-		const char * szSpellCheckWordList = NULL;
-		getPrefsValue(AP_PREF_KEY_SpellCheckWordList,&szSpellCheckWordList);
-		UT_ASSERT((szSpellCheckWordList) && (*szSpellCheckWordList));
-		
-		char * szPathname = (char *)UT_calloc(sizeof(char),strlen(szISpellDirectory)+strlen(szSpellCheckWordList)+2);
-		UT_ASSERT(szPathname);
-		
-		sprintf(szPathname,"%s%s%s",
-				szISpellDirectory,
-				((szISpellDirectory[strlen(szISpellDirectory)-1]=='/') ? "" : "/"),
-				szSpellCheckWordList);
-
-		UT_DEBUGMSG(("TODO: Clean up the spell check stuff \n"));
-//		UT_DEBUGMSG(("Loading SpellCheckWordList [%s]\n",szPathname));
-//		SpellCheckInit(szPathname);
-		free(szPathname);
-		
-		// we silently go on if we cannot load it....
-	}
-#endif
-	
 	//////////////////////////////////////////////////////////////////
 	// load the dialog and message box strings
 	//////////////////////////////////////////////////////////////////
@@ -255,18 +226,44 @@ bool AP_QNXApp::initialize(void)
 	
 	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
 	{
-	    (&fp_FieldTypes[i])->m_Desc = m_pStringSet->getValue(fp_FieldTypes[i].m_DescId);
+			(&fp_FieldTypes[i])->m_Desc = strdup(m_pStringSet->getValueUTF8(fp_FieldTypes[i].m_DescId).c_str());
 	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
 	}
 
 	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
 	{
-	    (&fp_FieldFmts[i])->m_Desc = m_pStringSet->getValue(fp_FieldFmts[i].m_DescId);
+			(&fp_FieldFmts[i])->m_Desc = strdup(m_pStringSet->getValueUTF8(fp_FieldFmts[i].m_DescId).c_str());
 	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
 	}
 
 	//////////////////////////////////////////////////////////////////
 
+    ///////////////////////////////////////////////////////////////////////
+    /// Build a labelset so the plugins can add themselves to something ///
+    ///////////////////////////////////////////////////////////////////////
+
+	const char * szMenuLabelSetName = NULL;
+	if (getPrefsValue( AP_PREF_KEY_StringSet, (const XML_Char**)&szMenuLabelSetName) && (szMenuLabelSetName) && (*szMenuLabelSetName))
+	{
+	}
+	else
+		szMenuLabelSetName = AP_PREF_DEFAULT_StringSet;
+
+	getMenuFactory()->buildMenuLabelSet(szMenuLabelSetName);
+
+	bool bLoadPlugins = true;
+	bool bFound = getPrefsValueBool(XAP_PREF_KEY_AutoLoadPlugins,&bLoadPlugins);
+	if(bLoadPlugins || !bFound)
+	{
+		loadAllPlugins();
+	}
+
+#ifdef ABI_OPT_PERL
+    // hack to keep the perl bindings working on unix
+    UT_ScriptLibrary& instance = UT_ScriptLibrary::instance(); 
+    instance.registerScript(new UT_PerlScriptSniffer());
+#endif
+
 	return true;
 }
 
@@ -326,7 +323,6 @@ const char * AP_QNXApp::getAbiSuiteAppDir(void) const
 	// we return a static string, use it quickly.
 	
 	static XML_Char buf[1024];
-	UT_ASSERT((strlen(getAbiSuiteLibDir()) + strlen(ABIWORD_APP_LIBDIR) + 2) < sizeof(buf));
 
 	sprintf(buf,"%s/%s",getAbiSuiteLibDir(),ABIWORD_APP_LIBDIR);
 	return buf;
@@ -342,7 +338,7 @@ const XAP_StringSet * AP_QNXApp::getStringSet(void) const
 #define Ph_CLIPBOARD_RTF "RTF"
 #define Ph_CLIPBOARD_TEXT "TEXT"
 
-void AP_QNXApp::copyToClipboard(PD_DocumentRange * pDocRange)
+void AP_QNXApp::copyToClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard)
 {
 	UT_ByteBuf rtfbuf, txtbuf;
 
@@ -383,14 +379,14 @@ void AP_QNXApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipbo
 	UT_uint32 iLen=0;
 
 	if (bHonorFormatting && m_pClipboard->getClipboardData(Ph_CLIPBOARD_RTF,(void**)&pData,&iLen)) {
-		iLen = MyMin(iLen,strlen((const char *) pData));
+		iLen = UT_MIN(iLen,strlen((const char *) pData));
 		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in RTF format.\n",iLen));
 		IE_Imp_RTF * pImpRTF = new IE_Imp_RTF(pDocRange->m_pDoc);
 		pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpRTF);
 	}
 	else if (m_pClipboard->getClipboardData(Ph_CLIPBOARD_TEXT,(void**)&pData,&iLen)) {
-		iLen = MyMin(iLen,strlen((const char *) pData));
+		iLen = UT_MIN(iLen,strlen((const char *) pData));
 		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in TEXTPLAIN format.\n",iLen));
 		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc);
 		pImpText->pasteFromBuffer(pDocRange,pData,iLen);
@@ -412,252 +408,6 @@ bool AP_QNXApp::canPasteFromClipboard(void)
 /*****************************************************************/
 /*****************************************************************/
 
-void AP_QNXApp::setSelectionStatus(AV_View * pView)
-{
-#if 0
-	// this is called by the view-listeners when the state
-	// of the X Selection is changed by the user on one of
-	// our windows.
-	//
-	// we need to notify the clipboard so that it can assert
-	// or release the X Selection.
-	//
-	// we remember the last view that called us so that
-	// clearSelection() can do it's job when another application
-	// asserts the X Selection.
-
-	if (m_bSelectionInFlux)
-		return;
-	m_bSelectionInFlux = true;
-
-	bool bSelectionStateInThisView = ( ! pView->isSelectionEmpty() );
-	
-	if (m_pViewSelection && m_pFrameSelection && m_bHasSelection && (pView != m_pViewSelection))
-	{
-		// one window has a selection currently and another window just
-		// asserted one.  we force clear the old one to enforce the X11
-		// style.
-
-		m_pViewSelection->cmdUnselectSelection();
-	}
-
-	// now fill in all of our variables for this window
-	// and notify the XServer that we have/don't have a
-	// selection.  if we were asked to clear the selection
-	// and we match the cached value (because of the warp
-	// effect on a X11 style middle mouse), we don't actually
-	// tell the XServer of the release (so that the paste
-	// that will immediately follow will short circut to us
-	// rather than going to the server).
-
-
-	if (bSelectionStateInThisView)
-	{
-		m_bHasSelection = bSelectionStateInThisView;
-		m_pClipboard->assertSelection();
-	}
-	else if (pView == m_cacheSelectionView)
-	{
-		// if we are going to use the cache, we do not
-		// clear m_bHasSelection now.  rather, we defer
-		// this and the server notification until afterwards.
-		
-		UT_ASSERT(m_bHasSelection);
-		m_cacheDeferClear = true;
-	}
-	else
-	{
-		m_bHasSelection = bSelectionStateInThisView;
-		m_pClipboard->clearData(false,true);
-	}
-	
-	m_pViewSelection = pView;
-	m_pFrameSelection = (XAP_Frame *)pView->getParentData();
-
-	m_bSelectionInFlux = false;
-#else
-	UT_DEBUGMSG(("Hunt me down and kill me!"));
-#endif
-	return;
-}
-
-bool AP_QNXApp::forgetFrame(XAP_Frame * pFrame)
-{
-	// we intercept this so that we can erase our
-	// selection-related variables if necessary.
-	// wouldn't want to hold onto a stale frame or
-	// view pointer of a closed window when the
-	// selection is changed....
-
-	if (m_pFrameSelection && (pFrame==m_pFrameSelection))
-	{
-		m_pClipboard->clearClipboard();
-		m_pFrameSelection = NULL;
-		m_pViewSelection = NULL;
-	}
-	
-	return XAP_App::forgetFrame(pFrame);
-}
-
-void AP_QNXApp::clearSelection(void)
-{
-	// this method goes with setSelectionStatus().
-	//
-	// we are called by the clipboard (thru the callback chain)
-	// in response to another application stealing the X Selection.
-	//
-	// we need to notify the view so that it can clear the screen
-	// as is the custom on X -- only one selection at any time.
-	//
-	// we have to watch out here because when we call up to clear
-	// the selection, the view will notify the view-listeners of
-	// the change, which may cause setSelectionStatus() to get
-	// called and thus update the clipboard -- this could recurse
-	// a while....
-
-	if (m_bSelectionInFlux)
-		return;
-	m_bSelectionInFlux = true;
-	
-	if (m_pViewSelection && m_pFrameSelection && m_bHasSelection)
-	{
-		m_pViewSelection->cmdUnselectSelection();
-		m_bHasSelection = false;
-	}
-	
-	m_bSelectionInFlux = false;
-	return;
-}
-
-void AP_QNXApp::cacheCurrentSelection(AV_View * pView)
-{
-#if 0
-	if (pView)
-	{
-		// remember a temporary copy of the extent of the current
-		// selection in the given view.  this is intended for the
-		// X11 middle mouse trick -- where we need to warp to a
-		// new location and paste the current selection (not the
-		// clipboard) and the act of warping clears the selection.
-
-		// TODO if we ever support multiple view types, we'll have to
-		// TODO change this.
-#if defined(__WATCOMC__)
-		FV_View * pFVView = (FV_View *)(pView);
-#else
-		FV_View * pFVView = static_cast<FV_View *>(pView);
-#endif
-		pFVView->getDocumentRangeOfCurrentSelection(&m_cacheDocumentRangeOfSelection);
-
-		m_cacheSelectionView = pView;
-		UT_DEBUGMSG(("Clipboard::cacheCurrentSelection: [view %p][range %d %d]\n",
-					 pFVView,
-					 m_cacheDocumentRangeOfSelection.m_pos1,
-					 m_cacheDocumentRangeOfSelection.m_pos2));
-		m_cacheDeferClear = false;
-	}
-	else
-	{
-		if (m_cacheDeferClear)
-		{
-			m_cacheDeferClear = false;
-			m_bHasSelection = false;
-			m_pClipboard->clearData(false,true);
-		}
-		m_cacheSelectionView = NULL;
-	}
-
-	return;
-#else
-	UT_DEBUGMSG(("Hunt me down and kill me!"));
-#endif
-}
-
-bool AP_QNXApp::getCurrentSelection(const char** formatList,
-										void ** ppData, UT_uint32 * pLen,
-										const char **pszFormatFound)
-{
-	// get the current contents of the selection in the
-	// window last known to have a selection using one
-	// of the formats in the given list.
-
-	int j;
-	
-	*ppData = NULL;				// assume failure
-	*pLen = 0;
-	*pszFormatFound = NULL;
-	
-	if (!m_pViewSelection || !m_pFrameSelection || !m_bHasSelection)
-		return false;		// can't do it, give up.
-
-	PD_DocumentRange dr;
-
-	if (m_cacheSelectionView == m_pViewSelection)
-	{
-		dr = m_cacheDocumentRangeOfSelection;
-		UT_DEBUGMSG(("Clipboard::getCurrentSelection: *using cached values* [range %d %d]\n",dr.m_pos1,dr.m_pos2));
-	}
-	else
-	{
-		// TODO if we ever support multiple view types, we'll have to
-		// TODO change this.
-#if defined(__WATCOMC__)
-		FV_View * pFVView = (FV_View *)(m_pViewSelection);
-#else
-		FV_View * pFVView = static_cast<FV_View *>(m_pViewSelection);
-#endif
-	
-		pFVView->getDocumentRangeOfCurrentSelection(&dr);
-		UT_DEBUGMSG(("Clipboard::getCurrentSelection: [view %p][range %d %d]\n",pFVView,dr.m_pos1,dr.m_pos2));
-	}
-	
-	m_selectionByteBuf.truncate(0);
-
-	for (j=0; (formatList[j]); j++)
-	{
-		UT_DEBUGMSG(("Clipboard::getCurrentSelection: considering format [%s]\n",formatList[j]));
-
-		if (UT_stricmp(formatList[j],AP_CLIPBOARD_RTF) == 0)
-		{
-			IE_Exp_RTF * pExpRtf = new IE_Exp_RTF(dr.m_pDoc);
-			if (!pExpRtf)
-				return false;		// give up on memory errors
-
-			pExpRtf->copyToBuffer(&dr,&m_selectionByteBuf);
-			DELETEP(pExpRtf);
-			goto ReturnThisBuffer;
-		}
-			
-		if (   (UT_stricmp(formatList[j],AP_CLIPBOARD_TEXTPLAIN_8BIT) == 0)
-			|| (UT_stricmp(formatList[j],AP_CLIPBOARD_STRING) == 0))
-		{
-			IE_Exp_Text * pExpText = new IE_Exp_Text(dr.m_pDoc);
-			if (!pExpText)
-				return false;
-
-			pExpText->copyToBuffer(&dr,&m_selectionByteBuf);
-			DELETEP(pExpText);
-			goto ReturnThisBuffer;
-		}
-
-		// TODO add other formats as necessary
-	}
-
-	UT_DEBUGMSG(("Clipboard::getCurrentSelection: cannot create anything in one of requested formats.\n"));
-	return false;
-
-ReturnThisBuffer:
-	UT_DEBUGMSG(("Clipboard::getCurrentSelection: copying %d bytes in format [%s].\n",
-				 m_selectionByteBuf.getLength(),formatList[j]));
-	*ppData = (void *)m_selectionByteBuf.getPointer(0);
-	*pLen = m_selectionByteBuf.getLength();
-	*pszFormatFound = formatList[j];
-	return true;
-}
-
-/*****************************************************************/
-/*****************************************************************/
-
 static void * wSplash = NULL;
 static GR_Image * pSplashImage = NULL;
 static GR_QNXGraphics * pQNXGraphics = NULL;
@@ -676,6 +426,7 @@ static int s_hideSplash(PtWidget_t *w, void *data, PtCallbackInfo_t *info)
 }
 
 static int s_drawingarea_expose(PtWidget_t *widget, PhTile_t *damage) {
+
 	if (pQNXGraphics && pSplashImage) {
 		pQNXGraphics->drawImage(pSplashImage, 0, 0);
 
@@ -741,7 +492,6 @@ static GR_Image * _showSplash(PtWidget_t *spwin, UT_uint32 delay)
 		n = 0;
 		PtSetArg(&args[n++], Pt_ARG_WIDTH, iSplashWidth, 0);
 		PtSetArg(&args[n++], Pt_ARG_HEIGHT, iSplashHeight, 0);
-		//PtSetArg(&args[n++], Pt_ARG_USER_DATA, &data, sizeof(this)); 
 		PtSetArg(&args[n++], Pt_ARG_RAW_DRAW_F, &s_drawingarea_expose, 1);
 		PtWidget_t *da = PtCreateWidget(PtRaw, spwin, n, args);
 		PtAddEventHandler(da, Ph_EV_BUT_RELEASE, s_hideSplash, NULL);
@@ -749,7 +499,7 @@ static GR_Image * _showSplash(PtWidget_t *spwin, UT_uint32 delay)
 		// create image context
 		// TODO: find an XAP_App pointer for the following call:
 		pQNXGraphics = new GR_QNXGraphics(spwin, da, 0);
-		pSplashImage = pQNXGraphics->createNewImage("splash", pBB, iSplashWidth, iSplashHeight);
+		pSplashImage = pQNXGraphics->createNewImage("splash", pBB, pQNXGraphics->tlu(iSplashWidth), pQNXGraphics->tlu(iSplashHeight));
 
 		PtRealizeWidget(spwin);
 	}
@@ -776,55 +526,71 @@ int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
 
 	// initialize our application.
 
-	XAP_Args Args = XAP_Args(argc,(char**)argv);
-
-	//Initial state
-	bool bShowSplash = true;
-	bool bShowApp = true;
-
-	// Do a quick and dirty find for "-to"
-	for (int k = 1; k < Args.m_argc; k++)
- 		if (*Args.m_argv[k] == '-')
- 			if (UT_stricmp(Args.m_argv[k],"-to") == 0)
- 			{
-				bShowApp = false;
- 				bShowSplash = false;
- 				break;
- 			}
-
-	// Do a quick and dirty find for "-show"
- 	for (int k = 1; k < Args.m_argc; k++)
- 		if (*Args.m_argv[k] == '-')
- 			if (UT_stricmp(Args.m_argv[k],"-show") == 0)
- 			{
-				bShowApp = true;
- 				bShowSplash = true;
- 				break;
- 			}
-
-	// Do a quick and dirty find for "-nosplash"
-	for (int k = 1; k < Args.m_argc; k++)
-		if (*Args.m_argv[k] == '-')
-			if (UT_stricmp(Args.m_argv[k],"-nosplash") == 0)
-			{
-				bShowSplash = false;
-				break;
-			}
+	XAP_Args XArgs = XAP_Args(argc,argv);
 
 
 	//TODO: Do a PtAppInit() here with the main window being the splash screen
 	PtWidget_t *spwin;
-	spwin = PtAppInit(NULL, NULL /* Args.m_argc */, NULL /* Args.m_argv */, 0, NULL);
+	spwin = PtAppInit(NULL, NULL /* XArgs.m_argc */, NULL /* XArgs.m_argv */, 0, NULL);
+
+	/* Add all PhAB widgets we'll use */
+	ApAddClass("PtWindow",&PtWindow);
+	ApAddClass("PtGroup",&PtGroup);
+	ApAddClass("PtLabel",&PtLabel);
+	ApAddClass("PtButton",&PtButton);
+	ApAddClass("PtPane",&PtPane);
+	ApAddClass("PtPanelGroup",&PtPanelGroup);
+	ApAddClass("PtNumericFloat",&PtNumericFloat);
+	ApAddClass("PtNumericInteger",&PtNumericInteger);
+	ApAddClass("PtToggleButton",&PtToggleButton);
+	ApAddClass("PtMultiText",&PtMultiText);
+	ApAddClass("PtList",&PtList);
+	ApAddClass("PtText",&PtText);
+	ApAddClass("PtTree",&PtTree);
+	ApAddClass("PtComboBox",&PtComboBox);
+	ApAddClass("PtDivider",&PtDivider);
+	ApAddClass("PtRaw",&PtRaw);
+
+	AP_QNXApp * pMyQNXApp = new AP_QNXApp(&XArgs, szAppName);
+	AP_Args Args = AP_Args(&XArgs,szAppName,pMyQNXApp);
+	Args.parsePoptOpts();
+
+	// if the initialize fails, we don't have icons, fonts, etc.
+	if (!pMyQNXApp->initialize())
+	{
+		delete pMyQNXApp;
+		return -1;	// make this something standard?
+	}
+
+	//This is used by all the timer classes, and should probably be in the XAP contructor
+	PtArg_t args[2];
+	PtSetArg(&args[0], Pt_ARG_REGION_FIELDS, Ph_REGION_EV_SENSE, Ph_REGION_EV_SENSE);
+	PtSetArg(&args[1], Pt_ARG_REGION_SENSE, Ph_EV_TIMER, Ph_EV_TIMER);
+	PtSetParentWidget(NULL);
+	gTimerWidget = PtCreateWidget(PtRegion, NULL, 2, args);
+	PtRealizeWidget(gTimerWidget);
+
+	
+ // do we show the app&splash?
+ bool bShowSplash = Args.getShowSplash();
+
+ const XAP_Prefs * pPrefs = pMyQNXApp->getPrefs();
+ UT_ASSERT(pPrefs);
+ bool bSplashPref = true;
+ if (pPrefs && 
+	pPrefs->getPrefsValueBool (AP_PREF_KEY_ShowSplash, &bSplashPref))
+  {
+  	bShowSplash = bShowSplash && bSplashPref;
+  }
 	if (bShowSplash) {
 		_showSplash(spwin, 2000);
 	}
 	else {
 		PtDestroyWidget(spwin);
-	}
-	
-	AP_QNXApp * pMyQNXApp;
-	gQNXApp = pMyQNXApp = new AP_QNXApp(&Args, szAppName);
 
+  if (!Args.doWindowlessArgs())
+    return false;
+	
     // Setup signal handlers, primarily for segfault
     // If we segfaulted before here, we *really* blew it
 
@@ -844,227 +610,28 @@ int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
     sigaction(SIGFPE, &sa, NULL);
     // TODO: handle SIGABRT
 
-	//This is used by all the timer classes, and should probably be in the XAP contructor
-	PtArg_t args[2];
-	PtSetArg(&args[0], Pt_ARG_REGION_FIELDS, Ph_REGION_EV_SENSE, Ph_REGION_EV_SENSE);
-	PtSetArg(&args[1], Pt_ARG_REGION_SENSE, Ph_EV_TIMER, Ph_EV_TIMER);
-	PtSetParentWidget(NULL);
-	gTimerWidget = PtCreateWidget(PtRegion, NULL, 2, args);
-	PtRealizeWidget(gTimerWidget);
-
-	// if the initialize fails, we don't have icons, fonts, etc.
-	if (!pMyQNXApp->initialize())
-	{
-		delete pMyQNXApp;
-		return -1;	// make this something standard?
 	}
-
 	// this function takes care of all the command line args.
 	// if some args are botched, it returns false and we should
 	// continue out the door.
-	if (pMyQNXApp->parseCommandLine() && bShowApp)
+	// We used to check for bShowApp here.  It shouldn't be needed
+	// anymore, because doWindowlessArgs was supposed to bail already. -PL
+	if (pMyQNXApp->openCmdLineFiles(&Args))
 	{
 		PtMainLoop();
 	}
+	else
+	{
+		UT_DEBUGMSG(("Not parsing command line or showing app\n"));
+	}
 	
 	// destroy the App.  It should take care of deleting all frames.
 	pMyQNXApp->shutdown();
 	delete pMyQNXApp;
-	
+
 	return 0;
 }
 
-bool AP_QNXApp::parseCommandLine(void)
-{
-	// parse the command line
-	// <app> [-script <scriptname>]* [-dumpstrings] [<documentname>]*
-        
-	// TODO when we refactor the App classes, consider moving
-	// TODO this to app-specific, cross-platform.
-
-	// TODO replace this with getopt or something similar.
-        
-	// QNX puts the program name in argv[0], so [1] is the first argument.
-
-	int nFirstArg = 1;
-	int k;
-	int kWindowsOpened = 0;
-	char *to = NULL;
-	int verbose = 1;
-	bool show = false;
-        
-	for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
-	{
-		if (*m_pArgs->m_argv[k] == '-')
-		{
-			if (UT_stricmp(m_pArgs->m_argv[k],"-script") == 0)
-			{
-				// [-script scriptname]
-				k++;
-			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-lib") == 0)
-			{
-				// [-lib <AbiSuiteLibDirectory>]
-				// we've already processed this when we initialized the App class
-				k++;
-			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-dumpstrings") == 0)
-			{
-				// [-dumpstrings]
-#ifdef DEBUG
-				// dump the string table in english as a template for translators.
-				// see abi/docs/AbiSource_Localization.abw for details.
-				AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
-				pBuiltinStringSet->dumpBuiltinSet("en-US.strings");
-				delete pBuiltinStringSet;
-#endif
-			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-nosplash") == 0)
-			{
-				// we've alrady processed this before we initialized the App class
-			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-geometry") == 0)
-			{
-				// [-geometry [WIDTHxHEIGHT][+-xoffset+-yoffset]
-				k++;
-				parseAndSetGeometry(m_pArgs->m_argv[k]);	
-			}
-			else if (UT_stricmp (m_pArgs->m_argv[k],"-to") == 0)
-			{
-				k++;
-				to = m_pArgs->m_argv[k];
-			}
-			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0)
-			{
-				show = true;
-			}
-			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0)
-			{
-				k++;
-				verbose = atoi (m_pArgs->m_argv[k]);
-			}
-
-			else
-			{
-				UT_DEBUGMSG(("Unknown command line option [%s]\n",m_pArgs->m_argv[k]));
-				// TODO don't know if it has a following argument or not -- assume not
-
-				_printUsage();
-				return false;
-			}
-		}
-		else
-		{
-			// [filename]
-			if (to) {
- 				AP_Convert * conv = new AP_Convert(getApp());
- 				conv->setVerbose(verbose);
- 				conv->convertTo(m_pArgs->m_argv[k], to);
- 				delete conv;
-  			} 
-			else {
-                        
-				AP_QNXFrame * pFirstQNXFrame = new AP_QNXFrame(this);
-				pFirstQNXFrame->initialize();
-				UT_Error error = pFirstQNXFrame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown, true);
-				if (!error)
-				{
-					kWindowsOpened++;
-				}
-				else
-				{
-					// TODO: warn user that we couldn't open that file
-
-#if 1
-					// TODO we crash if we just delete this without putting something
-					// TODO in it, so let's go ahead and open an untitled document
-					// TODO for now.  this would cause us to get 2 untitled documents
-					// TODO if the user gave us 2 bogus pathnames....
-					kWindowsOpened++;
-					pFirstQNXFrame->loadDocument(NULL, IEFT_Unknown);
-
-					pFirstQNXFrame->raise();
-					
-					XAP_DialogFactory * pDialogFactory
-						= (XAP_DialogFactory *)(pFirstQNXFrame->getDialogFactory());
-					
-					XAP_Dialog_MessageBox * pDialog
-						= (XAP_Dialog_MessageBox *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_MESSAGE_BOX));
-					UT_ASSERT(pDialog);
-					
-					const XAP_StringSet * pSS = pFirstQNXFrame->getApp()->getStringSet();
-					
-					switch (error)
-					{
-					case -301:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FileNotFound),m_pArgs->m_argv[k]);
-						break;
-						
-					case -302:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_NoMemory),m_pArgs->m_argv[k]);
-						break;
-						
-					case -303:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnknownType),m_pArgs->m_argv[k]);
-						break;
-						
-					case -304:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_BogusDocument),m_pArgs->m_argv[k]);
-						break;
-						
-					case -305:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotOpen),m_pArgs->m_argv[k]);
-						break;
-							
-					case -306:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotWrite),m_pArgs->m_argv[k]);
-						break;
-						
-					case -307:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FakeType),m_pArgs->m_argv[k]);
-						break;
-						
-					case -311:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnsupportedType),m_pArgs->m_argv[k]);
-						break;
-						
-					default:
-						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_ImportError),m_pArgs->m_argv[k]);
-					}
-					pDialog->setButtons(XAP_Dialog_MessageBox::b_O);
-					pDialog->setDefaultAnswer(XAP_Dialog_MessageBox::a_OK);
-					
-					pDialog->runModal(pFirstQNXFrame);
-					
-					//XAP_Dialog_MessageBox::tAnswer ans = pDialog->getAnswer();
-					pDialog->getAnswer();
-					
-					pDialogFactory->releaseDialog(pDialog);
-#else
-					delete pFirstQNXFrame;
-#endif
-				}
-			}
-		}
-	}
-
-	// command-line conversion may not open any windows at all
-	if (to && !show) 
-	{
-		return true;
-	}
-
-	if (kWindowsOpened == 0)
-	{
-		// no documents specified or were able to be opened, open an untitled one
-
-		AP_QNXFrame * pFirstQNXFrame = new AP_QNXFrame(this);
-		pFirstQNXFrame->initialize();
-		pFirstQNXFrame->loadDocument(NULL, IEFT_Unknown);
-	}
-	return true;
-}
-
 /*** Signal handling functionality ***/
 void signalWrapper(int sig_num)
 {
@@ -1104,3 +671,119 @@ void AP_QNXApp::catchSignals(int sig_num)
 	abort();
 }
 
+/* Taken from unix july 1'th.
+ * Updated similarly July 20th (fjf)
+ */
+static int so_only (struct dirent *d)
+{
+	const char * name = d->d_name;
+	
+	if ( name )
+	{
+		int len = strlen (name);
+		
+		if (len >= 3)
+		{
+			if(!strcmp(name+(len-3), ".so"))
+    return 1;
+		}
+	}
+	return 0;
+}
+
+void AP_QNXApp::loadAllPlugins ()
+{
+  struct direct **namelist;
+  int n = 0;
+
+  UT_String pluginList[2];
+  UT_String pluginDir;
+
+  // the global plugin directory
+  pluginDir = getAbiSuiteAppDir();
+  pluginDir += "/plugins/";
+  pluginList[0] = pluginDir;
+
+  // the user-local plugin directory
+  pluginDir = getUserPrivateDirectory ();
+  pluginDir += "/AbiWord/plugins/";
+  pluginList[1] = pluginDir;
+
+  for(UT_uint32 i = 0; i < (sizeof(pluginList)/sizeof(pluginList[0])); i++)
+  {
+      pluginDir = pluginList[i];
+
+      n = scandir((char*)pluginDir.c_str(), &namelist, so_only, alphasort);
+      UT_DEBUGMSG(("DOM: found %d plugins in %s\n", n, pluginDir.c_str()));
+
+      if (n > 0)
+	  {
+		  while(n--) 
+		  {
+			  UT_String plugin (pluginDir + namelist[n]->d_name);
+
+			  UT_DEBUGMSG(("DOM: loading plugin %s\n", plugin.c_str()));
+
+			  int len = strlen (namelist[n]->d_name);
+			  if (len < 4)
+			  {
+				  UT_DEBUGMSG(("FJF: bad name for a plugin\n"));
+				  free(namelist[n]);
+				  continue;
+			  }
+			  if(strcmp (namelist[n]->d_name+(len-3), ".so") != 0)
+			  {
+				  UT_DEBUGMSG(("FJF: not really a plugin?\n"));
+				  free(namelist[n]);
+				  continue;
+			  }
+
+			  if (XAP_ModuleManager::instance().loadModule (plugin.c_str()))
+			  {
+				  UT_DEBUGMSG(("DOM: loaded plugin: %s\n", namelist[n]->d_name));
+			  }
+			  else
+			  {
+				  UT_DEBUGMSG(("DOM: didn't load plugin: %s\n", namelist[n]->d_name));
+			  }
+			  free(namelist[n]);
+		  }
+		  free(namelist);
+      }
+  }
+
+  /* SPI modules don't register automatically on loading, so
+   * now that we've loaded the modules we need to register them:
+   */
+  XAP_ModuleManager::instance().registerPending ();
+}
+
+void AP_QNXApp::errorMsgBadArg(AP_Args *Args,int nextopt)
+{
+  printf ("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n",
+	  poptBadOption (Args->poptcon, 0),
+	  poptStrerror (nextopt),
+	  Args->XArgs->m_argv[0]);
+}
+
+void AP_QNXApp::errorMsgBadFile(XAP_Frame * pFrame, const char * file, 
+				 UT_Error error)
+{
+}
+
+bool AP_QNXApp::doWindowlessArgs(const AP_Args *Args)
+{
+
+return false;
+}
+
+XAP_Frame * AP_QNXApp::newFrame(AP_App * app)
+{
+  AP_QNXFrame * pFrame = new AP_QNXFrame(app);
+  if (pFrame)
+    pFrame->initialize();
+  
+  return pFrame;
+}
+
+
