@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 2000 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -24,7 +24,8 @@
 
 #include "xap_App.h"
 #include "xap_QNXApp.h"
-#include "xap_QNXFrame.h"
+#include "xap_QNXFrameImpl.h"
+#include "xap_Frame.h"
 
 #include "ap_Strings.h"
 #include "ap_Dialog_Id.h"
@@ -56,21 +57,13 @@ AP_QNXDialog_Lists::~AP_QNXDialog_Lists(void)
 
 /**********************************************************************/
 
-static int s_customChanged(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
-{
-	AP_QNXDialog_Lists * dlg = (AP_QNXDialog_Lists *)data;
-	dlg->setDirty();
-	dlg->customChanged();
-	return Pt_CONTINUE;
-}
-
 static int s_somethingChanged(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
 {
 	AP_QNXDialog_Lists * dlg = (AP_QNXDialog_Lists *)data;
 	dlg->setDirty();
 	dlg->setXPFromLocal();
 	dlg->previewExposed();
-	
+
 	return Pt_CONTINUE;
 }
 
@@ -89,7 +82,7 @@ static int s_styleChanged (PtWidget_t *widget, void *data, PtCallbackInfo_t *inf
   	dlg->setDirty();
 	dlg->setXPFromLocal();
 	dlg->previewExposed();
-	
+
 	return Pt_CONTINUE;
 }
 
@@ -139,20 +132,16 @@ static int s_deleteClicked (PtWidget_t *widget, void *data, PtCallbackInfo_t *in
 	return Pt_CONTINUE;
 }
 
-static int s_preview_exposed(PtWidget_t * w, PhTile_t * damage) 
+static int s_preview_exposed(PtWidget_t * w, PhTile_t * damage)
 {
-	PtArg_t args[1];
-
    	PhRect_t rect;
    	PtSuperClassDraw(PtBasic, w, damage);
    	PtBasicWidgetCanvas(w, &rect);
 	//clip to our basic canvas (it's only polite).
     PtClipAdd( w, &rect );
 
-	AP_QNXDialog_Lists *pQNXDlg, **ppQNXDlg = NULL;
-	PtSetArg(&args[0], Pt_ARG_USER_DATA, &ppQNXDlg, 0);
-	PtGetResources(w, 1, args);
-	pQNXDlg = (ppQNXDlg) ? *ppQNXDlg : NULL;
+	AP_QNXDialog_Lists *pQNXDlg;
+	PtGetResource(w, Pt_ARG_POINTER, &pQNXDlg,0);
 
 	UT_ASSERT(pQNXDlg);
 	pQNXDlg->previewExposed();
@@ -161,14 +150,6 @@ static int s_preview_exposed(PtWidget_t * w, PhTile_t * damage)
 	return Pt_CONTINUE;
 }
 
-static int s_update (void)
-{
-	printf("TODO: Static update \n");
-//	Current_Dialog->updateDialog();
-	return true;
-}
-
-
 /**********************************************************************/
 
 void AP_QNXDialog_Lists::activate()
@@ -188,9 +169,9 @@ void AP_QNXDialog_Lists::destroy()
 	m_bDestroy_says_stopupdating = true;
 	while (m_bAutoUpdate_happening_now == true) ;
 	m_pAutoUpdateLists->stop();
-	setAnswer(AP_Dialog_Lists::a_CLOSE);	
+	setAnswer(AP_Dialog_Lists::a_CLOSE);
 	modeless_cleanup();
-	
+
 	PtWidget_t *tmp = m_mainWindow;
 	m_mainWindow = NULL;
 	PtDestroyWidget(tmp);
@@ -213,7 +194,7 @@ void AP_QNXDialog_Lists::runModeless(XAP_Frame * pFrame)
 	// Save dialog the ID number and pointer to the widget
 	UT_sint32 sid = (UT_sint32) getDialogId ();
 	m_pApp->rememberModelessId(sid, (XAP_Dialog_Modeless *) m_pDialog);
- 
+
 	// This magic command displays the frame that characters will be
 	// inserted into.
     // This variation runs the additional static function shown afterwards.
@@ -287,7 +268,7 @@ void AP_QNXDialog_Lists::previewExposed(void)
 		setbisCustomized(true);
 		event_PreviewAreaExposed();
 	}
-} 
+}
 
 
 /*
@@ -395,7 +376,7 @@ void AP_QNXDialog_Lists::updateDialog(void)
 
 #if 0
 void AP_UnixDialog_Lists::setAllSensitivity(void)
-{ 
+{
 	PopulateDialogData();
 	if(m_isListAtPoint == true)
 	{
@@ -405,230 +386,100 @@ void AP_UnixDialog_Lists::setAllSensitivity(void)
 
 PtWidget_t * AP_QNXDialog_Lists::_constructWindow (void)
 {
-	PtWidget_t *vgroup;
 	PtWidget_t *lblStyle, *listStyle;
 	PtWidget_t *lblType, *listType;
+	PtWidget_t  *lblFormat;
 	PtWidget_t *togCustomize, *grpCustomize;
-	PtWidget_t *lblFormat, *lblFormat2;
 	PtWidget_t *numListLevel, *numListAlign, *numIndentAlign, *numStart;
 	PtWidget_t *radnewlist, *radexisting, *radsublist;
 	PtWidget_t *butOK, *butCancel;
 
-   	PtArg_t    	args[10];
-	int			n;
-
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
-	n = 0;
 	ConstructWindowName();
-	PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, getWindowName(), 0);
-	PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, 0, ABI_MODAL_WINDOW_RENDER_FLAGS);
-	PtSetArg(&args[n++], Pt_ARG_WINDOW_MANAGED_FLAGS, 0, ABI_MODAL_WINDOW_MANAGE_FLAGS);
-	m_mainWindow = PtCreateWidget(PtWindow, NULL, n, args);
+	m_mainWindow = abiCreatePhabDialog("ap_QNXDialog_Lists",(char*)getWindowName()); 
+
+	SetupContextHelp(m_mainWindow,this);
+	PtAddHotkeyHandler(m_mainWindow,Pk_F1,0,Pt_HOTKEY_SYM,this,OpenHelp);
 	PtAddCallback(m_mainWindow, Pt_CB_WINDOW_CLOSING, s_deleteClicked, this);
 
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, Pt_GROUP_STRETCH_HORIZONTAL, Pt_GROUP_STRETCH_HORIZONTAL);
-	PtSetArg(&args[n++], Pt_ARG_MARGIN_WIDTH, ABI_MODAL_MARGIN_SIZE, 0);
-	PtSetArg(&args[n++], Pt_ARG_MARGIN_HEIGHT, ABI_MODAL_MARGIN_SIZE, 0);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING_Y, 5, 0);
-   	vgroup = PtCreateWidget(PtGroup, m_mainWindow, n, args);
-
-	PtWidget_t *hgroup;
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING, 5, 0);
-	hgroup = PtCreateWidget(PtGroup, vgroup, n, args);
-
-	/*** Create the controls in a vertical group here ***/
-	PtWidget_t *ctlgroup;
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING, 10, 0);
-	ctlgroup = PtCreateWidget(PtGroup, hgroup, n, args);
-
-	PtWidget_t *group;
-	n = 0;
-	group = PtCreateWidget(PtGroup, ctlgroup, n, args);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Type), 0);
-	lblType = PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, 2*ABI_DEFAULT_BUTTON_WIDTH, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_FLAGS, 0, Pt_EDITABLE);
-	listType = PtCreateWidget(PtComboBox, group, n, args);	
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblType"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Type), 0);
+	
+	listType = abiPhabLocateWidget(m_mainWindow,"comboType");
 	PtAddCallback(listType, Pt_CB_SELECTION, s_typeChanged, this);
 
 	const char *text;
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Type_none);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).c_str();
 	PtListAddItems(listType, &text, 1, 0);
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Type_bullet);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_bullet).c_str();
 	PtListAddItems(listType, &text, 1, 0);
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Type_numbered);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered).c_str();
 	PtListAddItems(listType, &text, 1, 0);
 
-	n = 0;
-	group = PtCreateWidget(PtGroup, ctlgroup, n, args);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Style), 0);
-	lblStyle = PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, 2*ABI_DEFAULT_BUTTON_WIDTH, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_FLAGS, 0, Pt_EDITABLE);
-	listStyle = PtCreateWidget(PtComboBox, group, n, args);	
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblStyle"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Style), 0);
+
+	listStyle = abiPhabLocateWidget(m_mainWindow,"comboStyle"); 
 	PtAddCallback(listStyle, Pt_CB_SELECTION, s_styleChanged, this);
 
-#if 0
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Customize), 0);
-	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_OUTLINE, 0);
-	PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_SET);
-	togCustomize = PtCreateWidget(PtToggleButton, ctlgroup, n, args);	
-	PtAddCallback(togCustomize, Pt_CB_ACTIVATE, s_customChanged, this);
-	m_bisCustomFrameHidden = true;
-#endif
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"grpCustomLists"),Pt_ARG_TITLE,_(AP,DLG_Lists_Customize ),0);
 
-	n = 0;
-#define OUTLINE_GROUP (Pt_TOP_OUTLINE | Pt_TOP_BEVEL | \
-                                           Pt_BOTTOM_OUTLINE | Pt_BOTTOM_BEVEL | \
-                                           Pt_LEFT_OUTLINE | Pt_LEFT_BEVEL | \
-                                           Pt_RIGHT_OUTLINE | Pt_RIGHT_BEVEL)
-	PtSetArg(&args[n++], Pt_ARG_BASIC_FLAGS, OUTLINE_GROUP, OUTLINE_GROUP);
-	PtSetArg(&args[n++], Pt_ARG_BEVEL_WIDTH, 1, 0);
-/*
-	PtSetArg(&args[n++], Pt_ARG_FLAGS, Pt_DELAY_REALIZE | Pt_HIGHLIGHTED, Pt_DELAY_REALIZE | Pt_HIGHLIGHTED);
-*/
-	grpCustomize = PtCreateWidget(PtGroup, ctlgroup, n, args);
-	pretty_group(grpCustomize, pSS->getValue(AP_STRING_ID_DLG_Lists_Customize));
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblFormat"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Format), 0);
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblFont"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Font), 0);
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblLevelDelimiter"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_DelimiterString), 0);
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblStartAt"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start), 0);
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblTextAlign"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Align), 0);
+	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblLabelAlign"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Indent), 0);
 
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING, 5, 0);
-	group = PtCreateWidget(PtGroup, grpCustomize, n, args);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Format), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Font), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_DelimiterString), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Start), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Align), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Indent), 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, Pt_GROUP_EQUAL_SIZE_HORIZONTAL, Pt_GROUP_EQUAL_SIZE_HORIZONTAL);
-	group = PtCreateWidget(PtGroup, grpCustomize, n, args);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, 1.5*ABI_DEFAULT_BUTTON_WIDTH, 0);
-	lblFormat = PtCreateWidget(PtText, group, n, args);	
+	lblFormat = abiPhabLocateWidget(m_mainWindow,"textFormat"); 
 	PtAddCallback(lblFormat, Pt_CB_ACTIVATE, s_somethingChanged, this);
-#if 0
-	/* This is for something ... I just don't know what */
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, 1.5*ABI_DEFAULT_BUTTON_WIDTH, 0);
-	lblFormat2 = PtCreateWidget(PtText, group, n, args);	
-#endif
 
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_FLAGS, 0, Pt_EDITABLE);
-	PtWidget_t *font = PtCreateWidget(PtComboBox, group, n, args);	
-	//TODO: Fill this with the current fonts 
+	PtWidget_t *font = abiPhabLocateWidget(m_mainWindow,"comboFonts"); 
+	//TODO: Fill this with the current fonts
 	{
 		text = "Current Font";
 		PtListAddItems(font, &text, 1, 0);
 	}
 	UT_QNXComboSetPos(font, 1);
-	
-	double d;
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_MIN, 0, 0);
-	numListLevel = PtCreateWidget(PtNumericInteger, group, n, args);	
+
+	numListLevel = abiPhabLocateWidget(m_mainWindow,"NumericLevelDelimiter"); 
 	PtAddCallback(numListLevel, Pt_CB_NUMERIC_CHANGED, s_somethingChanged, this);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_MIN, 0, 0);
-	numStart = PtCreateWidget(PtNumericInteger, group, n, args);	
+
+	numStart = abiPhabLocateWidget(m_mainWindow,"NumericStartAt");
 	PtAddCallback(numStart, Pt_CB_NUMERIC_CHANGED, s_somethingChanged, this);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_SUFFIX, "in", 0);
-	d = 0.1;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_INCREMENT, &d, 0);
-	numListAlign = PtCreateWidget(PtNumericFloat, group, n, args);	
+	
+	numListAlign = abiPhabLocateWidget(m_mainWindow,"NumericTextAlign"); 
 	PtAddCallback(numListAlign, Pt_CB_NUMERIC_CHANGED, s_somethingChanged, this);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_SUFFIX, "in", 0);
-	d = 0.1;
-	PtSetArg(&args[n++], Pt_ARG_NUMERIC_INCREMENT, &d, 0);
-	numIndentAlign = PtCreateWidget(PtNumericFloat, group, n, args);	
+	
+	numIndentAlign =abiPhabLocateWidget(m_mainWindow,"NumericLabelAlign"); 
 	PtAddCallback(numIndentAlign, Pt_CB_NUMERIC_CHANGED, s_somethingChanged, this);
 
-	/*** Create the preview in the next dialog ***/
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH,  200, 0);
-	PtSetArg(&args[n++], Pt_ARG_HEIGHT,  300, 0);
-	m_wPreviewGroup = PtCreateWidget(PtGroup, hgroup, n, args);
-	pretty_group(m_wPreviewGroup, "Preview");
+	 PtSetResource(abiPhabLocateWidget(m_mainWindow,"grpPreview"),Pt_ARG_TITLE,_(AP,DLG_Lists_Preview),0);
 
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH,  200, 0);
-	PtSetArg(&args[n++], Pt_ARG_HEIGHT,  300, 0);
-	void *data = (void *)this;
-	PtSetArg(&args[n++], Pt_ARG_USER_DATA, &data, sizeof(this)); 
-	PtSetArg(&args[n++], Pt_ARG_RAW_DRAW_F, &s_preview_exposed, 1); 
-	m_wPreviewArea = PtCreateWidget(PtRaw, m_wPreviewGroup, n, args);
+	m_wPreviewArea = abiPhabLocateWidget(m_mainWindow,"rawPreview"); 
+	PtSetResource(m_wPreviewArea, Pt_ARG_POINTER, this, 0);
+	PtSetResource(m_wPreviewArea, Pt_ARG_RAW_DRAW_F, &s_preview_exposed, 1);
 
-	/*** Create the radio buttons below this group ***/
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, Pt_GROUP_EXCLUSIVE, Pt_GROUP_EXCLUSIVE);
-	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING, 5, 0);
-	group = PtCreateWidget(PtGroup, vgroup, n, args);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Start_New), 0);
-	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
-	PtSetArg(&args[n++], Pt_ARG_FLAGS, Pt_SET, Pt_SET);
-	radnewlist = PtCreateWidget(PtToggleButton, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Apply_Current), 0);
-	radexisting = PtCreateWidget(PtToggleButton, group, n, args);	
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Start_Sub), 0);
-	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
-	radsublist = PtCreateWidget(PtToggleButton, group, n, args);	
-		
-	/*** Then we have the final cancellation buttons ***/
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_GROUP_HORZ_ALIGN, Pt_GROUP_HORZ_RIGHT, 0);
-	group = PtCreateWidget(PtGroup, vgroup, n, args);
-/*
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, 200, 0);
-	PtCreateWidget(PtLabel, group, n, args);	
-*/
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue (XAP_STRING_ID_DLG_Apply), 0);
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	butOK = PtCreateWidget(PtButton, group, n, args);	
+	radnewlist = abiPhabLocateWidget(m_mainWindow,"toggleNew"); 
+	PtSetResource(radnewlist, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start_New), 0);
+
+	radexisting = abiPhabLocateWidget(m_mainWindow,"toggleCurrent");
+	PtSetResource(radexisting, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Apply_Current), 0);
+
+	radsublist = abiPhabLocateWidget(m_mainWindow,"togglePrevious"); 
+	PtSetResource(radsublist, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start_Sub), 0);
+
+	butOK = abiPhabLocateWidget(m_mainWindow,"btnApply"); 
+	PtSetResource(butOK, Pt_ARG_TEXT_STRING, pSS->getValue (XAP_STRING_ID_DLG_Apply), 0);
 	PtAddCallback(butOK, Pt_CB_ACTIVATE, s_applyClicked, this);
-	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue (XAP_STRING_ID_DLG_Close), 0);
-	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	butCancel = PtCreateWidget(PtButton, group, n, args);	
+
+
+	butCancel = abiPhabLocateWidget(m_mainWindow,"btnClose");
+	PtSetResource(butCancel, Pt_ARG_TEXT_STRING, pSS->getValue (XAP_STRING_ID_DLG_Close), 0);
 	PtAddCallback(butCancel, Pt_CB_ACTIVATE, s_closeClicked, this);
 
 	/** Done **/
 	m_wDelimEntry = lblFormat;
-	m_wDecimalEntry = lblFormat2;
+//	m_wDecimalEntry = lblFormat2;
 
 	m_wCustomFrame = grpCustomize;
 	m_wCustomLabel = togCustomize;
@@ -660,7 +511,7 @@ void AP_QNXDialog_Lists::_fillNoneStyleMenu( PtWidget_t *listmenu)
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Type_none);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)NOT_A_LIST);
 }
@@ -673,25 +524,31 @@ void AP_QNXDialog_Lists::_fillNumberedStyleMenu( PtWidget_t *listmenu)
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Numbered_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Numbered_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)NUMBERED_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Lower_Case_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Case_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)LOWERCASE_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Upper_Case_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Case_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)UPPERCASE_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Lower_Roman_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Roman_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)LOWERROMAN_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Upper_Roman_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Roman_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)UPPERROMAN_LIST);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Arabic_List).c_str();
+	PtListAddItems(listmenu, &text, 1, 0);
+	m_styleVector.addItem((void *)ARABICNUMBERED_LIST);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hebrew_List).c_str();
+	PtListAddItems(listmenu, &text, 1, 0);
+	m_styleVector.addItem((void *)HEBREW_LIST);
 }
 
 void AP_QNXDialog_Lists::_fillBulletedStyleMenu( PtWidget_t *listmenu)
@@ -702,63 +559,59 @@ void AP_QNXDialog_Lists::_fillBulletedStyleMenu( PtWidget_t *listmenu)
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Bullet_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Bullet_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)BULLETED_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Dashed_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Dashed_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)DASHED_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Square_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Square_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)SQUARE_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Triangle_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Triangle_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)TRIANGLE_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Diamond_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Diamond_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)DIAMOND_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Star_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Star_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)STAR_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Implies_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Implies_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)IMPLIES_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Tick_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Tick_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)TICK_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Box_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Box_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)BOX_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Hand_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hand_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)HAND_LIST);
 
-	text = pSS->getValue(AP_STRING_ID_DLG_Lists_Heart_List);
+text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Heart_List).c_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)HEART_LIST);
 }
 
-void AP_QNXDialog_Lists::_setRadioButtonLabels(void) 
+void AP_QNXDialog_Lists::_setRadioButtonLabels(void)
 {
 	//	char *tmp;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	PopulateDialogData();
 	// Button 0 is Start New List, button 2 is resume list
-	PtSetResource(m_wStartNewList, Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Start_New), 0);
-	PtSetResource(m_wStartSubList, Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_Lists_Resume), 0);
-/*
-	gtk_label_set_text( GTK_LABEL(m_wStartNew_label), pSS->getValue(AP_STRING_ID_DLG_Lists_Start_New));
-	gtk_label_set_text( GTK_LABEL(m_wStartSub_label), pSS->getValue(AP_STRING_ID_DLG_Lists_Resume));
-*/
+PtSetResource(m_wStartNewList, Pt_ARG_TEXT_STRING, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New).c_str(), 0);
+PtSetResource(m_wStartSubList, Pt_ARG_TEXT_STRING, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume).c_str(), 0);
 }
 
 //
@@ -767,13 +620,13 @@ void AP_QNXDialog_Lists::_setRadioButtonLabels(void)
 //
 void AP_QNXDialog_Lists::_loadXPDataIntoLocal(void)
 {
-	int i;
+	unsigned int i;
 	double d;
 
 	m_bDontUpdate = true;
 
 	UT_DEBUGMSG(("_loadXP newListType = %d \n", getNewListType()));
-	
+
 	//PtSetResource(m_wLevelSpin, Pt_ARG_NUMERIC_VALUE, m_iLevel, 0);
 
 	d = getfAlign();
@@ -785,34 +638,9 @@ void AP_QNXDialog_Lists::_loadXPDataIntoLocal(void)
 		d = 0.0;
 		PtSetResource(m_wIndentAlignSpin, Pt_ARG_NUMERIC_VALUE, &d, 0);
 	} else {
-		PtSetResource(m_wIndentAlignSpin, Pt_ARG_NUMERIC_VALUE, &d, 0); 
+		PtSetResource(m_wIndentAlignSpin, Pt_ARG_NUMERIC_VALUE, &d, 0);
 	}
 
-	//
-	// Code to work out which is active Font
-	//
-#if 0
-	if(strcmp((char *) m_pszFont,"NULL") == 0 )
-	{
-		gtk_option_menu_set_history (GTK_OPTION_MENU (m_wFontOptions), 0 );
-	}
-	else
-	{
-		for(i=0; i < (gint) g_list_length(m_glFonts);i++)
-		{
-			if(strcmp((char *) m_pszFont,(char *) g_list_nth_data(m_glFonts,i)) == 0)
-				break;
-		}
-		if(i < (gint) g_list_length(m_glFonts))
-		{
-			gtk_option_menu_set_history (GTK_OPTION_MENU (m_wFontOptions), i+ 1 );
-		}
-		else
-		{
-			gtk_option_menu_set_history (GTK_OPTION_MENU (m_wFontOptions), 0 );
-		}
-	}
-#endif
 
 	PtSetResource(m_wStartSpin, Pt_ARG_NUMERIC_VALUE, getiStartValue(), 0);
 
@@ -886,21 +714,7 @@ void AP_QNXDialog_Lists::_gatherData(void)
 		PtSetResource(m_wIndentAlignSpin, Pt_ARG_NUMERIC_VALUE, &indent, sizeof(indent));
 	}
 
-#if 0
-	PtWidget_t * wfont = gtk_menu_get_active(GTK_MENU(m_wFontOptions_menu));
-	gint ifont =  GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wfont)));
-	if(ifont == 0)
-	{
-                 UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *)  "NULL");
-	}
-	else
-	{
-                 UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *)  g_list_nth_data(m_glFonts, ifont-1));
-	}
-#else
-	//UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *)  "NULL");
 	copyCharToFont( "NULL");
-#endif
 
 
 	PtGetResource(m_wStartSpin, Pt_ARG_NUMERIC_VALUE, &idata, 0);
