@@ -17,10 +17,13 @@
  * 02111-1307, USA.
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#undef GDK_DISABLE_DEPRECATED
+
 #include "ut_assert.h"
 #include "xap_App.h"
 #include "xap_UnixApp.h"
-#include "xap_UnixFrame.h"
+#include "xap_Frame.h"
 #include "ap_Strings.h"
 #include "xap_UnixDialogHelper.h"
 
@@ -51,7 +54,7 @@ create_pixmap (GtkWidget *w, char **data)
 
   colormap = gtk_widget_get_colormap (w);
   gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
-						     NULL, (gchar **)data);
+						     NULL, static_cast<gchar **>(data));
 
   pixmap = gtk_pixmap_new (gdkpixmap, mask);
   gdk_pixmap_unref (gdkpixmap);
@@ -66,7 +69,7 @@ _ev_convert (char * bufResult,
 	UT_ASSERT (szString && bufResult);
 	
 	char *pl = bufResult;
-	char *s = (char *)szString;
+	const char *s = static_cast<const char *>(szString);
 
 	int len = strlen (szString);
 	int i;
@@ -89,12 +92,11 @@ create_spinentry (float v)
 {
   gchar * val;
   GtkAdjustment * SpinAdj = (GtkAdjustment *) 
-        gtk_adjustment_new( 10.0, 1.0, 5000.0, 0.1, 1.0, 0.0);
+        gtk_adjustment_new( v, 1.0, 5000.0, 0.1, 1.0, 0.0);
   GtkWidget * e = gtk_spin_button_new (SpinAdj, 1.0, 2);
   val = g_strdup_printf (FMT_STRING, v);
   gtk_entry_set_text (GTK_ENTRY (e), val);
   gtk_entry_set_editable (GTK_ENTRY (e), TRUE);
-  gtk_widget_set_usize (e, gdk_string_measure (e->style->font, val) + 15, 0);
   g_free (val);
 
   return e;
@@ -124,7 +126,7 @@ static char _ev_buf[256];
 #ifdef _
 #undef _
 #endif
-#define _(a, x) _ev_convert (_ev_buf, pSS->getValue (a##_STRING_ID_##x))
+#define _(a, x) _ev_convert (_ev_buf, pSS->getValueUTF8 (a##_STRING_ID_##x).c_str())
 
 // string tags to stuff stored in widget data
 #define WIDGET_MENU_OPTION_PTR		"menuoptionptr"
@@ -133,47 +135,27 @@ static char _ev_buf[256];
 // convenience macro
 #define CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(w, m, d, f)				\
         do {												\
-                gtk_object_set_data (GTK_OBJECT (w), WIDGET_MENU_OPTION_PTR, (gpointer)m);                \
-                gtk_object_set_data (GTK_OBJECT (w), WIDGET_MENU_VALUE_TAG,  GINT_TO_POINTER(d));                \
-	        gtk_signal_connect (GTK_OBJECT (w), "activate",	\
-                GTK_SIGNAL_FUNC (f),		\
-                (gpointer)this);							\
+                g_object_set_data (G_OBJECT (w), WIDGET_MENU_OPTION_PTR, static_cast<gpointer>(m));                \
+                g_object_set_data (G_OBJECT (w), WIDGET_MENU_VALUE_TAG,  GINT_TO_POINTER(d));                \
+	        g_signal_connect (G_OBJECT (w), "activate",	\
+                G_CALLBACK (f),		\
+                static_cast<gpointer>(this));							\
         } while (0)
 
 /*********************************************************************************/
 
 // static event callbacks
 
-static void s_ok_clicked (GtkWidget * w, AP_UnixDialog_PageSetup *dlg)
-{
-  UT_ASSERT (dlg);
-  dlg->event_OK ();
-}
-
-static void s_cancel_clicked (GtkWidget * w, AP_UnixDialog_PageSetup *dlg)
-{
-  UT_ASSERT (dlg);
-  dlg->event_Cancel ();
-}
-
-static void s_delete_clicked (GtkWidget * w,
-			      gpointer data,
-			      AP_UnixDialog_PageSetup * dlg)
-{
-  UT_ASSERT (dlg);
-  dlg->event_WindowDelete ();
-}
-
 static void s_menu_item_activate (GtkWidget * widget)
 {
-	GtkWidget *option_menu = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (widget),
-								   WIDGET_MENU_OPTION_PTR);
+	GtkWidget *option_menu = static_cast<GtkWidget *>(g_object_get_data (G_OBJECT (widget),
+								   WIDGET_MENU_OPTION_PTR));
 	UT_ASSERT(option_menu && GTK_IS_OPTION_MENU (option_menu));
 
-	gpointer p = gtk_object_get_data (GTK_OBJECT (widget),
+	gpointer p = g_object_get_data (G_OBJECT (widget),
 					  WIDGET_MENU_VALUE_TAG);
 
-	gtk_object_set_data (GTK_OBJECT (option_menu), WIDGET_MENU_VALUE_TAG, p);
+	g_object_set_data (G_OBJECT (option_menu), WIDGET_MENU_VALUE_TAG, p);
 }
 
 static void s_page_size_changed (GtkWidget * w, GtkWidget * child, 
@@ -216,20 +198,32 @@ static void s_entryPageHeight_changed(GtkWidget * widget, AP_UnixDialog_PageSetu
 
 void AP_UnixDialog_PageSetup::_setWidth(const char * buf)
 {
-	if( atof(buf) >= 0.0 && atof(buf) != m_PageSize.Width(getPageUnits()) )
+	double width = static_cast<double>(atof(buf));
+	double height = static_cast<double>(m_PageSize.Width(getPageUnits()));
+	if(m_PageSize.match(height,10.0))
 	{
-		m_PageSize.Set( (double)atof(buf),
-						(double)m_PageSize.Height(getPageUnits()),
+		return;
+	}
+	if( width >= 0.00001 && !m_PageSize.match(width, static_cast<double>(m_PageSize.Width(getPageUnits()) )))
+	{
+		m_PageSize.Set( width,
+						height,
 						getPageUnits() );
 	}
 }
 
 void AP_UnixDialog_PageSetup::_setHeight(const char * buf)
 {
-	if( atof(buf) >= 0.0 && atof(buf) != m_PageSize.Height(getPageUnits()) )
+	double height = static_cast<double>(atof(buf));
+	if(m_PageSize.match(height,10.0))
 	{
-		m_PageSize.Set( m_PageSize.Width(getPageUnits()),
-						atof(buf),
+		return;
+	}
+	double width = static_cast<double>(m_PageSize.Width(getPageUnits()));
+	if( height >= 0.00001 && !m_PageSize.match(height,static_cast<double>(m_PageSize.Height(getPageUnits()))) )
+	{
+		m_PageSize.Set( width,
+						height,
 						getPageUnits() );
 	}
 }
@@ -240,11 +234,11 @@ void AP_UnixDialog_PageSetup::doWidthEntry(void)
 
 	_setWidth(szAfter);
 
-	gtk_signal_handler_block(GTK_OBJECT(m_entryPageWidth), m_iEntryPageWidthID);
+	g_signal_handler_block(G_OBJECT(m_entryPageWidth), m_iEntryPageWidthID);
 	int pos = gtk_editable_get_position(GTK_EDITABLE(m_entryPageWidth));
 	gtk_entry_set_text( GTK_ENTRY(m_entryPageWidth),szAfter );
-	gtk_entry_set_position(GTK_ENTRY(m_entryPageWidth), pos);
-	gtk_signal_handler_unblock(GTK_OBJECT(m_entryPageWidth),m_iEntryPageWidthID);
+	gtk_editable_set_position(GTK_EDITABLE(m_entryPageWidth), pos);
+	g_signal_handler_unblock(G_OBJECT(m_entryPageWidth),m_iEntryPageWidthID);
 
 	_updatePageSizeList();
 }
@@ -254,11 +248,11 @@ void AP_UnixDialog_PageSetup::doHeightEntry(void)
 	const char * szAfter = gtk_entry_get_text(GTK_ENTRY(m_entryPageHeight));
 	_setHeight(szAfter);
 
-	gtk_signal_handler_block(GTK_OBJECT(m_entryPageHeight), m_iEntryPageHeightID);
+	g_signal_handler_block(G_OBJECT(m_entryPageHeight), m_iEntryPageHeightID);
 	int pos = gtk_editable_get_position(GTK_EDITABLE(m_entryPageHeight));
 	gtk_entry_set_text( GTK_ENTRY(m_entryPageHeight),szAfter );
-	gtk_entry_set_position(GTK_ENTRY(m_entryPageHeight), pos);
-	gtk_signal_handler_unblock(GTK_OBJECT(m_entryPageHeight),m_iEntryPageHeightID);
+	gtk_editable_set_position(GTK_EDITABLE(m_entryPageHeight), pos);
+	g_signal_handler_unblock(G_OBJECT(m_entryPageHeight),m_iEntryPageHeightID);
 
 	_updatePageSizeList();
 }
@@ -266,13 +260,14 @@ void AP_UnixDialog_PageSetup::doHeightEntry(void)
 /* The paper size may have changed, update the Paper Size listbox */
 void AP_UnixDialog_PageSetup::_updatePageSizeList(void)
 {
-  gint last_page_size = (gint)fp_PageSize::NameToPredefined 
-	  (m_PageSize.getPredefinedName ());
+	xxx_UT_DEBUGMSG(("_updatePageSize set to %s \n",m_PageSize.getPredefinedName()));
+  gint last_page_size = static_cast<gint>(fp_PageSize::NameToPredefined 
+	  (m_PageSize.getPredefinedName ()));
 
   GtkList * optionPageSizeList = GTK_LIST(GTK_COMBO(m_optionPageSize)->list);
-  gtk_signal_handler_block(GTK_OBJECT(optionPageSizeList), m_iOptionPageSizeListID);
+  g_signal_handler_block(G_OBJECT(optionPageSizeList), m_iOptionPageSizeListID);
   gtk_list_select_item (optionPageSizeList, last_page_size);
-  gtk_signal_handler_unblock(GTK_OBJECT(optionPageSizeList), m_iOptionPageSizeListID);
+  g_signal_handler_unblock(G_OBJECT(optionPageSizeList), m_iOptionPageSizeListID);
 }
 
 void AP_UnixDialog_PageSetup::event_OK (void)
@@ -288,7 +283,6 @@ void AP_UnixDialog_PageSetup::event_OK (void)
 								 XAP_Dialog_MessageBox::b_O,
 								 XAP_Dialog_MessageBox::a_OK);
 		setAnswer(a_CANCEL);
-		gtk_main_quit();
 		return;
 	}
 	
@@ -298,19 +292,17 @@ void AP_UnixDialog_PageSetup::event_OK (void)
 	setPageOrientation (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (m_radioPagePortrait)) ? PORTRAIT : LANDSCAPE);
 	setPageScale (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (m_spinPageScale)));
 
-	setMarginTop (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginTop)));
-	setMarginBottom (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginBottom)));
-	setMarginLeft (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginLeft)));
-	setMarginRight (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginRight)));
-	setMarginHeader (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginHeader)));
-	setMarginFooter (gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginFooter)));
+	setMarginTop (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginTop)));
+	setMarginBottom (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginBottom)));
+	setMarginLeft (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginLeft)));
+	setMarginRight (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginRight)));
+	setMarginHeader (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginHeader)));
+	setMarginFooter (gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginFooter)));
 
 	// The window will only close (on an OK click) if the margins
 	// fit inside the paper size.
 	if ( validatePageSettings() ) {
-		setAnswer (a_OK);
-		
-		gtk_main_quit();
+		setAnswer (a_OK);		
 	}
 	else {
 		// "The margins selected are too large to fit on the page."
@@ -323,26 +315,20 @@ void AP_UnixDialog_PageSetup::event_OK (void)
 void AP_UnixDialog_PageSetup::event_Cancel (void)
 {
 	setAnswer (a_CANCEL);
-	gtk_main_quit();
-}
-
-void AP_UnixDialog_PageSetup::event_WindowDelete (void)
-{
-        event_Cancel();
 }
 
 void AP_UnixDialog_PageSetup::event_PageUnitsChanged (void)
 {
-  UT_Dimension pu = (UT_Dimension) GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (m_optionPageUnits), 
-										   WIDGET_MENU_VALUE_TAG));
+  UT_Dimension pu = static_cast<UT_Dimension>(GPOINTER_TO_INT (g_object_get_data (G_OBJECT (m_optionPageUnits), 
+										   WIDGET_MENU_VALUE_TAG)));
 
   double width, height;
 
   fp_PageSize ps = m_PageSize;
   
   // convert values  
-  width  = (double)ps.Width (pu);
-  height = (double)ps.Height (pu);
+  width  = static_cast<double>(ps.Width (pu));
+  height = static_cast<double>(ps.Height (pu));
 
   m_PageSize.Set(width, height, pu);
   setPageUnits(pu);
@@ -350,11 +336,11 @@ void AP_UnixDialog_PageSetup::event_PageUnitsChanged (void)
   // set values
   gchar * val;
 
-  val = g_strdup_printf (FMT_STRING, (float)width);
+  val = g_strdup_printf (FMT_STRING, static_cast<float>(width));
   gtk_entry_set_text (GTK_ENTRY (m_entryPageWidth), val);
   g_free (val);
 
-  val = g_strdup_printf (FMT_STRING, (float)height);
+  val = g_strdup_printf (FMT_STRING, static_cast<float>(height));
   gtk_entry_set_text (GTK_ENTRY (m_entryPageHeight), val);
   g_free (val);
 }
@@ -375,7 +361,7 @@ void AP_UnixDialog_PageSetup::event_PageSizeChanged (fp_PageSize::Predefined pd)
   w = ps.Width (new_units);
   h = ps.Height (new_units);
 
-  if (fp_PageSize::psCustom != pd)
+  if (true /* fp_PageSize::psCustom != pd */)
   {
 	  // set entry values for a non-custom pagesize
 	  gchar * val;
@@ -394,25 +380,25 @@ void AP_UnixDialog_PageSetup::event_PageSizeChanged (fp_PageSize::Predefined pd)
   {
 	  ps.Set(atof(gtk_entry_get_text(GTK_ENTRY(m_entryPageWidth))),
 			 atof(gtk_entry_get_text(GTK_ENTRY(m_entryPageHeight))),
-			 (UT_Dimension) GPOINTER_TO_INT (gtk_object_get_data 
-												  (GTK_OBJECT (m_optionPageUnits), 
-						   WIDGET_MENU_VALUE_TAG)));
+			 static_cast<UT_Dimension>(GPOINTER_TO_INT (g_object_get_data 
+												  (G_OBJECT (m_optionPageUnits), 
+						   WIDGET_MENU_VALUE_TAG))));
   }
 }
 
 void AP_UnixDialog_PageSetup::event_MarginUnitsChanged (void)
 {
-  UT_Dimension mu = (UT_Dimension) GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (m_optionMarginUnits),
-										   WIDGET_MENU_VALUE_TAG));
+  UT_Dimension mu = static_cast<UT_Dimension>(GPOINTER_TO_INT (g_object_get_data (G_OBJECT (m_optionMarginUnits),
+										   WIDGET_MENU_VALUE_TAG)));
 
   float top, bottom, left, right, header, footer;
 
-  top    = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginTop));
-  bottom = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginBottom));
-  left   = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginLeft));
-  right  = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginRight));
-  header = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginHeader));
-  footer = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (m_spinMarginFooter));
+  top    = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginTop));
+  bottom = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginBottom));
+  left   = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginLeft));
+  right  = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginRight));
+  header = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginHeader));
+  footer = gtk_spin_button_get_value (GTK_SPIN_BUTTON (m_spinMarginFooter));
 
   top = UT_convertDimensions (top,    last_margin_unit, mu);
   bottom = UT_convertDimensions (bottom, last_margin_unit, mu);
@@ -455,101 +441,59 @@ AP_UnixDialog_PageSetup::~AP_UnixDialog_PageSetup (void)
 
 void AP_UnixDialog_PageSetup::runModal (XAP_Frame *pFrame)
 {
+	UT_return_if_fail(pFrame);
+	
 	// snarf the parent pagesize.
 	m_PageSize = getPageSize();
-
+	m_pFrame = pFrame;
+	
     // Build the window's widgets and arrange them
     GtkWidget * mainWindow = _constructWindow();
-    UT_ASSERT(mainWindow);
+    UT_return_if_fail(mainWindow);
+	m_PageSize = getPageSize();
+	_updatePageSizeList();
 
-    connectFocus(GTK_WIDGET(mainWindow), pFrame);
+	switch(abiRunModalDialog(GTK_DIALOG(mainWindow), pFrame, this,
+							 BUTTON_CANCEL, false))
+	{
+		case BUTTON_OK:
+			event_OK() ; break;
+		default:
+			event_Cancel() ; break ;
+	}
 
-	m_pFrame = pFrame;
-
-    // To center the dialog, we need the frame of its parent.
-    XAP_UnixFrame * pUnixFrame = static_cast<XAP_UnixFrame *>(pFrame);
-    UT_ASSERT(pUnixFrame);
-    
-    // Get the GtkWindow of the parent frame
-    GtkWidget * parentWindow = pUnixFrame->getTopLevelWindow();
-    UT_ASSERT(parentWindow);
-    
-    // Center our new dialog in its parent and make it a transient
-    // so it won't get lost underneath
-    centerDialog(parentWindow, mainWindow);
-
-    // Show the top level dialog,
-    gtk_widget_show(mainWindow);
-
-    // Make it modal, and stick it up top
-    gtk_grab_add(mainWindow);
-
-    // Run into the GTK event loop for this window.
-    gtk_main();
-    if(mainWindow && GTK_IS_WIDGET(mainWindow))
-      gtk_widget_destroy(mainWindow);
+	abiDestroyWidget ( mainWindow ) ;
 }
 
 void AP_UnixDialog_PageSetup::_connectSignals (void)
 {
   	// the control buttons
-	gtk_signal_connect(GTK_OBJECT(m_buttonOK),
-			   "clicked",
-			   GTK_SIGNAL_FUNC(s_ok_clicked),
-			   (gpointer) this);
-	
-	gtk_signal_connect(GTK_OBJECT(m_buttonCancel),
-			   "clicked",
-			   GTK_SIGNAL_FUNC(s_cancel_clicked),
-			   (gpointer) this);
-
- 	m_iEntryPageWidthID = gtk_signal_connect(GTK_OBJECT(m_entryPageWidth),
+ 	m_iEntryPageWidthID = g_signal_connect(G_OBJECT(m_entryPageWidth),
  					   "changed",
- 					  GTK_SIGNAL_FUNC(s_entryPageWidth_changed),
- 					   (gpointer) this);
+ 					  G_CALLBACK(s_entryPageWidth_changed),
+ 					   static_cast<gpointer>(this));
 
- 	m_iEntryPageHeightID = gtk_signal_connect(GTK_OBJECT(m_entryPageHeight),
+ 	m_iEntryPageHeightID = g_signal_connect(G_OBJECT(m_entryPageHeight),
  					   "changed",
- 					  GTK_SIGNAL_FUNC(s_entryPageHeight_changed),
- 					   (gpointer) this);
-
-	// the catch-alls
-	
-	gtk_signal_connect(GTK_OBJECT(m_window),
-			   "delete_event",
-			   GTK_SIGNAL_FUNC(s_delete_clicked),
-			   (gpointer) this);
-
-	gtk_signal_connect_after(GTK_OBJECT(m_window),
-				 "destroy",
-				 NULL,
-				 NULL);
-
+ 					  G_CALLBACK(s_entryPageHeight_changed),
+ 					   static_cast<gpointer>(this));
 }
 
 GtkWidget * AP_UnixDialog_PageSetup::_constructWindow (void)
 {
   const XAP_StringSet * pSS = m_pApp->getStringSet();
 
-  m_window = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (m_window), _(AP, DLG_PageSetup_Title));
+  m_window = abiDialogNew ("page setup dialog", TRUE, _(AP, DLG_PageSetup_Title));
 
   _constructWindowContents (GTK_DIALOG(m_window)->vbox);
 
-  m_buttonOK = gtk_button_new_with_label (_(XAP, DLG_OK));
-  gtk_widget_show (m_buttonOK);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(m_window)->action_area), 
-		     m_buttonOK);
-
-  m_buttonCancel = gtk_button_new_with_label (_(XAP, DLG_Cancel));
-  gtk_widget_show (m_buttonCancel);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(m_window)->action_area), m_buttonCancel);
-
+  abiAddStockButton(GTK_DIALOG(m_window), GTK_STOCK_CANCEL, BUTTON_CANCEL);
+  abiAddStockButton(GTK_DIALOG(m_window), GTK_STOCK_OK, BUTTON_OK);
   _connectSignals ();
+
   return m_window;
 }
 
-// this code is glade generated
 void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
 {
   GtkWidget *notebook;
@@ -614,19 +558,16 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
   gtk_box_pack_start (GTK_BOX (container), notebook, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 7);
 
-  packerPage = gtk_packer_new ();
-  gtk_packer_set_default_border_width (GTK_PACKER (packerPage), 2);
-  gtk_packer_set_default_pad (GTK_PACKER (packerPage), 8, 8);
-  gtk_packer_set_default_ipad (GTK_PACKER (packerPage), 2, 2);
+  packerPage = gtk_vbox_new ( true, 2 ) ;
   gtk_widget_show (packerPage);
   gtk_container_add (GTK_CONTAINER (notebook), packerPage);
   gtk_container_set_border_width (GTK_CONTAINER (packerPage), 2);
 
   framePaper = gtk_frame_new (_(AP, DLG_PageSetup_Paper));
   gtk_widget_show (framePaper);
-  gtk_packer_add_defaults (GTK_PACKER (packerPage), framePaper, GTK_SIDE_TOP,
-                           GTK_ANCHOR_CENTER, (GtkPackerOptions) (GTK_FILL_X));
-
+  gtk_container_add (GTK_CONTAINER (packerPage), framePaper);
+  gtk_frame_set_shadow_type(GTK_FRAME(framePaper), GTK_SHADOW_NONE);
+  
   tablePaper = gtk_table_new (3, 4, TRUE);
   gtk_widget_show (tablePaper);
   gtk_container_add (GTK_CONTAINER (framePaper), tablePaper);
@@ -671,20 +612,12 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
 
   // create the drop-down menu with all of our supported page sizes
   GList *popdown_items = NULL;
-  for (int i = (int)fp_PageSize::_first_predefined_pagesize_; i < (int)fp_PageSize::_last_predefined_pagesize_dont_use_; i++)
-    {
-      popdown_items = g_list_append (popdown_items, (void*)fp_PageSize::PredefinedToName ((fp_PageSize::Predefined)i) );
-    }
+  for (int i = static_cast<int>(fp_PageSize::_first_predefined_pagesize_); i < static_cast<int>(fp_PageSize::_last_predefined_pagesize_dont_use_); i++)
+      popdown_items = g_list_append (popdown_items, const_cast<char *>(fp_PageSize::PredefinedToName ((fp_PageSize::Predefined)i)) );
   gtk_combo_set_popdown_strings (GTK_COMBO (optionPageSize), popdown_items);
-
-  gint last_page_size = (gint)fp_PageSize::NameToPredefined 
-	  (m_PageSize.getPredefinedName ());
-
   GtkList * optionPageSizeList = GTK_LIST(GTK_COMBO(optionPageSize)->list);
-  gtk_list_select_item (optionPageSizeList, last_page_size);
-
-  m_iOptionPageSizeListID = gtk_signal_connect(GTK_OBJECT(optionPageSizeList), "select-child",
-		     GTK_SIGNAL_FUNC(s_page_size_changed), (gpointer)this);
+  m_iOptionPageSizeListID = g_signal_connect(G_OBJECT(optionPageSizeList), "select-child",
+											 G_CALLBACK(s_page_size_changed), static_cast<gpointer>(this));
 
   labelPageUnits = gtk_label_new (_(AP, DLG_PageSetup_Units));
   gtk_widget_show (labelPageUnits);
@@ -703,40 +636,40 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_inch));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionPageUnits, DIM_IN, s_page_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionPageUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionPageUnits_menu), glade_menuitem);
 
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_cm));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionPageUnits, DIM_CM, s_page_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionPageUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionPageUnits_menu), glade_menuitem);
 
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_mm));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionPageUnits, DIM_MM, s_page_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionPageUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionPageUnits_menu), glade_menuitem);
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (optionPageUnits), optionPageUnits_menu);
   gtk_option_menu_set_history (GTK_OPTION_MENU (optionPageUnits), fp_2_pos (getPageUnits()));
 
   frameOrientation = gtk_frame_new (_(AP, DLG_PageSetup_Orient));
   gtk_widget_show (frameOrientation);
-  gtk_packer_add_defaults (GTK_PACKER (packerPage), frameOrientation, GTK_SIDE_TOP,
-                           GTK_ANCHOR_CENTER, (GtkPackerOptions) (GTK_FILL_X));
+  gtk_frame_set_shadow_type(GTK_FRAME(frameOrientation), GTK_SHADOW_NONE);
 
+  gtk_container_add (GTK_CONTAINER(packerPage), frameOrientation) ;
   tableOrientation = gtk_table_new (2, 2, TRUE);
   gtk_widget_show (tableOrientation);
   gtk_container_add (GTK_CONTAINER (frameOrientation), tableOrientation);
   gtk_table_set_row_spacings (GTK_TABLE (tableOrientation), 1);
 
   radioPageLandscape = gtk_radio_button_new_with_label (tableOrientation_group, _(AP, DLG_PageSetup_Landscape));
-  tableOrientation_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radioPageLandscape));
+  tableOrientation_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioPageLandscape));
   gtk_widget_show (radioPageLandscape);
   gtk_table_attach (GTK_TABLE (tableOrientation), radioPageLandscape, 1, 2, 1, 2,
                     (GtkAttachOptions) (GTK_EXPAND),
                     (GtkAttachOptions) (0), 0, 0);
 
   radioPagePortrait = gtk_radio_button_new_with_label (tableOrientation_group, _(AP, DLG_PageSetup_Portrait));
-  tableOrientation_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radioPagePortrait));
+  tableOrientation_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioPagePortrait));
   gtk_widget_show (radioPagePortrait);
   gtk_table_attach (GTK_TABLE (tableOrientation), radioPagePortrait, 0, 1, 1, 2,
                     (GtkAttachOptions) (GTK_EXPAND),
@@ -761,9 +694,9 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
 
   frameScale = gtk_frame_new (_(AP, DLG_PageSetup_Scale));
   gtk_widget_show (frameScale);
-  gtk_packer_add_defaults (GTK_PACKER (packerPage), frameScale, GTK_SIDE_TOP,
-                           GTK_ANCHOR_CENTER, (GtkPackerOptions) (GTK_FILL_X));
+  gtk_frame_set_shadow_type(GTK_FRAME(frameScale), GTK_SHADOW_NONE);
 
+  gtk_container_add (GTK_CONTAINER(packerPage), frameScale) ;
   table1 = gtk_table_new (1, 4, TRUE);
   gtk_widget_show (table1);
   gtk_container_add (GTK_CONTAINER (frameScale), table1);
@@ -785,7 +718,7 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
 
   spinPageScale_adj = gtk_adjustment_new (100, 1, 1000, 1, 25, 25);
   spinPageScale = gtk_spin_button_new (GTK_ADJUSTMENT (spinPageScale_adj), 1, 0);
-  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinPageScale), (float)getPageScale ());
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinPageScale), static_cast<float>(getPageScale ()));
   gtk_widget_show (spinPageScale);
   gtk_table_attach (GTK_TABLE (table1), spinPageScale, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -913,17 +846,17 @@ void AP_UnixDialog_PageSetup::_constructWindowContents (GtkWidget *container)
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_inch));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionMarginUnits, DIM_IN, s_margin_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionMarginUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionMarginUnits_menu), glade_menuitem);
 
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_cm));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionMarginUnits, DIM_CM, s_margin_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionMarginUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionMarginUnits_menu), glade_menuitem);
 
   glade_menuitem = gtk_menu_item_new_with_label (_(XAP, DLG_Unit_mm));
   CONNECT_MENU_ITEM_SIGNAL_ACTIVATE (glade_menuitem, optionMarginUnits, DIM_MM, s_margin_units_changed);
   gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionMarginUnits_menu), glade_menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (optionMarginUnits_menu), glade_menuitem);
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (optionMarginUnits), optionMarginUnits_menu);
   last_margin_unit = getMarginUnits ();
