@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -41,10 +41,11 @@
 #include "ap_FrameData.h"
 
 AP_Dialog_Options::AP_Dialog_Options(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_NonPersistent(pDlgFactory,id),
+	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialogpreferences"),
 	  m_answer(a_OK),
 	  m_pFrame(0),	// needs to be set from runModal for some of the event_'s to work
-	  m_pageNum(-1)
+	  m_pageNum(-1),
+	  m_bInitialPop(false)
 {
 }
 
@@ -57,8 +58,8 @@ AP_Dialog_Options::tAnswer AP_Dialog_Options::getAnswer(void) const
 	return m_answer;
 }
 
-inline void Save_Pref_Bool(  XAP_PrefsScheme *pPrefsScheme, 
-							 XML_Char const * key, 
+inline void Save_Pref_Bool(  XAP_PrefsScheme *pPrefsScheme,
+							 XML_Char const * key,
 							 bool var )
 {
 	XML_Char szBuffer[2] = {0,0};
@@ -114,15 +115,16 @@ void AP_Dialog_Options::_storeWindowData(void)
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_StatusBarVisible, _gatherViewShowStatusBar() );
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_ParaVisible, _gatherViewUnprintable() );
 	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_AllowCustomToolbars, _gatherAllowCustomToolbars() );
+#if defined(XP_UNIX_TARGET_GTK)
+	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_EnableSmoothScrolling, _gatherEnableSmoothScrolling() );
+#endif
 	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_AutoLoadPlugins, _gatherAutoLoadPlugins() );
-#ifdef BIDI_ENABLED
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_DefaultDirectionRtl, _gatherOtherDirectionRtl() );
-	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_UseContextGlyphs, _gatherOtherUseContextGlyphs() );
 	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_SaveContextGlyphs, _gatherOtherSaveContextGlyphs() );
 	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_UseHebrewContextGlyphs, _gatherOtherHebrewContextGlyphs() );
-#endif
-
-#if 1
+	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_ChangeLanguageWithKeyboard, _gatherLanguageWithKeyboard() );
+	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_DirMarkerAfterClosingParenthesis, _gatherDirMarkerAfterClosingParenthesis());
+	
 	// JOAQUIN - fix this: Dom
 	UT_DEBUGMSG(("Saving Auto Save File [%i]\n", _gatherAutoSaveFile()));
 	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_AutoSaveFile, _gatherAutoSaveFile() );
@@ -135,7 +137,16 @@ void AP_Dialog_Options::_storeWindowData(void)
 	_gatherAutoSaveFilePeriod(stVal);
 	UT_DEBUGMSG(("Saving Auto Save File with a period of [%s]\n", stVal.c_str()));
 	pPrefsScheme->setValue(XAP_PREF_KEY_AutoSaveFilePeriod, stVal.c_str());
-#endif
+	
+	// Jordi: win32 specific for now
+	
+	_gatherUILanguage(stVal);
+	if (stVal.length())
+	{
+		UT_DEBUGMSG(("Setting default UI language to [%s]\n", stVal.c_str()));
+		pPrefsScheme->setValue(AP_PREF_KEY_StringSet, stVal.c_str());
+	}
+	
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// If we changed whether the ruler is to be visible
 	// or hidden, then update the current window:
@@ -167,10 +178,10 @@ void AP_Dialog_Options::_storeWindowData(void)
 		m_pFrame->toggleBar(1, pFrameData->m_bShowBar[1]);
 	}
 
-	if (_gatherViewShowExtraBar() != pFrameData->m_bShowBar[2])
+	if (_gatherViewShowExtraBar() != pFrameData->m_bShowBar[3])
 	{
-		pFrameData->m_bShowBar[2] = _gatherViewShowExtraBar();
-		m_pFrame->toggleBar(2, pFrameData->m_bShowBar[2]);
+		pFrameData->m_bShowBar[3] = _gatherViewShowExtraBar();
+		m_pFrame->toggleBar(3, pFrameData->m_bShowBar[3]);
 	}
 
 	if ( _gatherViewUnprintable() != pFrameData->m_bShowPara )
@@ -189,19 +200,17 @@ void AP_Dialog_Options::_storeWindowData(void)
 	{
 		m_pFrame->getApp()->setToolbarsCustomizable(_gatherAllowCustomToolbars());
 	}
-
+#if defined(XP_UNIX_TARGET_GTK)
+	if ( _gatherEnableSmoothScrolling() != m_pFrame->getApp()->isSmoothScrollingEnabled() )
+	{
+		m_pFrame->getApp()->setEnableSmoothScrolling(_gatherEnableSmoothScrolling());
+	}
+#endif
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// save ruler units value
 	pPrefsScheme->setValue((XML_Char*)AP_PREF_KEY_RulerUnits,
 				   (XML_Char*)UT_dimensionName( _gatherViewRulerUnits()) );
 
-	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	// save default paper size
-	UT_ASSERT(sizeof(XML_Char) && sizeof(char));
-	pPrefsScheme->setValue((XML_Char*)XAP_PREF_KEY_DefaultPageSize,
-				   (XML_Char*)fp_PageSize::PredefinedToName( _gatherDefaultPageSize()) );
-
-
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// save screen color
 	UT_ASSERT(sizeof(XML_Char) && sizeof(char));
@@ -254,7 +263,7 @@ void AP_Dialog_Options::_populateWindowData(void)
 	bool			b;
 	XAP_Prefs		*pPrefs = 0;
 	const XML_Char	*pszBuffer = 0;
-
+	m_bInitialPop = true;
 	// TODO: move this logic when we get a PrefsListener API and turn this
 	//		 dialog into an app-specific
 
@@ -278,11 +287,6 @@ void AP_Dialog_Options::_populateWindowData(void)
 	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_SmartQuotesEnable,&b))
 		_setSmartQuotesEnable (b);
 
-	if (pPrefs->getPrefsValue((XML_Char*)XAP_PREF_KEY_DefaultPageSize, &pszBuffer)) {
-		UT_ASSERT(sizeof(XML_Char) == sizeof(char));
-		_setDefaultPageSize (fp_PageSize::NameToPredefined((char*) pszBuffer));
-	}
-
 	// ------------ Prefs
 	_setPrefsAutoSave( pPrefs->getAutoSavePrefs() );
 
@@ -319,10 +323,13 @@ void AP_Dialog_Options::_populateWindowData(void)
 	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_AllowCustomToolbars,&b))
 		_setAllowCustomToolbars(b);
 
+#if defined(XP_UNIX_TARGET_GTK)
+	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_EnableSmoothScrolling,&b))
+		_setEnableSmoothScrolling(b);
+#endif
 	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_AutoLoadPlugins,&b))
 		_setAutoLoadPlugins(b);
 
-#if 1
 	// TODO: JOAQUIN FIX THIS
 	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_AutoSaveFile,&b))
 		_setAutoSaveFile (b);
@@ -333,8 +340,11 @@ void AP_Dialog_Options::_populateWindowData(void)
 
 	if (pPrefs->getPrefsValue(XAP_PREF_KEY_AutoSaveFilePeriod, stBuffer))
 		_setAutoSaveFilePeriod(stBuffer);
-#endif
-
+		
+	//Just for win32 
+	if (pPrefs->getPrefsValue(AP_PREF_KEY_StringSet, stBuffer))
+		_setUILanguage(stBuffer);
+		
 	// ------------ Screen Color
 
 	const XML_Char * pszColorForTransparent = NULL;
@@ -348,51 +358,50 @@ void AP_Dialog_Options::_populateWindowData(void)
 		_setNotebookPageNum (atoi(pszBuffer));
 	else
 	  _setNotebookPageNum(which);
-#ifdef BIDI_ENABLED
+
 	//------------- other
 	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_DefaultDirectionRtl,&b))
 		_setOtherDirectionRtl (b);
-	if (pPrefs->getPrefsValueBool(XAP_PREF_KEY_UseContextGlyphs,&b))
-		_setOtherUseContextGlyphs (b);
 	if (pPrefs->getPrefsValueBool(XAP_PREF_KEY_SaveContextGlyphs,&b))
 		_setOtherSaveContextGlyphs (b);
 	if (pPrefs->getPrefsValueBool(XAP_PREF_KEY_UseHebrewContextGlyphs,&b))
 		_setOtherHebrewContextGlyphs (b);
-#endif
 
+	if (pPrefs->getPrefsValueBool(XAP_PREF_KEY_ChangeLanguageWithKeyboard,&b))
+		_setLanguageWithKeyboard (b);
+
+	if (pPrefs->getPrefsValueBool(XAP_PREF_KEY_DirMarkerAfterClosingParenthesis,&b))
+		_setDirMarkerAfterClosingParenthesis (b);
+	
 	// enable/disable controls
 	_initEnableControls();
+	m_bInitialPop = false;
 }
 
 
 void AP_Dialog_Options::_enableDisableLogic( tControl id )
 {
-#if 1
-#ifdef BIDI_ENABLED
 	UT_DEBUGMSG(("_enableDisableLogic: id %d, myId %d\n", id,id_CHECK_OTHER_USE_CONTEXT_GLYPHS));
-#endif
+
 	switch (id)
 	{
 
 /*	- Since HIDE_ERRORS is not implemented, no need to toggle it on/off
 	case id_CHECK_SPELL_CHECK_AS_TYPE:
 		// if we 'check as we type', then enable the 'hide' option
-		_controlEnable( id_CHECK_SPELL_HIDE_ERRORS, 
+		_controlEnable( id_CHECK_SPELL_HIDE_ERRORS,
 						_gatherSpellCheckAsType() );
 		break;
 */
-#ifdef BIDI_ENABLED
-	case id_CHECK_OTHER_USE_CONTEXT_GLYPHS:
-		_controlEnable( id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS, _gatherOtherUseContextGlyphs());
-		_controlEnable( id_CHECK_OTHER_HEBREW_CONTEXT_GLYPHS, _gatherOtherUseContextGlyphs());
-		break;
-#endif
 
+	case id_CHECK_DIR_MARKER_AFTER_CLOSING_PARENTHESIS:
+		_controlEnable( id_CHECK_DIR_MARKER_AFTER_CLOSING_PARENTHESIS, _gatherLanguageWithKeyboard());
+		break;
+		
 	default:
 		// do nothing
 		break;
-	}	
-#endif
+	}
 }
 
 // The initialize the controls (i.e., disable controls not coded yet)
@@ -416,15 +425,20 @@ void AP_Dialog_Options::_initEnableControls()
 
 	// general
 	_controlEnable( id_BUTTON_SAVE, 				false );
-	
-#ifdef BIDI_ENABLED
-	_controlEnable( id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS, _gatherOtherUseContextGlyphs());
-	_controlEnable( id_CHECK_OTHER_HEBREW_CONTEXT_GLYPHS, _gatherOtherUseContextGlyphs());
-#endif
-//
-// If the prefs color for transparent is white initially disable the choose
-// color button
-//
+
+	// language
+	// only works on win32 for now (must precede
+	// id_CHECK_DIR_MARKER... in the bidi controls !!!)
+	_controlEnable( id_CHECK_LANG_WITH_KEYBOARD,    false ); 
+
+	// bidi controls
+	_controlEnable( id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS, false); // not
+																// implemented
+	_controlEnable( id_CHECK_DIR_MARKER_AFTER_CLOSING_PARENTHESIS,_gatherLanguageWithKeyboard());  
+	//
+	// If the prefs color for transparent is white initially disable the choose
+	// color button
+	//
 	if(UT_strcmp(m_CurrentTransparentColor,"ffffff") == 0)
 	{
 		_controlEnable( id_PUSH_CHOOSE_COLOR_FOR_TRANSPARENT, false);
@@ -433,7 +447,8 @@ void AP_Dialog_Options::_initEnableControls()
 	{
 		_controlEnable( id_PUSH_CHOOSE_COLOR_FOR_TRANSPARENT, true);
 	}
-	
+
+	_initEnableControlsPlatformSpecific();
 }
 
 void AP_Dialog_Options::_event_SetDefaults(void)
@@ -447,7 +462,7 @@ void AP_Dialog_Options::_event_SetDefaults(void)
 
 	// SetDefaults
 	//	To set the defaults, save the scheme name and notebook page number,
-	//	re-populate the window with the _builtin_ scheme, then reset the 
+	//	re-populate the window with the _builtin_ scheme, then reset the
 	//	scheme name and page number.
 	// If the user hits cancel, then nothing is saved in user_prefs
 
@@ -455,8 +470,8 @@ void AP_Dialog_Options::_event_SetDefaults(void)
 
 	currentPage = _gatherNotebookPageNum();
 
-	pPrefs->setCurrentScheme("_builtin_");		
-	
+	pPrefs->setCurrentScheme("_builtin_");
+
 	_populateWindowData();
 
 	// TODO i'm not sure you want to do the following at this
@@ -464,7 +479,7 @@ void AP_Dialog_Options::_event_SetDefaults(void)
 	// TODO set us to "_builtin_" and that's it.  if the user
 	// TODO then changes something, we should create a new
 	// TODO scheme and fill in the new value.  --jeff
-	_setNotebookPageNum( currentPage ); 	
+	_setNotebookPageNum( currentPage );
 	pPrefs->setCurrentScheme(old_name);
 }
 
@@ -475,7 +490,7 @@ void AP_Dialog_Options::_event_IgnoreReset(void)
 
 	// TODO:  shack@uiuc.edu: waiting for a vote for reset strings...
 
-	// Ask "Do you want to reset ignored words in the current document?" 
+	// Ask "Do you want to reset ignored words in the current document?"
 	XAP_Dialog_MessageBox::tAnswer ans = m_pFrame->showMessageBox(AP_STRING_ID_DLG_Options_Prompt_IgnoreResetCurrent,
 								XAP_Dialog_MessageBox::b_YNC,
 								XAP_Dialog_MessageBox::a_NO); // should this be YES?
@@ -500,19 +515,19 @@ void AP_Dialog_Options::_event_IgnoreReset(void)
 	UT_ASSERT(pApp);
 	if (pApp->getFrameCount() > 1)
 	{
-		
+
 		ans = m_pFrame->showMessageBox(AP_STRING_ID_DLG_Options_Prompt_IgnoreResetAll,
 									XAP_Dialog_MessageBox::b_YNC,
 									XAP_Dialog_MessageBox::a_NO); // should this be YES?
 
 
-		// if cancel, don't to ANYTHING 
+		// if cancel, don't to ANYTHING
 		if (ans == XAP_Dialog_MessageBox::a_CANCEL )
 		{
 			UT_DEBUGMSG(("No/Canceled\n"));
 			return;
 		}
-	} 
+	}
 
 	// ------------------------- actually do it
 	if ( ans == XAP_Dialog_MessageBox::a_NO )
@@ -522,8 +537,8 @@ void AP_Dialog_Options::_event_IgnoreReset(void)
 		m_pFrame->getCurrentDoc()->clearIgnores();
 
 		((FV_View *)m_pFrame->getCurrentView())->getLayout()->recheckIgnoredWords();
-	}	
-	else 
+	}
+	else
 	{
 		// reset all doc's ignored words
 		UT_uint32 ndx;
