@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -46,7 +46,7 @@
 #include "ap_Dialog_Tab.h"
 
 AP_Dialog_Styles::AP_Dialog_Styles(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_NonPersistent(pDlgFactory,id)
+	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialogstyles")
 {
 
 	m_answer = a_OK;
@@ -66,13 +66,13 @@ AP_Dialog_Styles::~AP_Dialog_Styles(void)
 	for(i=0; i<m_vecAllProps.getItemCount(); i++)
 	{
 		char * psz = (char *) m_vecAllProps.getNthItem(i);
-		delete [] psz;
+		FREEP(psz);
 	}
 	m_vecAllProps.clear();
 	for(i=0; i<m_vecAllAttribs.getItemCount(); i++)
 	{
 		char * psz = (char *) m_vecAllAttribs.getNthItem(i);
-		delete [] psz;
+		FREEP(psz);
 	}
 	m_vecAllAttribs.clear();
 }
@@ -83,13 +83,13 @@ AP_Dialog_Styles::tAnswer AP_Dialog_Styles::getAnswer(void) const
 }
 
 /*!
- * This method adds the key,value pair pszProp,pszVal to the Vector of 
+ * This method adds the key,value pair pszProp,pszVal to the Vector of
  * all properties of the current style.
  * If the Property already exists it's value is replaced with pszVal
 \param const XML_Char * pszProp the property name
 \param const XML_Char * pszVal the value of this property.
 */
-void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp, 
+void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp,
 												 const XML_Char * pszVal)
 {
 	UT_sint32 iCount = m_vecAllProps.getItemCount();
@@ -111,7 +111,7 @@ void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp,
 	if(i < iCount)
 	{
 		char * pSV = (char *) m_vecAllProps.getNthItem(i+1);
-		delete [] pSV;
+		FREEP(pSV);
 		pSV = UT_strdup(pszVal);
 		m_vecAllProps.setNthItem(i+1, (void *) pSV, NULL);
 	}
@@ -127,7 +127,7 @@ void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp,
 
 
 /*!
- * This method removes the key,value pair  (pszProp,pszVal) given by pszProp 
+ * This method removes the key,value pair  (pszProp,pszVal) given by pszProp
  * from the Vector of all properties of the current style.
  * If the Property does not exists nothing happens
 \param const XML_Char * pszProp the property name
@@ -151,8 +151,8 @@ void AP_Dialog_Styles::removeVecProp(const XML_Char * pszProp)
 	{
 		char * pSP = (char *) m_vecAllProps.getNthItem(i);
 		char * pSV = (char *) m_vecAllProps.getNthItem(i+1);
-		delete [] pSP;
-		delete [] pSV;
+		FREEP(pSP);
+		FREEP(pSV);
 		m_vecAllProps.deleteNthItem(i+1);
 		m_vecAllProps.deleteNthItem(i);
 	}
@@ -160,13 +160,13 @@ void AP_Dialog_Styles::removeVecProp(const XML_Char * pszProp)
 }
 
 /*!
- * This method adds the key,value pair pszProp,pszVal to the Vector of 
+ * This method adds the key,value pair pszProp,pszVal to the Vector of
  * all attributes of the current style.
  * If the Property already exists it's value is replaced with pszVal
 \param const XML_Char * pszProp the property name
 \param const XML_Char * pszVal the value of this property.
 */
-void AP_Dialog_Styles::addOrReplaceVecAttribs(const XML_Char * pszProp, 
+void AP_Dialog_Styles::addOrReplaceVecAttribs(const XML_Char * pszProp,
 												 const XML_Char * pszVal)
 {
 	UT_sint32 iCount = m_vecAllAttribs.getItemCount();
@@ -188,7 +188,7 @@ void AP_Dialog_Styles::addOrReplaceVecAttribs(const XML_Char * pszProp,
 	if(i < iCount)
 	{
 		char * pSV = (char *) m_vecAllAttribs.getNthItem(i+1);
-		delete [] pSV;
+		FREEP(pSV);
 		pSV = UT_strdup(pszVal);
 		m_vecAllAttribs.setNthItem(i+1, (void *) pSV, NULL);
 	}
@@ -260,17 +260,17 @@ void AP_Dialog_Styles::fillVecWithProps(const XML_Char * szStyle, bool bReplaceA
 		return;
 	}
 
-	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", "margin-top", "margin-bottom", "line-height","tabstops","start-value","list-delim", "list-style","list-decimal","field-font","field-color"};
+	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", "margin-top", "margin-bottom", "line-height","tabstops","start-value","list-delim", "list-style","list-decimal","field-font","field-color", "keep-together","keep-with-next","orphans","widows","dom-dir"};
 
 	const size_t nParaFlds = sizeof(paraFields)/sizeof(paraFields[0]);
 
-	const static XML_Char * charFields[] = 
-	{"bgcolor","color","font-family","font-size","font-stretch","font-style", 
+	const static XML_Char * charFields[] =
+	{"bgcolor","color","font-family","font-size","font-stretch","font-style",
 	 "font-variant", "font-weight","text-decoration","lang"};
 
 	const size_t nCharFlds = sizeof(charFields)/sizeof(charFields[0]);
 
-	const static XML_Char * attribs[] = 
+	const static XML_Char * attribs[] =
 	{PT_FOLLOWEDBY_ATTRIBUTE_NAME,PT_BASEDON_ATTRIBUTE_NAME,PT_LISTID_ATTRIBUTE_NAME,PT_PARENTID_ATTRIBUTE_NAME,PT_LEVEL_ATTRIBUTE_NAME,PT_NAME_ATTRIBUTE_NAME,PT_STYLE_ATTRIBUTE_NAME,PT_TYPE_ATTRIBUTE_NAME};
 
 	const size_t nattribs = sizeof(attribs)/sizeof(attribs[0]);
@@ -281,7 +281,7 @@ void AP_Dialog_Styles::fillVecWithProps(const XML_Char * szStyle, bool bReplaceA
 	for(i = 0; i < nParaFlds; i++)
 	{
 		const XML_Char * szName = paraFields[i];
-		const XML_Char * szValue = NULL;		
+		const XML_Char * szValue = NULL;
 		pStyle->getProperty(szName,szValue);
 		if(szValue)
 			addOrReplaceVecProp(szName, szValue);
@@ -292,7 +292,7 @@ void AP_Dialog_Styles::fillVecWithProps(const XML_Char * szStyle, bool bReplaceA
 	for(i = 0; i < nCharFlds; i++)
 	{
 		const XML_Char * szName = charFields[i];
-		const XML_Char * szValue = NULL;		
+		const XML_Char * szValue = NULL;
 		pStyle->getProperty(szName,szValue);
 		if(szValue)
 			addOrReplaceVecProp(szName, szValue);
@@ -305,7 +305,7 @@ void AP_Dialog_Styles::fillVecWithProps(const XML_Char * szStyle, bool bReplaceA
 		for(i = 0; i < nattribs; i++)
 		{
 			const XML_Char * szName = attribs[i];
-			const XML_Char * szValue = NULL;		
+			const XML_Char * szValue = NULL;
 			pStyle->getAttribute(szName,szValue);
 			if(szValue)
 				addOrReplaceVecAttribs(szName, szValue);
@@ -324,7 +324,7 @@ void AP_Dialog_Styles::fillVecWithProps(const XML_Char * szStyle, bool bReplaceA
 const XML_Char * AP_Dialog_Styles::getPropsVal(const XML_Char * szProp) const
 {
 	UT_sint32 i = m_vecAllProps.getItemCount();
-	if(i <= 0) 
+	if(i <= 0)
 		return NULL;
 	UT_sint32 j;
 	const XML_Char * pszV = NULL;
@@ -351,7 +351,7 @@ const XML_Char * AP_Dialog_Styles::getPropsVal(const XML_Char * szProp) const
 const XML_Char * AP_Dialog_Styles::getAttsVal(const XML_Char * szProp) const
 {
 	UT_sint32 i = m_vecAllAttribs.getItemCount();
-	if(i <= 0) 
+	if(i <= 0)
 		return NULL;
 	UT_sint32 j;
 	const XML_Char * pszV = NULL;
@@ -379,7 +379,7 @@ const XML_Char * AP_Dialog_Styles::getAttsVal(const XML_Char * szProp) const
 const XML_Char * AP_Dialog_Styles::getVecVal(const UT_Vector *v, const XML_Char * szProp) const
 {
 	UT_sint32 i = v->getItemCount();
-	if(i <= 0) 
+	if(i <= 0)
 		return NULL;
 	UT_sint32 j;
 	const XML_Char * pszV = NULL;
@@ -396,7 +396,7 @@ const XML_Char * AP_Dialog_Styles::getVecVal(const UT_Vector *v, const XML_Char
 }
 
 /*!
- * This method runs the language dialog to allow the user to set the language 
+ * This method runs the language dialog to allow the user to set the language
  * property of their style.
  */
 void AP_Dialog_Styles::ModifyLang(void)
@@ -437,7 +437,7 @@ void AP_Dialog_Styles::ModifyLang(void)
 }
 
 /*!
- * This method runs the tabs dialog to allow the user to edit the character 
+ * This method runs the tabs dialog to allow the user to edit the character
  * properties of their style.
  */
 void AP_Dialog_Styles::ModifyFont(void)
@@ -482,7 +482,7 @@ void AP_Dialog_Styles::ModifyFont(void)
 	sprintf(background, "%02x%02x%02x",bgCol->m_red,
 			bgCol->m_grn,bgCol->m_blu);
 	pDialog->setBackGroundColor( (const XML_Char *) background);
-		
+
 	// these behave a little differently since they are
 	// probably just check boxes and we don't have to
 	// worry about initializing a combo box with a choice
@@ -504,7 +504,6 @@ void AP_Dialog_Styles::ModifyFont(void)
 	}
 	pDialog->setFontDecoration(bUnderline,bOverline,bStrikeOut,bTopline,bBottomline);
 /*
-#ifdef BIDI_ENABLED
     bool bDirection;
 	s = UT_getAttribute("dir", props_in);
 	if (s)
@@ -512,7 +511,6 @@ void AP_Dialog_Styles::ModifyFont(void)
 	     bDirection = (strstr(s, "rtl") != NULL);
 	}
 	pDialog->setDirection(bDirection);
-#endif
 */
 
 	// run the dialog
@@ -568,10 +566,8 @@ void AP_Dialog_Styles::ModifyFont(void)
 		bool bBottomline = false;
 		bool bChangedBottomline = pDialog->getChangedBottomline(&bBottomline);
 /*
-#ifdef BIDI_ENABLED
 		bool bDirection = false;
 		bool bChangedDirection = pDialog->getChangedDirection(&bDirection);
-#endif
 */
 
 		if (bChangedUnderline || bChangedStrikeOut || bChangedOverline || bChangedTopline || bChangedBottomline)
@@ -595,17 +591,15 @@ void AP_Dialog_Styles::ModifyFont(void)
 			addOrReplaceVecProp("text-decoration", s);
 		}
 /*
-#ifdef BIDI_ENABLED
 		if(bChangedDirection)
 		{
 		    if (bDirection == 1)
 		        s = "rtl";
 		    else
 		        s = "ltr";
-		
+
 			addOrReplaceVecProp("dir", s);
 		}
-#endif
 */
 	}
 	pDialogFactory->releaseDialog(pDialog);
@@ -628,7 +622,7 @@ void AP_Dialog_Styles::_tabCallback(const char *szTabStops,
  * Used to extract data out of the Tabs dialog.
  */
 static void
-s_TabSaveCallBack (AP_Dialog_Tab * pDlg, FV_View * pView, 
+s_TabSaveCallBack (AP_Dialog_Tab * pDlg, FV_View * pView,
 				   const char * szTabStops, const char * szDflTabStop,
 				   void * closure)
 {
@@ -682,7 +676,7 @@ void AP_Dialog_Styles::ModifyLists(void)
   		= (XAP_DialogFactory *) getFrame()->getDialogFactory();
 
 //
-// Use this method so that we can have modeless Lists dialog and this 
+// Use this method so that we can have modeless Lists dialog and this
 // modal version simultaneously
 //
 	AP_Dialog_Lists * pDialog
@@ -692,7 +686,7 @@ void AP_Dialog_Styles::ModifyLists(void)
 
 //
 // Fill input list for Lists dialog
-// 
+//
 	if(getPropsVal("list-style"))
 	{
 		vp.addItem((void *) "list-style");
@@ -781,7 +775,7 @@ void AP_Dialog_Styles::ModifyLists(void)
 		}
 //
 // Whew we're done!
-//		
+//
 	}
 	delete pDialog;
 }
@@ -798,7 +792,7 @@ void AP_Dialog_Styles::ModifyParagraph(void)
 		= (AP_Dialog_Paragraph *)(pDialogFactory->requestDialog(AP_DIALOG_ID_PARAGRAPH));
 	UT_ASSERT(pDialog);
 
-	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", "margin-top", "margin-bottom", "line-height","tabstops","start-value","list-delim", "list-decimal","list-style","field-font","field-color"};
+	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", "margin-top", "margin-bottom", "line-height","tabstops","start-value","list-delim", "list-decimal","list-style","field-font","field-color", "keep-together","keep-with-next","orphans","widows","dom-dir"};
 
     const size_t NUM_PARAPROPS = sizeof(paraFields)/sizeof(paraFields[0]);
 //
@@ -870,7 +864,7 @@ void AP_Dialog_Styles::ModifyParagraph(void)
 
 			while (propitem[0] && propitem[1])
 			{
-				
+
 				FREEP(propitem[0]);
 				FREEP(propitem[1]);
 				propitem += 2;
@@ -885,7 +879,7 @@ void AP_Dialog_Styles::ModifyParagraph(void)
 
 /*!
  * Extract all the props from the vector and apply them to the abi preview. We use
- * the style "tmp" to display the current style in the preview. 
+ * the style "tmp" to display the current style in the preview.
  */
 void AP_Dialog_Styles::updateCurrentStyle(void)
 {
@@ -935,8 +929,7 @@ void AP_Dialog_Styles::updateCurrentStyle(void)
 	getLView()->setPoint(m_posFocus+1);
 	getLView()->setStyle("tmp");
 	drawLocal();
-	DELETEP(props);
-
+	FREEP(props);
 }
 
 
@@ -980,7 +973,7 @@ bool AP_Dialog_Styles::createNewStyle(const XML_Char * szName)
 // Append the new style to the main document
 //
 	PD_Style * pStyle = NULL;
-	
+
 	UT_ASSERT(szName);
 	if(szName == NULL)
 		return false;
@@ -1061,8 +1054,8 @@ bool AP_Dialog_Styles::applyModifiedStyleToDoc(void)
 	}
 #endif
 	bool bres = getDoc()->setAllStyleAttributes(szStyle,attribs);
-	DELETEP(props);
-	DELETEP(attribs);
+	FREEP(props);
+	FREEP(attribs);
 	return bres;
 }
 /*!
@@ -1126,13 +1119,13 @@ void AP_Dialog_Styles::_createParaPreviewFromGC(GR_Graphics * gc,
 
 	UT_UCSChar * str;
 
-	UT_UCS_cloneString_char (&str, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
+	UT_UCS4_cloneString_char (&str, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
 
 	m_pParaPreview = new AP_Preview_Paragraph(gc, str, static_cast<XAP_Dialog*>(this));
 	UT_ASSERT(m_pParaPreview);
 
 	FREEP(str);
-	
+
 	m_pParaPreview->setWindowSize(width, height);
 }
 
@@ -1154,13 +1147,13 @@ void AP_Dialog_Styles::_createCharPreviewFromGC(GR_Graphics * gc,
 
 	m_pCharPreview = new XAP_Preview_FontPreview(gc,background);
 	UT_ASSERT(m_pCharPreview);
-	
+
 	m_pCharPreview->setWindowSize(width, height);
 //
 // Text for the Preview
 //
 	static UT_UCSChar szString[60];
-	UT_UCS_strcpy_char( (UT_UCSChar *) szString, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
+	UT_UCS4_strcpy_char( (UT_UCSChar *) szString, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
 	m_pCharPreview->setDrawString((const UT_UCSChar *) szString);
 //
 // set our Vector of Character Properties into the preview class.
@@ -1180,7 +1173,7 @@ void AP_Dialog_Styles::_createAbiPreviewFromGC(GR_Graphics * gc,
 	UT_ASSERT(gc);
 	if(m_pAbiPreview)
 		DELETEP(m_pAbiPreview);
-	m_pAbiPreview = new AP_Preview_Abi(gc,width,height,getFrame(),PREVIEW_ZOOMED);
+	m_pAbiPreview = new AP_Preview_Abi(gc,gc->tlu(width),gc->tlu(height),getFrame(),PREVIEW_ZOOMED);
 	UT_ASSERT(m_pAbiPreview);
 }
 
@@ -1227,14 +1220,14 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 	static UT_UCSChar sz3[4];
 	static UT_UCSChar szSpace[4];
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-	UT_UCS_strcpy_char( (UT_UCSChar *) szString, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
-	UT_UCS_strcpy_char( (UT_UCSChar *) sz1, " 1");
-	UT_UCS_strcpy_char( (UT_UCSChar *) sz2, " 2");
-	UT_UCS_strcpy_char( (UT_UCSChar *) sz3, " 3");
-	UT_UCS_strcpy_char( (UT_UCSChar *) szSpace, "  ");
-	UT_uint32 len =UT_UCS_strlen(szString);
-	UT_uint32 len1 =UT_UCS_strlen(sz1);
-	UT_uint32 lenSpace =UT_UCS_strlen(szSpace);
+	UT_UCS4_strcpy_char( (UT_UCSChar *) szString, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
+	UT_UCS4_strcpy_char( (UT_UCSChar *) sz1, " 1");
+	UT_UCS4_strcpy_char( (UT_UCSChar *) sz2, " 2");
+	UT_UCS4_strcpy_char( (UT_UCSChar *) sz3, " 3");
+	UT_UCS4_strcpy_char( (UT_UCSChar *) szSpace, "  ");
+	UT_uint32 len =UT_UCS4_strlen(szString);
+	UT_uint32 len1 =UT_UCS4_strlen(sz1);
+	UT_uint32 lenSpace =UT_UCS4_strlen(szSpace);
 	const char * szStyle = NULL;
 	if(!isNew)
 	{
@@ -1251,7 +1244,7 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 							   "page-margin-header","0.0in",NULL};
 	getLView()->setSectionFormat(props);
 //
-// First Paragraph 
+// First Paragraph
 //
 	m_posBefore = getLView()->getPoint();
 	UT_uint32 i=0;
@@ -1305,7 +1298,7 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 //
 	const XML_Char * GreyCol[3] = {"color",(const XML_Char *) Grey,NULL};
 	getLDoc()->changeSpanFmt(PTC_AddFmt,m_posBefore,getLView()->getPoint(),NULL,GreyCol);
-	
+
 	getLView()->insertParagraphBreak();
 //
 // Second Paragraph in focus. Our Vectors containing the current settings have
@@ -1363,7 +1356,7 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 		const XML_Char * FGCol[3] = {"color",(const XML_Char *) szFGColor,NULL};
 		getLView()->setCharFormat(FGCol);
 	}
-	DELETEP(lprop);
+	FREEP(lprop);
 	for(i=0; i<8; i++)
 	{
 		getLView()->cmdCharInsert((UT_UCSChar *) szString,len);
@@ -1398,7 +1391,7 @@ void AP_Dialog_Styles::destroyAbiPreview(void)
 void AP_Dialog_Styles::event_paraPreviewUpdated (const XML_Char * pageLeftMargin,
 						 const XML_Char * pageRightMargin,
 						 const XML_Char * align,
-						 const XML_Char * firstLineIndent,						 
+						 const XML_Char * firstLineIndent,
 						 const XML_Char * leftIndent,
 						 const XML_Char * rightIndent,
 						 const XML_Char * beforeSpacing,
@@ -1474,7 +1467,7 @@ void AP_Dialog_Styles::event_paraPreviewUpdated (const XML_Char * pageLeftMargin
 							   afterSpacing,
 							   lineSpacing,
 							   tSpacing);
-	
+
 	// force a redraw
 	m_pParaPreview->draw();
 }
@@ -1487,7 +1480,7 @@ void AP_Dialog_Styles::event_charPreviewUpdated (void) const
 	UT_ASSERT (m_pCharPreview); // add this when we make a char preview
 
 	// force a redraw
-	if(m_pCharPreview) 
+	if(m_pCharPreview)
 	{
 		m_pCharPreview->setVecProperties( &m_vecCharProps);
 		m_pCharPreview->draw();
@@ -1503,13 +1496,13 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 	PD_Style * pStyle = NULL;
 	const char * szStyle = NULL;
 
-	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", 
+	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right",
 											"margin-top", "margin-bottom", "line-height"};
 	const size_t nParaFlds = sizeof(paraFields)/sizeof(paraFields[0]);
 	const XML_Char * paraValues [nParaFlds];
 
-	const static XML_Char * charFields[] = 
-	{"bgcolor","color","font-family","font-size","font-stretch","font-style", 
+	const static XML_Char * charFields[] =
+	{"bgcolor","color","font-family","font-size","font-stretch","font-style",
 	 "font-variant", "font-weight","text-decoration"};
 	const size_t nCharFlds = sizeof(charFields)/sizeof(charFields[0]);
 	const XML_Char * charValues [nCharFlds];
@@ -1539,7 +1532,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 		for(i = 0; i < nParaFlds; i++)
 		{
 			const XML_Char * szName = paraFields[i];
-			const XML_Char * szValue = NULL;		
+			const XML_Char * szValue = NULL;
 
 			if (!pStyle->getProperty(szName, szValue))
 				if (!pStyle->getAttribute(szName, szValue))
@@ -1547,7 +1540,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 					paraValues[i] = 0;
 					continue;
 				}
-				
+
 			m_curStyleDesc += (const char *)szName;
 			m_curStyleDesc += ":";
 			m_curStyleDesc += (const char *)szValue;
@@ -1565,7 +1558,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 		for(i = 0; i < nCharFlds; i++)
 		{
 			const XML_Char * szName = charFields[i];
-			const XML_Char * szValue = NULL;		
+			const XML_Char * szValue = NULL;
 
 			if (!pStyle->getProperty(szName, szValue))
 				if (!pStyle->getAttribute(szName, szValue))
@@ -1585,7 +1578,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 //
 			m_vecCharProps.addItem((void *) szName);
 			m_vecCharProps.addItem((void *) szValue);
-					
+
 		}
 
 		if (!m_curStyleDesc.empty())
@@ -1601,12 +1594,12 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 			// these aren't set at a style level, but we need to put them in there anyway
 			const XML_Char ** props_in = NULL;
 			getView()->getSectionFormat(&props_in);
-			
+
 			if(!isModify)
-				event_paraPreviewUpdated(UT_getAttribute("page-margin-left", props_in), 
+				event_paraPreviewUpdated(UT_getAttribute("page-margin-left", props_in),
 									 UT_getAttribute("page-margin-right", props_in),
 									 (const XML_Char *)paraValues[0], (const XML_Char *)paraValues[1],
-									 (const XML_Char *)paraValues[2], (const XML_Char *)paraValues[3], 
+									 (const XML_Char *)paraValues[2], (const XML_Char *)paraValues[3],
 									 (const XML_Char *)paraValues[4], (const XML_Char *)paraValues[5],
 									 (const XML_Char *)paraValues[6]);
 			if(!isModify)
@@ -1614,8 +1607,3 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 		}
 	}
 }
-
-
-
-
-
