@@ -1,24 +1,23 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
 #include <stdlib.h>
-#include <locale.h>
 #include "ut_string.h"
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
@@ -57,11 +56,11 @@ IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument)
 {
 	m_error = 0;
 	m_pListenerWriteDoc = NULL;
-	m_pListenerGetProps = NULL;
 	m_bNeedUnicodeText = false;
 	m_braceLevel = 0;
 	m_bLastWasKeyword = false;
 	m_atticFormat = false;
+	m_CharRTL = FRIBIDI_TYPE_UNSET;
 }
 
 IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument,bool atticFormat)
@@ -69,11 +68,11 @@ IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument,bool atticFormat)
 {
 	m_error = 0;
 	m_pListenerWriteDoc = NULL;
-	m_pListenerGetProps = NULL;
 	m_bNeedUnicodeText = false;
 	m_braceLevel = 0;
 	m_bLastWasKeyword = false;
 	m_atticFormat = atticFormat;
+	m_CharRTL = FRIBIDI_TYPE_UNSET;
 }
 
 IE_Exp_RTF::~IE_Exp_RTF()
@@ -86,6 +85,21 @@ IE_Exp_RTF::~IE_Exp_RTF()
 /*****************************************************************/
 /*****************************************************************/
 
+IE_Exp_RTF_Sniffer::IE_Exp_RTF_Sniffer ()
+	: IE_ExpSniffer(IE_IMPEXPNAME_RTF)
+{
+	// 
+}
+
+UT_Confidence_t IE_Exp_RTF_Sniffer::supportsMIME (const char * szMIME)
+{
+	if (UT_strcmp (szMIME, IE_MIME_RTF) == 0)
+		{
+			return UT_CONFIDENCE_GOOD;
+		}
+	return UT_CONFIDENCE_ZILCH;
+}
+
 bool IE_Exp_RTF_Sniffer::recognizeSuffix(const char * szSuffix)
 {
 	return (!UT_stricmp(szSuffix,".rtf"));
@@ -111,6 +125,12 @@ bool IE_Exp_RTF_Sniffer::getDlgLabels(const char ** pszDesc,
 
 /*for attic*/
 
+IE_Exp_RTF_attic_Sniffer::IE_Exp_RTF_attic_Sniffer ()
+	: IE_ExpSniffer(IE_IMPEXPNAME_RTFATTIC)
+{
+	// 
+}
+
 bool IE_Exp_RTF_attic_Sniffer::recognizeSuffix(const char * szSuffix)
 {
 	return (!UT_stricmp(szSuffix,".rtf"));
@@ -133,9 +153,12 @@ bool IE_Exp_RTF_attic_Sniffer::getDlgLabels(const char ** pszDesc,
 	*ft = getFileType();
 	return true;
 }
-	
 
-#if !defined(DEBUG)
+IE_Exp_MsWord_Hack_Sniffer::IE_Exp_MsWord_Hack_Sniffer ()
+	: IE_ExpSniffer(IE_IMPEXPNAME_RTFMSDOC)
+{
+	// 
+}
 
 bool IE_Exp_MsWord_Hack_Sniffer::recognizeSuffix(const char * szSuffix)
 {
@@ -145,7 +168,7 @@ bool IE_Exp_MsWord_Hack_Sniffer::recognizeSuffix(const char * szSuffix)
 UT_Error IE_Exp_MsWord_Hack_Sniffer::constructExporter(PD_Document * pDocument,
 											   IE_Exp ** ppie)
 {
-	IE_Exp_RTF * p = new IE_Exp_RTF(pDocument,1);
+	IE_Exp_RTF * p = new IE_Exp_RTF(pDocument);
 	*ppie = p;
 	return UT_OK;
 }
@@ -160,8 +183,6 @@ bool IE_Exp_MsWord_Hack_Sniffer::getDlgLabels(const char ** pszDesc,
 	return true;
 }
 
-#endif
-  
 /*****************************************************************/
 /*****************************************************************/
 
@@ -181,15 +202,21 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
 
 	_addColor("000000");				// load black as color 0.
 	_addColor("ffffff");                // load white as color 1.
-		
-	m_pListenerGetProps = new s_RTF_ListenerGetProps(getDoc(),this);
-	if (!m_pListenerGetProps)
+
+	s_RTF_ListenerGetProps * listenerGetProps = new s_RTF_ListenerGetProps(getDoc(),this);
+	if (!listenerGetProps)
 		return UT_IE_NOMEMORY;
 	if (getDocRange())
-		getDoc()->tellListenerSubset(static_cast<PL_Listener *>(m_pListenerGetProps),getDocRange());
+		getDoc()->tellListenerSubset(listenerGetProps,getDocRange());
 	else
-		getDoc()->tellListener(static_cast<PL_Listener *>(m_pListenerGetProps));
-	DELETEP(m_pListenerGetProps);
+		getDoc()->tellListener(listenerGetProps);
+
+	// if the bit to be pasted contains a new block anywhere within it,
+	// then we also want the block props for the first block in the
+	// clipboard.  mark this info down here, use it in the listener.
+	bool hasBlock = listenerGetProps->hasBlock();
+
+	DELETEP(listenerGetProps);
 
 	// Important: This must come before the header is written so
         // every font used in a style is properly entered in the font table.
@@ -199,24 +226,24 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
 
 	if (!_write_rtf_header())
 		return UT_IE_COULDNOTWRITE;
-	
+
 	// create and install a listener to receive the document
 	// and write its content in rtf.
-	
-	m_pListenerWriteDoc = new s_RTF_ListenerWriteDoc(getDoc(),this, (getDocRange()!=NULL));
+
+	m_pListenerWriteDoc = new s_RTF_ListenerWriteDoc(getDoc(),this, (getDocRange()!=NULL), hasBlock);
 	if (!m_pListenerWriteDoc)
 		return UT_IE_NOMEMORY;
 	if (getDocRange())
-		getDoc()->tellListenerSubset(static_cast<PL_Listener *>(m_pListenerWriteDoc),getDocRange());
+		getDoc()->tellListenerSubset(m_pListenerWriteDoc,getDocRange());
 	else
-		getDoc()->tellListener(static_cast<PL_Listener *>(m_pListenerWriteDoc));
+		getDoc()->tellListener(m_pListenerWriteDoc);
 	DELETEP(m_pListenerWriteDoc);
 
 	// write any rtf trailer matter
 
 	if (!_write_rtf_trailer())
 		return UT_IE_COULDNOTWRITE;
-	
+
 	return ((m_error) ? UT_IE_COULDNOTWRITE : UT_OK);
 }
 
@@ -227,7 +254,7 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
                                 export.
  \params pszHdrFtrID const char * identification string for the header/footer
  */
-void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID)
+void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID, const char * pszKeyWord)
 {
 
 // First find the header/footer section and id in the document.
@@ -241,14 +268,14 @@ void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID)
 
 	if(hdrSDH == NULL)
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	  UT_ASSERT_NOT_REACHED();
 		return;
 	}
 	PT_DocPosition posStart = getDoc()->getStruxPosition(hdrSDH);
 	PT_DocPosition posEnd = 0;
 	PL_StruxDocHandle nextSDH = NULL;
 	bool found = getDoc()->getNextStruxOfType(hdrSDH,PTX_SectionHdrFtr ,&nextSDH);
-	
+
 	if(!found || (nextSDH == NULL ))
 	{
 		getDoc()->getBounds(true, posEnd);
@@ -268,30 +295,7 @@ void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID)
 	}
 	_rtf_nl();
 	_rtf_open_brace();
-	if(strcmp(pszHdrFtr,"header") == 0)
-	{
-		_rtf_keyword("header");
-	}
-	else if(strcmp(pszHdrFtr,"footer") == 0)
-	{
-		_rtf_keyword("footer");
-	}
-	else if(strcmp(pszHdrFtr,"header-even") == 0)
-	{
-		_rtf_keyword("headerl");
-	}
-	else if(strcmp(pszHdrFtr,"header-first") == 0)
-	{
-		_rtf_keyword("headerf");
-	}
-	else if(strcmp(pszHdrFtr,"footer-even") == 0)
-	{
-		_rtf_keyword("footerl");
-	}
-	else if(strcmp(pszHdrFtr,"footer-first") == 0)
-	{
-		_rtf_keyword("footerf");
-	}
+	_rtf_keyword(pszKeyWord);
 	_rtf_keyword("pard");
 	_rtf_keyword("plain");
 	m_pListenerWriteDoc->m_bBlankLine = false;
@@ -342,7 +346,7 @@ void s_RTF_Listener::_handleDataItems(void)
 			UT_uint32 j;
 			for (j=0; j<jLimit; j+=72)
 			{
-				jSize = MyMin(72,(jLimit-j));
+				jSize = UT_MIN(72,(jLimit-j));
 				m_pie->write((const char *)bb64.getPointer(j),jSize);
 				m_pie->write("\n");
 			}
@@ -363,7 +367,7 @@ UT_sint32 IE_Exp_RTF::_findColor(const char * szColor) const
 {
 	if (!szColor || !*szColor)
 		return 0;						// black
-	
+
 	UT_uint32 k;
 	UT_uint32 kLimit = m_vecColors.getItemCount();
 
@@ -379,7 +383,7 @@ UT_sint32 IE_Exp_RTF::_findColor(const char * szColor) const
 
 void IE_Exp_RTF::_addColor(const char * szColor)
 {
-	UT_ASSERT(szColor && *szColor && (_findColor(szColor)==-1));
+	UT_return_if_fail(szColor && *szColor && (_findColor(szColor)==-1));
 
 	char * sz = NULL;
 	UT_cloneString(sz,szColor);
@@ -415,7 +419,7 @@ void IE_Exp_RTF::_rtf_close_brace(void)
 	write("}");
 	m_bLastWasKeyword = false;
 
-	UT_ASSERT(m_braceLevel >= 0);
+	UT_return_if_fail(m_braceLevel >= 0);
 }
 
 void IE_Exp_RTF::_rtf_keyword(const char * szKey)
@@ -488,7 +492,7 @@ void IE_Exp_RTF::_rtf_keyword_ifnotdefault_twips(const char * szKey, const char
 	// convert dimensioned value into twips (twentieths of a point) (aka 720 twips/inch)
 	double dbl = UT_convertToPoints(szValue);
 	UT_sint32 d = (UT_sint32)(dbl * 20.0);
-	
+
 	if (d == defaultValue)
 		return;
 
@@ -507,17 +511,17 @@ void IE_Exp_RTF::_rtf_semi(void)
 void IE_Exp_RTF::_rtf_fontname(const char * szFontName)
 {
 	write(" ");
-#if 0 
+#if 0
 	/*we handle 'helvetica' in a special way on import - so it's safe
 	 to output "Helvetica" unconditionally.
-	*/	
+	*/
 	if (!m_atticFormat && 0)
 	    write(szFontName);
 	else
-#endif	
+#endif
 	{
 		/*  map "Helvetic" to "Helvetica", since on Windows
-		    font "Helvetic" contains only Latin1 chars, while 
+		    font "Helvetic" contains only Latin1 chars, while
 		    "Helvetica" contains all needed chars. This is innocient
 		    since we do this when attic format is requested.
 		*/
@@ -536,7 +540,7 @@ void IE_Exp_RTF::_rtf_chardata(const char * pbuf, UT_uint32 buflen)
 		write(" ");
 		m_bLastWasKeyword = false;
 	}
-	
+
 	if(0 == buflen)
 		return;
 	write(pbuf,buflen);
@@ -558,17 +562,17 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 
 	_rtf_open_brace();
 	_rtf_keyword("rtf",1);				// major version number of spec version 1.5
-	
+
 	_rtf_keyword("ansi");
 	bool wrote_cpg = 0;
-	if (langcode) 
+	if (langcode)
 	{
-		char* cpgname = const_cast<char *>(wvLIDToCodePageConverter(langcode));
+		const char* cpgname = wvLIDToCodePageConverter(langcode);
 		UT_DEBUGMSG(("Belcon,after wvLIDToCodePageConverter(%d),cpgname=%s\n",langcode,cpgname));
-		if (UT_strnicmp(cpgname,"cp",2)==0 && UT_UCS_isdigit(cpgname[2])) 
+		if (UT_strnicmp(cpgname,"cp",2)==0 && UT_UCS4_isdigit(cpgname[2]))
 		{
 			int cpg;
-			if (sscanf(cpgname+2,"%d",&cpg)==1) 
+			if (sscanf(cpgname+2,"%d",&cpg)==1)
 			{
 				_rtf_keyword("ansicpg",cpg);
 				wrote_cpg = 1;
@@ -579,8 +583,8 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 		// NOTE:but we need convert them to CPxx.:-(
 		else
 		{
-			const char* codepage=XAP_EncodingManager::get_instance()->CodepageFromCharset(cpgname);
-			if(UT_strnicmp(codepage,"cp",2)==0 && UT_UCS_isdigit(codepage[2]))
+			const char* codepage=XAP_EncodingManager::get_instance()->CodepageFromCharset(const_cast<char*>(cpgname));
+			if(UT_strnicmp(codepage,"cp",2)==0 && UT_UCS4_isdigit(codepage[2]))
 			{
 				int cpg;
 				if (sscanf(codepage+2,"%d",&cpg)==1)
@@ -597,7 +601,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 
 	_rtf_keyword("deff",0);				// default font is index 0 aka black
 	if (m_atticFormat)
-	{	
+	{
 		/* I'm not sure whether this makes any sense - VH */
 		if (langcode)
 			_rtf_keyword("deflang",langcode);
@@ -613,7 +617,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 		_rtf_nl();
 		_rtf_open_brace();
 		_rtf_keyword("fonttbl");
-		UT_uint32 charsetcode = XAP_EncodingManager::get_instance()->getWinCharsetCode();
+		/*UT_uint32 charsetcode =*/ XAP_EncodingManager::get_instance()->getWinCharsetCode();
 		for (k=0; k<kLimit; k++)
 		{
 			const _rtf_font_info * pk = (const _rtf_font_info *)m_vecFonts.getNthItem(k);
@@ -624,21 +628,21 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 			_rtf_keyword("fcharset",pk->getFontCharset());
 			_rtf_keyword("fprq",pk->getFontPitch());							// {0==default,1==fixed,2==variable}
 			_rtf_keyword((pk->isTrueType()) ? "fttruetype" : "ftnil");	// {\fttruetype,\ftnil}
-			
+
 			// we do nothing with or use default values for
 			// \falt \panose \fname \fbias \ftnil \fttruetype \fontfile
-			
+
 			// after we write the various generic font properties, we write
 			// the actual font name and a semicolon -- i couldn't see this
 			// described in the specification, but it was in other RTF files
 			// that i saw and really seems to help Word and WordPad....
 			_rtf_fontname(pk->getFontName());
-			
+
 			_rtf_close_brace();
 		}
 		_rtf_close_brace();
 	}
-	
+
 	// TODO write the "file table" if necessary...
 
 	kLimit = m_vecColors.getItemCount();
@@ -670,7 +674,220 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 	_rtf_nl();
 	_rtf_keyword("kerning",0);			// turn off kerning
 	_rtf_keyword("cf",0);				// set color 0 -- black
-	
+	_rtf_keyword("ftnbj");				// set footnotes to bottom of page
+	_rtf_keyword("fet",2);				// Allow both footnotes and endnotes
+	_rtf_keyword("ftnstart",1);			// First footnote is one - later use
+	                                    // document properties
+	const XML_Char * pszFootnoteType = NULL;
+	const PP_AttrProp* pDocAP = getDoc()->getAttrProp();
+	UT_ASSERT(pDocAP);
+	pDocAP->getProperty("document-footnote-type", (const XML_Char *&)pszFootnoteType);
+	if (pszFootnoteType == NULL)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(pszFootnoteType[0] == 0)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"numeric") == 0)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"numeric-square-brackets") == 0)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"numeric-paren") == 0)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"numeric-open-paren") == 0)
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"upper") == 0)
+	{
+		_rtf_keyword("ftnnauc");			// Alphabetic Upper case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"upper-paren") == 0)
+	{
+		_rtf_keyword("ftnnauc");			// Alphabetic Upper case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"upper-paren-open") == 0)
+	{
+		_rtf_keyword("ftnnauc");			// Alphabetic Upper case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"lower") == 0)
+	{
+		_rtf_keyword("ftnnalc");			// Alphabetic Lower case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"lower-paren") == 0)
+	{
+		_rtf_keyword("ftnnalc");			// Alphabetic Lower case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"lower-paren-open") == 0)
+	{
+		_rtf_keyword("ftnnalc");			// Alphabetic Lower case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"lower-roman") == 0)
+	{
+		_rtf_keyword("ftnnrlc");			// Roman Lower case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"lower-roman-paren") == 0)
+	{
+		_rtf_keyword("ftnnrlc");			// Roman Lower case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"upper-roman") == 0)
+	{
+		_rtf_keyword("ftnnruc");			// Roman Upper case
+	}
+	else if(UT_XML_strcmp(pszFootnoteType,"upper-roman-paren") == 0)
+	{
+		_rtf_keyword("ftnnruc");			// Roman Upper case
+	}
+	else
+	{
+		_rtf_keyword("ftnnar");			// Numeric Footnotes
+	}
+
+	const XML_Char * pszEndnoteType = NULL;
+	pDocAP->getProperty("document-endnote-type", (const XML_Char *&)pszEndnoteType);
+	if (pszEndnoteType == NULL)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(pszEndnoteType[0] == 0)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"numeric") == 0)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"numeric-square-brackets") == 0)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"numeric-paren") == 0)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"numeric-open-paren") == 0)
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"upper") == 0)
+	{
+		_rtf_keyword("aftnnauc");			// Alphabetic Upper Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"upper-paren") == 0)
+	{
+		_rtf_keyword("aftnnauc");			// Alphabetic Upper Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"upper-paren-open") == 0)
+	{
+		_rtf_keyword("aftnnauc");			// Alphabetic Upper Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"lower") == 0)
+	{
+		_rtf_keyword("aftnnalc");			// Alphabetic Lower Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"lower-paren") == 0)
+	{
+		_rtf_keyword("aftnnalc");			// Alphabetic Lower Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"lower-paren-open") == 0)
+	{
+		_rtf_keyword("aftnnalc");			// Alphabetic Lower Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"lower-roman") == 0)
+	{
+		_rtf_keyword("aftnnrlc");			// Roman Lower Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"lower-roman-paren") == 0)
+	{
+		_rtf_keyword("aftnnrlc");			// Roman Lower Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"upper-roman") == 0)
+	{
+		_rtf_keyword("aftnnruc");			// Roman Upper Endnotes
+	}
+	else if(UT_XML_strcmp(pszEndnoteType,"upper-roman-paren") == 0)
+	{
+		_rtf_keyword("aftnnruc");			// Roman Upper Endnotes
+	}
+	else
+	{
+		_rtf_keyword("aftnnar");			// Numeric Endnotes
+	}
+
+	const XML_Char * pszTmp = NULL;
+	pDocAP->getProperty("document-footnote-initial", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		_rtf_keyword("ftnstart",atoi(pszTmp));			// First footnote
+	}
+	else
+	{
+		_rtf_keyword("ftnstart",1);			// First footnote
+	}
+
+	pDocAP->getProperty("document-footnote-restart-section", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		if(UT_XML_strcmp(pszTmp,"1") == 0)
+		{
+			_rtf_keyword("ftnrestart");			// footnote restarts each section
+		}
+	}
+
+	pDocAP->getProperty("document-footnote-restart-page", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		if(UT_XML_strcmp(pszTmp,"1") == 0)
+		{
+			_rtf_keyword("ftnrstpg");			// footnote restarts each page
+		}
+	}
+
+	pDocAP->getProperty("document-endnote-initial", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		_rtf_keyword("aftnstart", atoi(pszTmp)); // initial endnote value
+	}
+	pDocAP->getProperty("document-endnote-restart-section", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		if(UT_XML_strcmp(pszTmp,"1") == 0)
+		{
+			_rtf_keyword("aftnrestart"); // restart endnotes each section
+		}
+	}
+
+	pDocAP->getProperty("document-endnote-place-endsection", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		if(UT_XML_strcmp(pszTmp,"1") == 0)
+		{
+			_rtf_keyword("aendnotes"); // endnotes at end of section
+		}
+	}
+
+	pDocAP->getProperty("document-endnote-place-enddoc", (const XML_Char *&)pszTmp);
+	if(pszTmp && pszTmp[0])
+	{
+		if(UT_XML_strcmp(pszTmp,"1") == 0)
+		{
+			_rtf_keyword("aenddoc"); // endnotes at end of document
+		}
+	}
+//
+// Write out the facingp and titlepg keywords so that we can export our fancy
+// header-footers to RTF
+//
+	_rtf_keyword("facingp"); // Allow odd-even headers/footers
+	_rtf_keyword("titlepg"); // Allow first page headers/footers
 	return (m_error == 0);
 }
 
@@ -683,8 +900,8 @@ bool IE_Exp_RTF::_write_rtf_header(void)
  * !param szRTFName    The RTF keyword to use if the property
  *                     doesn't have the default value.
  */
-void IE_Exp_RTF::_write_prop_ifnotdefault(const PD_Style * pStyle, 
-					  const XML_Char * szPropName, 
+void IE_Exp_RTF::_write_prop_ifnotdefault(const PD_Style * pStyle,
+					  const XML_Char * szPropName,
 					  const char * szRTFName)
 {
 	const XML_Char * sz = NULL;
@@ -696,8 +913,8 @@ void IE_Exp_RTF::_write_prop_ifnotdefault(const PD_Style * pStyle,
 /*!
  * Write an RTF keyword if the given property is "yes".
  */
-void IE_Exp_RTF::_write_prop_ifyes(const PD_Style * pStyle, 
-				   const XML_Char * szPropName, 
+void IE_Exp_RTF::_write_prop_ifyes(const PD_Style * pStyle,
+				   const XML_Char * szPropName,
 				   const char * szRTFName)
 {
     const XML_Char * sz = NULL;
@@ -709,7 +926,7 @@ void IE_Exp_RTF::_write_prop_ifyes(const PD_Style * pStyle,
 /*
  * Used to hold tab information by _write_tabdef.
  */
-class _t 
+class _t
 {
 public:
 	_t(const char * szTL, const char * szTT, const char * szTK, UT_sint32 tp)
@@ -749,7 +966,7 @@ void IE_Exp_RTF::_write_tabdef(const char * szTabStops)
 		// TODO we should extract both of them and share the code.
 
 		UT_Vector vecTabs;
-		
+
 		const char* pStart = szTabStops;
 		while (*pStart)
 		{
@@ -788,7 +1005,7 @@ void IE_Exp_RTF::_write_tabdef(const char * szTabStops)
 
 			char pszPosition[32];
 			UT_uint32 iPosLen = p1 - pStart;
-			UT_ASSERT(iPosLen < 32);
+			UT_return_if_fail(iPosLen < 32);
 			UT_uint32 k;
 			for (k=0; k<iPosLen; k++)
 				pszPosition[k] = pStart[k];
@@ -796,7 +1013,7 @@ void IE_Exp_RTF::_write_tabdef(const char * szTabStops)
 			// convert position into twips
 			double dbl = UT_convertToPoints(pszPosition);
 			UT_sint32 d = (UT_sint32)(dbl * 20.0);
-			
+
 			_t * p_t = new _t(szTL,szTT,szTK,d);
 			vecTabs.addItem(p_t);
 
@@ -851,7 +1068,7 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 	if (szColor && UT_stricmp (szColor, "transparent") != 0)
 	{
 		ndxColor = _findColor((char*)szColor);
-		UT_ASSERT(ndxColor != -1);
+		UT_ASSERT_HARMLESS(ndxColor != -1);
 		if (ndxColor != 1) // white background, the default
 		{
 			_rtf_keyword("cb",ndxColor);
@@ -915,25 +1132,25 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 	if ( szLang )
 	  {
 	    UT_DEBUGMSG(("DOM: lang,lid = %s,%d\n", szLang, wvLangToLIDConverter(szLang)));
-	    _rtf_keyword("lang", wvLangToLIDConverter(szLang)); 
+	    _rtf_keyword("lang", wvLangToLIDConverter(szLang));
 	  }
 
-#ifdef BIDI_ENABLED
-
 	//###TF const XML_Char * szDir = apa.getProperty("dir");
 	const XML_Char * szDirOvrr = apa.getProperty("dir-override");
-	
+
 	bool bProceed = true;
 	if (szDirOvrr)
 	{
 		if (!UT_strcmp (szDirOvrr, "ltr"))
 		{
 			_rtf_keyword ("ltrch");
+			m_CharRTL = FRIBIDI_TYPE_LTR;
 			bProceed = false;
 		}
 		else if (!UT_strcmp (szDirOvrr, "rtl"))
 		{
 			_rtf_keyword ("rtlch");
+			m_CharRTL = FRIBIDI_TYPE_RTL;
 			bProceed  = false;
 		}
 	}
@@ -946,7 +1163,6 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 			_rtf_keyword ("rtlch");
 	}  */
 
-#endif
 	const XML_Char * szListTag = apa.getProperty("list-tag");
 	if (szListTag && *szListTag)
 	{
@@ -976,36 +1192,36 @@ void IE_Exp_RTF::_write_style_fmt(const PD_Style * pStyle)
     _write_prop_ifyes(pStyle, "keep-with-next", "keepn");
 
     const XML_Char * sz = NULL;
-    if (pStyle->getProperty((const XML_Char *)"text-align", sz)) 
+    if (pStyle->getProperty((const XML_Char *)"text-align", sz))
 	{
-		if (UT_strcmp(sz, "left") == 0) 
+		if (UT_strcmp(sz, "left") == 0)
 		{
 			// Default, so no need to print anything
-		} 
-		else if (UT_strcmp(sz, "right") == 0) 
+		}
+		else if (UT_strcmp(sz, "right") == 0)
 		{
 			_rtf_keyword("qr");
-		} 
-		else if (UT_strcmp(sz, "center") == 0) 
+		}
+		else if (UT_strcmp(sz, "center") == 0)
 		{
 			_rtf_keyword("qc");
 		}
-		else if (UT_strcmp(sz, "justify") == 0) 
+		else if (UT_strcmp(sz, "justify") == 0)
 		{
 			_rtf_keyword("qj");
-		} 
-		else 
+		}
+		else
 		{
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  UT_ASSERT_NOT_REACHED();
 		}
     }
 
     const XML_Char * szLineHeight = NULL;
     if (pStyle->getProperty((const XML_Char *) "line-height", szLineHeight)
-		&& strcmp(szLineHeight,"1.0") != 0) 
+		&& strcmp(szLineHeight,"1.0") != 0)
 	{
 		double f = UT_convertDimensionless(szLineHeight);
-		if (f != 0.0) 
+		if (f != 0.0)
 		{
 			UT_sint32 dSpacing = (UT_sint32)(f * 240.0);
 			_rtf_keyword("sl",dSpacing);
@@ -1023,7 +1239,7 @@ void IE_Exp_RTF::_write_style_fmt(const PD_Style * pStyle)
 
     // tabdef
     if (pStyle->getProperty((const XML_Char *) "tabstops", sz)) _write_tabdef(sz);
-    
+
 
     // shading
 
@@ -1035,12 +1251,12 @@ void IE_Exp_RTF::_write_style_fmt(const PD_Style * pStyle)
  * This is just a pair: style and style number. It is used for
  * two-way association of PD_Style and RTF style number.
  */
-struct NumberedStyle 
+struct NumberedStyle
 {
     const PD_Style * pStyle;
     UT_uint32 n;
 
-    NumberedStyle(const PD_Style * pStyle, UT_uint32 n) : 
+    NumberedStyle(const PD_Style * pStyle, UT_uint32 n) :
 	pStyle(pStyle), n(n) {}
 };
 
@@ -1052,6 +1268,11 @@ void IE_Exp_RTF::_clearStyles()
     UT_HASH_PURGEDATA(NumberedStyle *, &m_hashStyles, delete);
 }
 
+#ifdef _MSC_VER	// MSVC++ warns about 'e' : unreferenced local variable
+#pragma warning(disable: 4101)
+#endif
+
+
 /*!
  * Select styles for export. This inserts all styles to be exported
  * into the style hash. Also, it makes sure that all fonts used in
@@ -1067,13 +1288,13 @@ void IE_Exp_RTF::_selectStyles()
     const PD_Style * pStyle;
 	UT_Vector vecStyles;
 	getDoc()->getAllUsedStyles(&vecStyles);
-    for (i = 0; getDoc()->enumStyles(i, &szName, &pStyle); ++i) 
+    for (i = 0; getDoc()->enumStyles(i, &szName, &pStyle); ++i)
 	{
 	  // DOM: hack for 2069 - we'll export all styles instead of just
 	  // user-defined styles and used styles. To fix it (and export fewer
 	  // styles in general) make this routine recursive to include
 	  // parent (basedon) styles as well...
-	  if (true /* pStyle->isUserDefined() || (vecStyles.findItem((void *) pStyle) >= 0)*/) 
+	  if (true /* pStyle->isUserDefined() || (vecStyles.findItem((void *) pStyle) >= 0)*/)
 		{
 //
 // Add this style to the hash
@@ -1088,7 +1309,7 @@ void IE_Exp_RTF::_selectStyles()
 					if (_findFont(&fi) == -1)
 						_addFont(&fi);
 				}
-				UT_CATCH(_rtf_no_font e)
+				UT_CATCH(UT_CATCH_ANY)
 				{
 				}
 				UT_END_CATCH
@@ -1101,7 +1322,7 @@ void IE_Exp_RTF::_selectStyles()
 					if (_findFont(&fi) == -1)
 						_addFont(&fi);
 				}
-				UT_CATCH(_rtf_no_font e)
+				UT_CATCH(UT_CATCH_ANY)
 				{
 				}
 				UT_END_CATCH
@@ -1131,7 +1352,7 @@ UT_uint32 IE_Exp_RTF::_getStyleNumber(const XML_Char * szStyle)
 		szStyle = "Normal";
 	}
 	NumberedStyle * pns = (NumberedStyle*)m_hashStyles.pick(szStyle);
-	UT_ASSERT(pns);
+	UT_ASSERT_HARMLESS(pns);
 	if(pns != NULL )
 	{
 		return pns->n;
@@ -1157,40 +1378,40 @@ void IE_Exp_RTF::_write_stylesheets(void)
 
     UT_StringPtrMap::UT_Cursor hc(&m_hashStyles);
     const NumberedStyle * pns;
-    for (pns = reinterpret_cast<const NumberedStyle *>(hc.first()); 
-		 hc.is_valid(); 
-		 pns = reinterpret_cast<const NumberedStyle *>(hc.next())) 
+    for (pns = reinterpret_cast<const NumberedStyle *>(hc.first());
+		 hc.is_valid();
+		 pns = reinterpret_cast<const NumberedStyle *>(hc.next()))
 	{
 		const PD_Style * pStyle = pns->pStyle;
 		_rtf_nl();
 		_rtf_open_brace();
-		
-		if (pStyle->isCharStyle()) 
+
+		if (pStyle->isCharStyle())
 		{
 			_rtf_keyword("*");
 			_rtf_keyword("cs", pns->n);
-		} 
-		else 
+		}
+		else
 		{
 			_rtf_keyword("s", pns->n);
 		}
-		
+
 		_write_style_fmt(pStyle);
-		
+
 		const PD_Style * pStyleBasedOn =  reinterpret_cast<const PD_Style *> (const_cast<PD_Style *>(pStyle)->getBasedOn());
 		// TODO: Can this really return NULL?
-		if (pStyleBasedOn != NULL) 
+		if (pStyleBasedOn != NULL)
 		{
 			_rtf_keyword("sbasedon", _getStyleNumber(pStyleBasedOn));
 		}
-		
+
 		const PD_Style * pStyleNext = reinterpret_cast<const PD_Style *> (const_cast<PD_Style *>(pStyle)->getFollowedBy());
 		// TODO: Can this really return NULL?
-		if (pStyleNext != NULL) 
+		if (pStyleNext != NULL)
 		{
 			_rtf_keyword("snext", _getStyleNumber(pStyleNext));
 		}
-	
+
 		_rtf_chardata(pStyle->getName(), strlen(pStyle->getName()));
 		_rtf_chardata(";",1);
 		_rtf_close_brace();
@@ -1200,7 +1421,7 @@ void IE_Exp_RTF::_write_stylesheets(void)
 }
 
 /*!
- * Write the listatble group of the RTF header. 
+ * Write the listatble group of the RTF header.
  */
 void IE_Exp_RTF::_write_listtable(void)
 {
@@ -1252,7 +1473,7 @@ void IE_Exp_RTF::_write_listtable(void)
 //
 // OK now fill the MultiLevel list structure.
 //
-	
+
 	for(k=0; k < m_vecMultiLevel.getItemCount(); k++)
 	{
 		pList97 = (ie_exp_RTF_MsWord97ListMulti *) m_vecMultiLevel.getNthItem(k);
@@ -1277,7 +1498,7 @@ void IE_Exp_RTF::_write_listtable(void)
 				pList97->addLevel(depth, pCur97);
 			}
 			else
-			{	
+			{
 				bFoundAtPrevLevel = false;
 				for(i=0; i < iCount; i++)
 				{
@@ -1359,47 +1580,47 @@ void IE_Exp_RTF::_write_listtable(void)
 /*!
  * Get ith Multilevel list
  */
-ie_exp_RTF_MsWord97ListMulti * IE_Exp_RTF::getNthMultiLevel(UT_uint32 i) const 
-{ 
+ie_exp_RTF_MsWord97ListMulti * IE_Exp_RTF::getNthMultiLevel(UT_uint32 i) const
+{
 	return (ie_exp_RTF_MsWord97ListMulti *) m_vecMultiLevel.getNthItem(i);
-} 
+}
 
 /*!
  * Get ith Simple list
  */
-ie_exp_RTF_MsWord97ListSimple *  IE_Exp_RTF::getNthSimple(UT_uint32 i) const 
-{ 
+ie_exp_RTF_MsWord97ListSimple *  IE_Exp_RTF::getNthSimple(UT_uint32 i) const
+{
 	return (ie_exp_RTF_MsWord97ListSimple *) m_vecSimpleList.getNthItem(i);
-} 
+}
 
 /*!
  * Get ith Overide
  */
-ie_exp_RTF_ListOveride *  IE_Exp_RTF::getNthOveride(UT_uint32 i) const 
-{ 
+ie_exp_RTF_ListOveride *  IE_Exp_RTF::getNthOveride(UT_uint32 i) const
+{
 	return (ie_exp_RTF_ListOveride *) m_vecOverides.getNthItem(i);
 }
 
 /*!
  * Get Number of multilevel lists in the document
  */
-UT_uint32  IE_Exp_RTF::getMultiLevelCount(void) const 
-{ 
+UT_uint32  IE_Exp_RTF::getMultiLevelCount(void) const
+{
 	return m_vecMultiLevel.getItemCount();
 }
 
 /*!
  * Get Number of simple lists in the document
  */
-UT_uint32  IE_Exp_RTF::getSimpleListCount(void) const 
-{ 
+UT_uint32  IE_Exp_RTF::getSimpleListCount(void) const
+{
 	return m_vecSimpleList.getItemCount();
 }
 /*!
  * Get Number of overides in the document
  */
-UT_uint32  IE_Exp_RTF::getOverideCount(void) const 
-{ 
+UT_uint32  IE_Exp_RTF::getOverideCount(void) const
+{
 	return m_vecOverides.getItemCount();
 }
 
@@ -1431,11 +1652,16 @@ void IE_Exp_RTF::_output_MultiLevelRTF(ie_exp_RTF_MsWord97ListMulti * pMulti)
 {
 	_rtf_open_brace();
 	_rtf_keyword("list");
+#if 0
 	UT_uint32 tempID = UT_rand();
 	while(tempID < 10000)
 	{
 		tempID = UT_rand();
 	}
+#else
+	UT_uint32 tempID = getDoc()->getUID(UT_UniqueId::List);
+#endif
+	
 	_rtf_keyword("listtemplateid",tempID);
 	UT_uint32 i = 0;
 	fl_AutoNum * pAuto = NULL;
@@ -1450,11 +1676,11 @@ void IE_Exp_RTF::_output_MultiLevelRTF(ie_exp_RTF_MsWord97ListMulti * pMulti)
 //
 // Strategy: Dump out all the list info for the first list in each level. Then
 // use the overides to redefine subsequent lists at each level.
-// 
+//
 			pAuto = pList97->getAuto();
 			if(i==0 && pAuto->getParent() != NULL)
 			{
-				UT_ASSERT(0);
+				UT_ASSERT_NOT_REACHED();
 			}
 			_output_ListRTF(pAuto,i);
 		}
@@ -1484,7 +1710,7 @@ void IE_Exp_RTF::_output_LevelText(fl_AutoNum * pAuto, UT_uint32 iLevel, UT_UCSC
 	_rtf_open_brace();
 	_rtf_keyword("leveltext");
 	if(bulletsym == 0)
-	{ 
+	{
 		_generate_level_Text(pAuto,LevelText,LevelNumbers,lenText,ifoundLevel);
 		UT_String tmp;
 		_rtf_nonascii_hex2(lenText,tmp);
@@ -1690,11 +1916,9 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 	case UPPERCASE_LIST:
 		Param = 3;
 		break;
-#ifdef BIDI_ENABLED
 	case HEBREW_LIST:
 		Param = 45;
 		break;
-#endif
 	case LOWERCASE_LIST:
 		Param = 4;
 		break;
@@ -1770,14 +1994,12 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 	if(pAuto == NULL)
 	{
 		float marg = LIST_DEFAULT_INDENT;
-		float indent =  LIST_DEFAULT_INDENT_LABEL;
+		float indent =  (float)LIST_DEFAULT_INDENT_LABEL;
 		UT_String smarg;
 		UT_String sindent;
-		marg = (((float) iLevel) +1.0) * marg;
-		char *old_locale = setlocale (LC_NUMERIC, "C");
+		marg = (((float) iLevel) +1.0f) * marg;
 		UT_String_sprintf(smarg,"%fin",marg);
 		UT_String_sprintf(sindent,"%fin",indent);
-		setlocale (LC_NUMERIC, old_locale);
 		_rtf_keyword_ifnotdefault_twips("li",(char*)smarg.c_str(),0);
 		_rtf_keyword_ifnotdefault_twips("fi",(char*)sindent.c_str(),0);
 	}
@@ -1795,12 +2017,12 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 			if(bres)
 			{
 				_rtf_keyword_ifnotdefault_twips("fi",(char*)szIndent,0);
-			}		
+			}
 			bres = getDoc()->getPropertyFromSDH(sdh,"margin-left",&szAlign);
 			if(bres)
 			{
 				_rtf_keyword_ifnotdefault_twips("li",(char*)szAlign,0);
-			}		
+			}
 		}
 	}
 
@@ -1816,7 +2038,7 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 		UT_sint32 ifont = _findFont(&fi);
 		if(ifont < 0)
 		{
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  UT_ASSERT_NOT_REACHED();
 			ifont = 0;
 		}
 		_rtf_keyword("f",ifont);
@@ -1832,11 +2054,15 @@ void IE_Exp_RTF::_output_SimpleListRTF(ie_exp_RTF_MsWord97ListSimple * pSimple)
 {
 	_rtf_open_brace();
 	_rtf_keyword("list");
+#if 0
 	UT_uint32 tempID = UT_rand();
 	while(tempID < 10000)
 	{
 		tempID = UT_rand();
 	}
+#else
+	UT_uint32 tempID = getDoc()->getUID(UT_UniqueId::List);
+#endif
 	_rtf_keyword("listtemplateid",tempID);
 	_rtf_keyword("listsimple");
 	fl_AutoNum * pAuto = pSimple->getAuto();
@@ -1844,7 +2070,7 @@ void IE_Exp_RTF::_output_SimpleListRTF(ie_exp_RTF_MsWord97ListSimple * pSimple)
 	_rtf_keyword("listlevel");
 //
 // Strategy: Dump out all the list info for the first list in each level.
-// 
+//
 	_output_ListRTF(pAuto,0);
 	_rtf_close_brace();
 	_rtf_keyword("listid",pSimple->getID());
@@ -1869,7 +2095,7 @@ void IE_Exp_RTF::_output_OveridesRTF(ie_exp_RTF_ListOveride * pOver, UT_uint32 i
 	_rtf_keyword("listid",pTop->getID());
 //
 // Strategy: Dump out all the list info for the first list in each level.
-// 
+//
 	_output_ListRTF(pAuto,0);
 	_rtf_keyword("ls",pOver->getOverideID());
 	_rtf_close_brace();
@@ -1888,7 +2114,7 @@ bool IE_Exp_RTF::_write_rtf_trailer(void)
  */
 UT_sint32 IE_Exp_RTF::_findFont(const _rtf_font_info * pfi) const
 {
-	UT_ASSERT(pfi);
+	UT_return_val_if_fail(pfi, -1);
 
 	UT_uint32 k;
 	UT_uint32 kLimit = m_vecFonts.getItemCount();
@@ -1912,7 +2138,7 @@ UT_sint32 IE_Exp_RTF::_findFont(const s_RTF_AttrPropAdapter * apa) const
 	    ifont = _findFont(&fi);
 	    return ifont;
 	  }
-	UT_CATCH(_rtf_no_font e)
+	UT_CATCH(UT_CATCH_ANY)
 	  {
 	    return -1;
 	  }
@@ -1925,7 +2151,7 @@ UT_sint32 IE_Exp_RTF::_findFont(const s_RTF_AttrPropAdapter * apa) const
  */
 void IE_Exp_RTF::_addFont(const _rtf_font_info * pfi)
 {
-	UT_ASSERT(pfi && (_findFont(pfi)==-1));
+	UT_return_if_fail(pfi && (_findFont(pfi)==-1));
 	_rtf_font_info * pNew = new _rtf_font_info(*pfi);
 	if (pNew)
 		m_vecFonts.addItem(pNew);
@@ -1933,7 +2159,7 @@ void IE_Exp_RTF::_addFont(const _rtf_font_info * pfi)
 	return;
 }
 
-_rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldFont) 
+_rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldFont)
 	UT_THROWS((_rtf_no_font))
 {
     // Not a typo. The AbiWord "font-family" property is what RTF
@@ -1959,7 +2185,7 @@ _rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldF
 	{
 		_rtf_no_font e;
 		UT_THROW(e);
-	}    
+	}
 
     static const char * t_ff[] = { "fnil", "froman", "fswiss", "fmodern", "fscript", "fdecor", "ftech", "fbidi" };
     GR_Font::FontFamilyEnum ff;
@@ -1977,11 +2203,11 @@ _rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldF
 }
 
 
-_rtf_font_info::~_rtf_font_info(void) 
+_rtf_font_info::~_rtf_font_info(void)
 {
 }
 
-_rtf_font_info::_rtf_font_info(const char * szFontName) 
+_rtf_font_info::_rtf_font_info(const char * szFontName)
 	UT_THROWS((_rtf_no_font))
 {
     // Not a typo. The AbiWord "font-family" property is what RTF
@@ -1990,7 +2216,7 @@ _rtf_font_info::_rtf_font_info(const char * szFontName)
 	{
 		_rtf_no_font e;
 		UT_THROW(e);
-	}    
+	}
 
 	m_szName = szFontName;
 
