@@ -1,20 +1,20 @@
 /* -*- c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*- */
-/* AbiWord
- * Copyright (C) 1999 AbiSource, Inc.
+/* Copyright (C) 1999 AbiSource, Inc.
+ * Copyright (C) 2003 Tomas Frydrych <tomas@frydrych.uklinux.net>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -35,13 +35,14 @@
 #include "fl_AutoLists.h"
 #include "fl_AutoNum.h"
 #include "fl_BlockLayout.h"
-
-#ifdef BIDI_ENABLED
-#include "fribidi.h"
-#endif
+#include <fribidi/fribidi.h>
+#include  "ie_Table.h"
 
 class IE_Imp_RTF;
 class RTF_msword97_list;
+class ie_imp_cell;
+class ie_imp_table;
+class ie_imp_table_control;
 
 // Font table entry
 struct ABI_EXPORT RTFFontTableItem
@@ -79,18 +80,20 @@ public:
 	bool	m_topline;
 	bool	m_botline;
 	bool    m_superscript;
-	double    m_superscript_pos;       // unit is pt. if 0.0, ignore
+	double  m_superscript_pos;       // unit is pt. if 0.0, ignore
 	bool    m_subscript;
-	double    m_subscript_pos;         // unit is pt. if 0.0, ignore	
-	double    m_fontSize;			// font size in points
+	double  m_subscript_pos;         // unit is pt. if 0.0, ignore
+	double  m_fontSize;			// font size in points
 	UT_uint32    m_fontNumber;		// index into font table
 	bool    m_hasColour;        // if false, ignore colour number
     UT_uint32    m_colourNumber;	// index into colour table
 	bool    m_hasBgColour; // if false, ignore colour number
-	UT_uint32    m_bgcolourNumber; // index into colour table
+	UT_uint32  m_bgcolourNumber; // index into colour table
 	UT_sint32  m_styleNumber ; //index into the style table
 	UT_uint32  m_listTag; // tag for lists to hang off
 	const char * m_szLang;
+	bool    m_RTL;
+	FriBidiCharType m_dirOverride;
 };
 
 class ABI_EXPORT RTFProps_bCharProps
@@ -110,7 +113,7 @@ public:
 	bool bm_superscript;
 	bool bm_superscript_pos;       // unit is pt. if 0.0, ignore
 	bool bm_subscript;
-	bool bm_subscript_pos;         // unit is pt. if 0.0, ignore	
+	bool bm_subscript_pos;         // unit is pt. if 0.0, ignore
 	bool bm_fontSize;			// font size in points
 	bool bm_fontNumber;		// index into font table
 	bool    bm_hasColour;        // if false, ignore colour number
@@ -118,10 +121,12 @@ public:
 	bool bm_hasBgColour; // if false, ignore colour number
 	bool bm_bgcolourNumber; // index into colour table
 	bool bm_listTag; // tag for lists to hanfg off
+	bool bm_RTL;
+	bool bm_dirOverride;
 };
 
 struct ABI_EXPORT _rtfListTable
-{      
+{
 	UT_uint32 start_value;
 	UT_uint32 level;
 	bool bullet;
@@ -148,6 +153,33 @@ struct ABI_EXPORT _rtfListTable
 	UT_uint32 iWord97Level;
 };
 
+enum rtfCellBorder
+{
+	rtfCellBorderTop,
+	rtfCellBorderLeft,
+	rtfCellBorderRight,
+	rtfCellBorderBot
+};
+
+struct ABI_EXPORT RTFProps_CellProps
+{
+	RTFProps_CellProps();
+	RTFProps_CellProps & operator=(const RTFProps_CellProps&);
+	bool      m_bVerticalMerged;
+	bool      m_bVerticalMergedFirst;
+	bool      m_bHorizontalMerged;
+	bool      m_bHorizontalMergedFirst;
+	UT_String m_sCellProps;
+	rtfCellBorder  m_iCurBorder;
+	UT_sint32 m_iCellx;
+};
+
+struct ABI_EXPORT RTFProps_TableProps
+{
+	RTFProps_TableProps();
+	RTFProps_TableProps& operator=(const RTFProps_TableProps&);
+	bool      m_bAutoFit;
+};
 
 // Paragraph properties
 struct ABI_EXPORT RTFProps_ParaProps
@@ -179,14 +211,14 @@ struct ABI_EXPORT RTFProps_ParaProps
 	UT_uint32       m_startValue;         // Start value of the list
 	eTabType        m_curTabType;        // Current Tab type
 	eTabLeader      m_curTabLeader;       // Current Tab Leader
-	UT_uint32       m_iOverride;          // 1's index to overide table
+	UT_uint32       m_iOverride;          // 1's index to override table
 	UT_uint32       m_iOverrideLevel;     // 0's index to the level
-	_rtfListTable   m_rtfListTable; 
+	_rtfListTable   m_rtfListTable;
 	UT_sint32  m_styleNumber ; //index into the style table
-#ifdef BIDI_ENABLED
-	FriBidiCharType m_dom_dir;
-#endif
-};                  
+	bool            m_RTL;
+	UT_sint32       m_tableLevel; //nesting level of the paragram in a table.
+	bool            m_bInTable; // true if paragraph is in a table
+};
 
 // These are set true if changed in list definitions.
 class ABI_EXPORT RTFProps_bParaProps
@@ -208,10 +240,8 @@ public:
 	bool        bm_tabLeader;
 	bool        bm_curTabType;        // Current Tab type
 	bool        bm_curTabLeader;       // Current Tab Leader
-	bool        bm_rtfListTable; 
-#ifdef BIDI_ENABLED
-	bool bm_dom_dir;
-#endif
+	bool        bm_rtfListTable;
+	bool        bm_dom_dir;
 };
 
 
@@ -231,20 +261,19 @@ class ABI_EXPORT RTF_msword97_level
 public:
     RTF_msword97_level(	RTF_msword97_list * pmsword97List, UT_uint32 level);
 	~RTF_msword97_level();
-	void buildAbiListProperties( const char ** szListID, 
-								 const char ** szParentID, 
+	void buildAbiListProperties( const char ** szListID,
+								 const char ** szParentID,
 								 const char ** szLevel,
-								 const char ** szStartat, 
-								 const char ** szFieldFont, 
-								 const char ** szListDelim, 
-								 const char ** szListDecimal, 
-								 const char ** szAlign, 
-								 const char ** szIndent, 
+								 const char ** szStartat,
+								 const char ** szFieldFont,
+								 const char ** szListDelim,
+								 const char ** szListDecimal,
+								 const char ** szAlign,
+								 const char ** szIndent,
 								 const char ** szListStyle);
 	bool ParseLevelText(const UT_String & szLevelText,const UT_String & szLevelNumbers, UT_uint32 iLevel);
 	UT_sint32 m_levelStartAt;
 	UT_uint32 m_AbiLevelID;
-	static UT_uint32 m_sLastAssignedLevelID;
 	static UT_uint32 m_sPreviousLevel;
 	UT_uint32 m_RTFListType;
 	UT_String m_listDelim;
@@ -269,7 +298,6 @@ public:
 	UT_uint32 m_RTF_listID;
 	UT_uint32 m_RTF_listTemplateID;
 	RTF_msword97_level * m_RTF_level[9];
-private:
 	IE_Imp_RTF * m_pie_rtf;
 };
 
@@ -279,15 +307,15 @@ class ABI_EXPORT RTF_msword97_listOverride
 public:
 	RTF_msword97_listOverride(	IE_Imp_RTF * pie_rtf);
     ~RTF_msword97_listOverride();
-	void buildAbiListProperties( const char ** szListID, 
-								 const char ** szParentID, 
+	void buildAbiListProperties( const char ** szListID,
+								 const char ** szParentID,
 								 const char ** szLevel,
-								 const char ** szStartat, 
-								 const char ** szFieldFont, 
-								 const char ** szListDelim, 
-								 const char ** szListDecimal, 
-								 const char ** szAlign, 
-								 const char ** szIndent, 
+								 const char ** szStartat,
+								 const char ** szFieldFont,
+								 const char ** szListDelim,
+								 const char ** szListDecimal,
+								 const char ** szAlign,
+								 const char ** szIndent,
 								 const char ** szListStyle,
 								 UT_uint32 iLevel);
 	UT_uint32 m_RTF_listID;
@@ -340,7 +368,7 @@ private:
 struct ABI_EXPORT RTFProps_ImageProps
 {
 	enum IPSizeType { ipstNone, ipstGoal, ipstScale };
-	
+
 	RTFProps_ImageProps ();
 	IPSizeType sizeType;
 	UT_uint32 wGoal;
@@ -363,16 +391,15 @@ struct ABI_EXPORT RTFProps_SectionProps
 	bool m_bColumnLine;
 	ESectionBreak	m_breakType;
 	EPageNumber		m_pageNumFormat;
-	UT_sint32       m_leftMargTwips;     
-	UT_sint32       m_rightMargTwips;     
-	UT_sint32       m_topMargTwips;     
-	UT_sint32       m_bottomMargTwips;     
-	UT_sint32       m_headerYTwips;     
-	UT_sint32       m_footerYTwips;     
-	UT_sint32       m_colSpaceTwips;     
-#ifdef BIDI_ENABLED
+	UT_sint32       m_leftMargTwips;
+	UT_sint32       m_rightMargTwips;
+	UT_sint32       m_topMargTwips;
+	UT_sint32       m_bottomMargTwips;
+	UT_sint32       m_headerYTwips;
+	UT_sint32       m_footerYTwips;
+	UT_sint32       m_gutterTwips;
+	UT_sint32       m_colSpaceTwips;
 	FriBidiCharType m_dir;
-#endif
 };
 
 
@@ -385,27 +412,27 @@ struct ABI_EXPORT RTFProps_SectionProps
 struct ABI_EXPORT RTFHdrFtr
 {
 	enum HdrFtrType {hftNone,
-					 hftHeader, 
-					 hftHeaderEven, 
-					 hftHeaderFirst, 
-					 hftHeaderLast,  
-					 hftFooter, 
-					 hftFooterEven, 
-					 hftFooterFirst, 
+					 hftHeader,
+					 hftHeaderEven,
+					 hftHeaderFirst,
+					 hftHeaderLast,
+					 hftFooter,
+					 hftFooterEven,
+					 hftFooterFirst,
 					 hftFooterLast };
 
 	RTFHdrFtr () : m_type(hftNone), m_id(0), m_buf(1024) {}
-	
+
 	HdrFtrType      m_type;
 	UT_uint32       m_id;
 	UT_ByteBuf      m_buf;
-};	
+};
 
 // RTFStateStore
 struct ABI_EXPORT RTFStateStore
 {
 	RTFStateStore();
-	
+
 //	RTFStateStore& operator=(const RTFStateStore&);
 
 	enum DestinationStateTypes { rdsNorm, rdsSkip, rdsFootnote, rdsHeader, rdsFooter, rdsField };
@@ -416,6 +443,8 @@ struct ABI_EXPORT RTFStateStore
 	RTFProps_CharProps		m_charProps;			// Character properties
 	RTFProps_ParaProps		m_paraProps;			// Paragraph properties
 	RTFProps_SectionProps	m_sectionProps;			// Section properties
+	RTFProps_CellProps      m_cellProps;            // Cell properties
+    RTFProps_TableProps     m_tableProps;           // Table properties
 
 	UT_uint32				m_unicodeAlternateSkipCount;	// value of N in "\ucN"
 	UT_uint32				m_unicodeInAlternate;			// chars left in alternate "\u<u><A>"
@@ -438,6 +467,23 @@ typedef struct doc_prop
 */
 
 
+// 
+// Little class to be used for tracking pasted table state.
+//
+class ABI_EXPORT ABI_Paste_Table
+{
+public:
+	ABI_Paste_Table(void);
+    virtual ~ABI_Paste_Table(void);
+	bool                  m_bHasPastedTableStrux;
+	bool                  m_bHasPastedCellStrux;
+	UT_sint32             m_iRowNumberAtPaste;
+	bool                  m_bHasPastedBlockStrux;
+	UT_sint32             m_iMaxRightCell;
+	UT_sint32             m_iCurRightCell;
+	UT_sint32             m_iCurTopCell;
+	bool                  m_bPasteAfterRow;
+};
 
 // The importer/reader for Rich Text Format files
 
@@ -446,10 +492,12 @@ class ABI_EXPORT IE_Imp_RTF_Sniffer : public IE_ImpSniffer
 	friend class IE_Imp;
 
 public:
-	IE_Imp_RTF_Sniffer() {}
+	IE_Imp_RTF_Sniffer();
 	virtual ~IE_Imp_RTF_Sniffer() {}
 
-	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
+	UT_Confidence_t supportsMIME (const char * szMIME);
+
+	virtual UT_Confidence_t recognizeContents (const char * szBuf,
 									UT_uint32 iNumbytes);
 	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
@@ -466,29 +514,35 @@ public:
 	IE_Imp_RTF(PD_Document * pDocument);
 	~IE_Imp_RTF();
 
+	virtual bool supportsLoadStylesOnly() {return true;}
 	virtual UT_Error	importFile(const char * szFilename);
 	virtual void		pasteFromBuffer(PD_DocumentRange * pDocRange,
-										unsigned char * pData, UT_uint32 lenData, const char * szEncoding = 0);
+										const unsigned char * pData, UT_uint32 lenData, const char * szEncoding = 0);
 	UT_sint32 get_vecWord97ListsCount(void) { return m_vecWord97Lists.getItemCount();}
-	RTF_msword97_list *  get_vecWord97NthList(UT_sint32 i) { return (RTF_msword97_list *) m_vecWord97Lists.getNthItem(i);}
+	RTF_msword97_list *  get_vecWord97NthList(UT_sint32 i) { return static_cast<RTF_msword97_list *>(m_vecWord97Lists.getNthItem(i));}
     bool  isWord97Lists(void) const { return (m_vecWord97Lists.getItemCount() > 0);}
+
+	enum PictFormat {
+		picNone,
+		picPNG,
+		picJPEG,
+		picBMP,
+		picWMF,
+		picPICT,
+		picEMF,
+		picGIF
+	};
+
 protected:
 	UT_Error			_parseFile(FILE * fp);
 	UT_Error			_writeHeader(FILE * fp);
 	UT_Error            _parseHdrFtr ();
+	UT_Error            _parseText();
+	
 
 // importer helper methods
 private:
-	enum PictFormat {
-		picNone, 
-		picPNG,
-		picJPEG, 
-		picBMP, 
-		picWMF, 
-		picPICT, 
-		picEMF, 
-		picGIF
-	};
+
 
 	typedef enum {
 		RBT_START = 0,
@@ -506,7 +560,7 @@ private:
 	bool ReadCharFromFileWithCRLF(unsigned char* pCh);
 	bool ReadCharFromFile(unsigned char* pCh);
 	bool SkipBackChar(unsigned char ch);
-	bool ReadKeyword(unsigned char* pKeyword, long* pParam, bool* pParamUsed, 
+	bool ReadKeyword(unsigned char* pKeyword, long* pParam, bool* pParamUsed,
 					 UT_uint32 keywordBuffLen);
 	bool TranslateKeyword(unsigned char* pKeyword, long param, bool fParam);
 	bool ReadColourTable();
@@ -515,16 +569,16 @@ private:
 	bool HandlePicture();
 	bool HandleObject();
 	bool HandleField();
-	bool HandleHyperlink();
 	bool HandleStyleDefinition(void);
 	bool HandleHeaderFooter(RTFHdrFtr::HdrFtrType hftype, UT_uint32 & headerID);
 	bool SkipCurrentGroup(bool bConsumeLastBrace = false);
 	bool StuffCurrentGroup(UT_ByteBuf & buf);
-	bool LoadPictData(PictFormat format, const char * image_name, 
-					  struct RTFProps_ImageProps & imgProps);
+	bool LoadPictData(PictFormat format, const char * image_name,
+					  struct RTFProps_ImageProps & imgProps,
+					  bool isBinary = false, long binaryLen = 0);
 	bool InsertImage (const UT_ByteBuf * buf, const char * image_name,
 					  const struct RTFProps_ImageProps & imgProps);
-	
+
 	RTFFontTableItem* GetNthTableFont(UT_uint32 fontNum);
 	UT_uint32 GetNthTableColour(UT_uint32 colNum);
 	UT_sint32 GetNthTableBgColour(UT_uint32 colNum);
@@ -537,13 +591,13 @@ private:
 	bool ParseCharParaProps( unsigned char * pKeyword, long param, bool fParam, RTFProps_CharProps * pChars, RTFProps_ParaProps * pParas, RTFProps_bCharProps * pbChars, RTFProps_bParaProps * pbParas);
 	bool ReadListOverrideTable(void);
 	bool HandleTableListOverride(void);
-	
-	bool buildAllProps( char * propBuffer,  RTFProps_ParaProps * pParas, 
-					   RTFProps_CharProps * pChars, 
-					   RTFProps_bParaProps * pbParas, 
+
+	bool buildAllProps( char * propBuffer,  RTFProps_ParaProps * pParas,
+					   RTFProps_CharProps * pChars,
+					   RTFProps_bParaProps * pbParas,
 					   RTFProps_bCharProps * pbChars);
 
-
+	
 	// Character property handlers
 	bool ResetCharacterAttributes();
 	bool buildCharacterProps(UT_String & propBuffer);
@@ -579,10 +633,39 @@ private:
 	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, eTabLeader tableader);
 	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, eTabLeader tabLeader,  RTFProps_ParaProps * pParas);
 
+// Paste AbiWord tables
+	bool HandleAbiTable(void);
+	bool HandleAbiCell(void);
+	bool HandleAbiEndTable(void);
+	bool HandleAbiEndCell(void);
 	bool HandleAbiLists(void);
-	bool HandleLists(_rtfListTable & rtfListTable );
-        UT_uint32 mapID(UT_uint32 id);
-	UT_uint32 mapParentID(UT_uint32 id);
+	bool isPastedTableOpen(void);
+	bool markPasteBlock(void);
+	bool isBlockNeededForPasteTable(void);
+
+	bool       HandleLists(_rtfListTable & rtfListTable );
+    UT_uint32  mapID(UT_uint32 id);
+	UT_uint32  mapParentID(UT_uint32 id);
+
+// Table methods
+    bool           ResetCellAttributes(void);
+	bool           ResetTableAttributes(void);
+    ie_imp_table * getTable(void);
+	ie_imp_cell *  getCell(void);
+	void           FlushCellProps(void);
+	void           FlushTableProps(void);
+	void           OpenTable(bool bDontFlush = false);
+	void           CloseTable(bool bForceClose = false);
+	void           SaveRowInfo(void);
+	void           RemoveRowInfo(void);
+    void           HandleCell(void);
+	void           HandleCellX(UT_sint32 cellx);
+    void           HandleRow(void);
+	void           HandleNote();
+	void           HandleNoteReference();
+	bool           bUseInsertNotAppend(void);
+// Little convience wrapper
+	void           _setStringProperty(UT_String & sPropString, const char * szProp, const char * szVal);
 
 	// Section property handlers
 	bool ApplySectionAttributes();
@@ -595,9 +678,13 @@ private:
 	    RTF_TOKEN_DATA,
 	    RTF_TOKEN_ERROR = -1
 	} RTFTokenType;
-	RTFTokenType NextToken (unsigned char *pKeyword, long* pParam, 
+	RTFTokenType NextToken (unsigned char *pKeyword, long* pParam,
 							bool* pParamUsed, UT_uint32 len, bool bIgnoreWhiteSpace=false);
 
+	UT_Error _isBidiDocument();
+	bool     _appendSpan();
+	bool     _insertSpan();
+	
 
 // import member vars
 private:
@@ -633,7 +720,7 @@ private:
 	UT_Vector m_styleTable;
 
 	struct _rtfAbiListTable
-	{      
+	{
 		UT_uint32 orig_id;
 		UT_uint32 orig_parentid;
 		UT_uint32 start_value;
@@ -643,7 +730,7 @@ private:
 		UT_uint32 mapped_parentid;
 	};
 	UT_Vector m_vecAbiListTable;
-	_rtfAbiListTable * getAbiList( UT_uint32 i) {return (_rtfAbiListTable *) m_vecAbiListTable.getNthItem(i);}
+	_rtfAbiListTable * getAbiList( UT_uint32 i) {return static_cast<_rtfAbiListTable *>(m_vecAbiListTable.getNthItem(i));}
 
 	RTF_msword97_listOverride* _getTableListOverride(UT_uint32 id);
 
@@ -652,25 +739,50 @@ private:
 	bool m_bisNOTList; // true if the current stream does not have  abi list extensions
 	bool m_bParaHasRTFList;
 	bool m_bParaHasRTFContinue;
- 
+
 	FILE* m_pImportFile;
 
-	unsigned char *		m_pPasteBuffer;
+	const unsigned char *		m_pPasteBuffer;
 	UT_uint32 			m_lenPasteBuffer;
-	unsigned char *		m_pCurrentCharInPasteBuffer;
+	const unsigned char *		m_pCurrentCharInPasteBuffer;
 	PT_DocPosition		m_dposPaste;
-	UT_uint32		deflangid;
-	UT_Mbtowc		m_mbtowc;
+	UT_uint32		    deflangid;
+	UT_UCS4_mbtowc		m_mbtowc;
 	bool                m_parsingHdrFtr;
 	UT_uint32           m_icurOverride;
 	UT_uint32           m_icurOverrideLevel;
 	UT_Vector           m_vecWord97Lists;
 	UT_Vector           m_vecWord97ListOverride;
 	void _appendHdrFtr ();
-	bool _appendField (const XML_Char *xmlField);
+	bool _appendField (const XML_Char *xmlField, const XML_Char ** pszAttribs=NULL);
 	XML_Char *_parseFldinstBlock (UT_ByteBuf & buf, XML_Char *xmlField, bool & isXML);
 	bool                m_bAppendAnyway;
 	RTFProps_SectionProps m_sectdProps ;
+	ie_imp_table_control  m_TableControl;
+	PL_StruxDocHandle     m_lastCellSDH;
+	bool                  m_bNestTableProps;
+	bool                  m_bParaWrittenForSection;
+	bool                  m_bCellBlank;
+	bool                  m_bEndTableOpen;
+	bool                  m_bInFootnote;
+	UT_uint32             m_iDepthAtFootnote;
+	UT_uint32             m_iLastFootnoteId;
+	UT_String             m_hyperlinkBase;
+	UT_uint32             m_iHyperlinkOpen;
+	bool                  m_bBidiMode;
+	UT_Stack              m_pasteTableStack;
+	bool                  m_bFootnotePending;
+	bool                  m_bFtnReferencePending;
+	bool                  m_bNoteIsFNote;
+	bool                  m_bStyleImportDone;
+	bool                  m_bCellHandled;
+	bool                  m_bContentFlushed;
+	bool                  m_bRowJustPassed;
+	UT_sint32             m_iStackLevelAtRow;
+	bool                  m_bDoCloseTable;
+	UT_sint32             m_iNoCellsSinceLastRow;
+	RTFStateStore         m_FootnoteRefState;
+	bool                  m_bFieldRecognized;
 };
 
 #endif /* IE_IMP_RTF_H */
