@@ -1,23 +1,25 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 2003 Tomas Frydrych <tomas@frydrych.uklinux.net> 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include "ut_types.h"
@@ -35,11 +37,16 @@
 #include "xap_DialogFactory.h"
 #include "xap_Dlg_Encoding.h"
 #include "ap_Prefs.h"
+#include <fribidi/fribidi.h>
+
+#include "pt_PieceTable.h"
+#include "pf_Frag_Strux.h"
 
 /*!
   Construct ImportStream
  */
 ImportStream::ImportStream() :
+	m_Mbtowc(XAP_EncodingManager::get_instance()->getNative8BitEncodingName()),
 	m_ucsLookAhead(0),
 	m_bEOF(false),
 	m_bRaw(false)
@@ -87,7 +94,7 @@ bool ImportStream::getChar(UT_UCSChar &ucs)
  */
 bool ImportStream::getRawChar(UT_UCSChar &ucs)
 {
-	wchar_t wc = 0;
+	UT_UCS4Char wc = 0;
 	unsigned char b;
 
 	if (m_bEOF)
@@ -115,7 +122,7 @@ bool ImportStream::getRawChar(UT_UCSChar &ucs)
 	// No, Private User Area is not evil!  Commenting it out for now.
 	/* UT_ASSERT(!((wc >= 0xDB80 && wc <= 0xDBFF)||(wc >= 0xE000 && wc <= 0xF8FF))); */
 	// AbiWord control characters
-	UT_ASSERT(wc != UCS_FIELDSTART && wc != UCS_FIELDEND && wc != UCS_BOOKMARKSTART && wc != UCS_BOOKMARKEND);
+	UT_ASSERT(wc < UCS_ABICONTROL_START || wc > UCS_ABICONTROL_END);
 	// Illegal characters
 	UT_ASSERT(wc != 0xFFFE && wc != 0xFFFF);
 
@@ -140,7 +147,7 @@ ImportStreamFile::ImportStreamFile(FILE *pFile) :
  */
 bool ImportStreamFile::_getByte(unsigned char &b)
 {
-	UT_ASSERT(m_pFile);
+	UT_return_val_if_fail(m_pFile, false);
 
 	return fread(&b, 1, sizeof(b), m_pFile) > 0;
 }
@@ -150,7 +157,7 @@ bool ImportStreamFile::_getByte(unsigned char &b)
  \param pClipboard Buffer to read from
  \param iLength Length of buffer
  */
-ImportStreamClipboard::ImportStreamClipboard(unsigned char *pClipboard, UT_uint32 iLength) :
+ImportStreamClipboard::ImportStreamClipboard(const unsigned char *pClipboard, UT_uint32 iLength) :
 	m_p(pClipboard),
 	m_pEnd(pClipboard + iLength)
 {
@@ -168,54 +175,38 @@ bool ImportStreamClipboard::_getByte(unsigned char &b)
 	return true;
 }
 
-/*!
-  Construct Inserter helper class
- \param pDocument Document to insert data into
- */
-Inserter::Inserter(PD_Document * pDocument) :
-	m_pDocument(pDocument),
-	m_bClipboard(false)
-{
-}
-
-/*!
-  Construct Inserter helper class
- \param pDocument Document to insert data into
- \param dPos Position in document to begin inserting at
- */
-Inserter::Inserter(PD_Document * pDocument, PT_DocPosition dPos) :
-	m_pDocument(pDocument),
-	m_bClipboard(true),
-	m_dPos(dPos)
-{
-}
-
 /*!
   Insert a Block into the document
 
  Uses appropriate function for clipboard or file
  */
-bool Inserter::insertBlock()
+bool IE_Imp_Text::_insertBlock()
 {
-	bool bRes;
+	bool ret = false;
+	m_bBlockDirectionPending = true;
+	m_bFirstBlockData = true;
 	
-	if (m_bClipboard)
-	{
-		bRes = m_pDocument->insertStrux(m_dPos, PTX_Block);
-		m_dPos++;
+	if (isClipboard ()) // intentional - don't append style
+						// information
+	{		
+		ret = appendStrux(PTX_Block, NULL);
 	}
 	else
-	  {
+	{
 	    // text gets applied in the Normal style
 	    const XML_Char * propsArray[3];
 	    propsArray[0] = "style";
 	    propsArray[1] = "Normal";
 	    propsArray[2] = 0;
 
-	    bRes = m_pDocument->appendStrux(PTX_Block, (const XML_Char **)propsArray);
-	  }
+	    ret = appendStrux(PTX_Block, static_cast<const XML_Char **>(&propsArray[0]));
+	}
 
-	return bRes;
+	pf_Frag * pf = getDoc()->getPieceTable()->getFragments().getLast();
+	UT_ASSERT( pf->getType() == pf_Frag::PFT_Strux );
+	m_pBlock = (pf_Frag_Strux *) pf;
+	UT_ASSERT( m_pBlock->getStruxType() == PTX_Block );
+	return ret;
 }
 
 /*!
@@ -224,26 +215,89 @@ bool Inserter::insertBlock()
 
  Uses appropriate function for clipboard or file
  */
-bool Inserter::insertSpan(UT_GrowBuf &b)
+bool IE_Imp_Text::_insertSpan(UT_GrowBuf &b)
 {
-	bool bRes;
-	
-	if (m_bClipboard)
+	UT_uint32 iLength = b.getLength();
+	const UT_UCS4Char * pData = (const UT_UCS4Char *)b.getPointer(0);
+
+	// handle block direction if needed ...
+	if(pData && m_bBlockDirectionPending)
 	{
-		bRes = m_pDocument->insertSpan(m_dPos, b.getPointer(0), b.getLength());
-		m_dPos += b.getLength();
+		const UT_UCS4Char * p = pData;
+
+		// we look for the first strong character
+		for(UT_uint32 i = 0; i < iLength; i++, p++)
+		{
+			FriBidiCharType type = fribidi_get_type(static_cast<FriBidiChar>(*p));
+
+			if(FRIBIDI_IS_STRONG(type))
+			{
+				m_bBlockDirectionPending = false;
+
+				// set 'dom-dir' property of the block ...
+				const XML_Char * propsArray[3];
+				propsArray[0] = "props";
+				propsArray[1] = NULL;
+				propsArray[2] = NULL;
+
+				UT_String props("dom-dir:");
+				
+				if(FRIBIDI_IS_RTL(type))
+					props += "rtl;text-align:right";
+				else
+					props += "ltr;text-align:left";
+
+				propsArray[1] = props.c_str();
+				
+				// we need to modify the existing formatting ...
+				appendStruxFmt(m_pBlock, static_cast<const XML_Char **>(&propsArray[0]));
+			
+				// if this is the first data in the block and the first
+				// character is LRM or RLM followed by a strong character,
+				// then we will remove it
+				if(m_bFirstBlockData && i==0 && iLength > 1 && (*p == UCS_LRM || *p == UCS_RLM))
+				{
+					FriBidiCharType next_type = fribidi_get_type(static_cast<FriBidiChar>(*(p+1)));
+					if(FRIBIDI_IS_STRONG(next_type))
+					{
+						pData++;
+						iLength--;
+					}
+				}
+				
+				break;
+			}
+		}
 	}
-	else
-		bRes = m_pDocument->appendSpan(b.getPointer(0), b.getLength());
-
+	
+	bool bRes = appendSpan (pData, iLength);
 	b.truncate(0);
-
+	m_bFirstBlockData = false;
 	return bRes;
 }
 
 /*****************************************************************/
 /*****************************************************************/
 
+IE_Imp_Text_Sniffer::IE_Imp_Text_Sniffer ()
+	: IE_ImpSniffer(IE_IMPEXPNAME_TEXT)
+{
+	// 
+}
+
+UT_Confidence_t IE_Imp_Text_Sniffer::supportsMIME (const char * szMIME)
+{
+	if (UT_strcmp (IE_FileInfo::mapAlias (szMIME), IE_MIME_Text) == 0)
+		{
+			return UT_CONFIDENCE_GOOD;
+		}
+	if (strncmp (szMIME, "text/", 5) == 0)
+		{
+			return UT_CONFIDENCE_SOSO;
+		}
+	return UT_CONFIDENCE_ZILCH;
+}
+
 /*!
   Check if buffer contains data meant for this importer.
 
@@ -253,12 +307,12 @@ bool Inserter::insertSpan(UT_GrowBuf &b)
  the Encoded Text importer.
  */
 UT_Confidence_t IE_Imp_Text_Sniffer::recognizeContents(const char * szBuf,
-						       UT_uint32 iNumbytes)
+													   UT_uint32 iNumbytes)
 {
   if (_recognizeUTF8 (szBuf, iNumbytes))
-    return UT_CONFIDENCE_PERFECT;
+	  return UT_CONFIDENCE_PERFECT - 1; // it's UTF-8, but might not be plaintext
   else if (UE_NotUCS != _recognizeUCS2(szBuf, iNumbytes, false))
-    return UT_CONFIDENCE_PERFECT;
+	  return UT_CONFIDENCE_PERFECT - 1; // it's UTF-8, but might not be plaintext
   return UT_CONFIDENCE_POOR; // anything can be a text document - we'll unfairly weight this down
 }
 
@@ -276,22 +330,25 @@ bool IE_Imp_Text_Sniffer::_recognizeUTF8(const char * szBuf,
 	while (p < reinterpret_cast<const unsigned char *>(szBuf + iNumbytes))
 	{
 		UT_sint32 iLen;
-		
+
+		if (*p == 0) return false;			// ??
+
 		if ((*p & 0x80) == 0)				// ASCII
 		{
 			++p;
 			continue;
 		}
-		else if ((*p & 0xc0) == 0x80)			// not UTF-8
+		if ((*p & 0xc0) == 0x80)			// not UTF-8
 		{
 			return false;
 		}
-		else if (*p == 0xfe || *p == 0xff)
+		if (*p == 0xfe || *p == 0xff)
 		{
 			// BOM shouldn't occur in UTF-8 - file may be UCS-2
 			return false;
 		}
-		else if ((*p & 0xfe) == 0xfc)			// lead byte in 6-byte sequence
+
+		if ((*p & 0xfe) == 0xfc)				// lead byte in 6-byte sequence
 			iLen = 6;
 		else if ((*p & 0xfc) == 0xf8)			// lead byte in 5-byte sequence
 			iLen = 5;
@@ -301,13 +358,13 @@ bool IE_Imp_Text_Sniffer::_recognizeUTF8(const char * szBuf,
 			iLen = 3;
 		else if ((*p & 0xe0) == 0xc0)			// lead byte in 2-byte sequence
 			iLen = 2;
-		else	
+		else
 		{
 			// the above code covers all cases - if we reach here the logic is wrong
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  UT_ASSERT_NOT_REACHED();
 			return false;
 		}
-	
+
 		while (--iLen)
 		{
 			++p;
@@ -323,7 +380,7 @@ bool IE_Imp_Text_Sniffer::_recognizeUTF8(const char * szBuf,
 		bSuccess = true;
 		++p;
 	}
-	
+
 	return bSuccess;
 }
 
@@ -338,7 +395,7 @@ IE_Imp_Text_Sniffer::UCS2_Endian IE_Imp_Text_Sniffer::_recognizeUCS2(const char
 																	 bool bDeep)
 {
 	UCS2_Endian eResult = UE_NotUCS;
-	
+
 	if (iNumbytes >= 2)
 	{
 		const unsigned char *p = reinterpret_cast<const unsigned char *>(szBuf);
@@ -416,8 +473,10 @@ IE_Imp_Text_Sniffer::UCS2_Endian IE_Imp_Text_Sniffer::_recognizeUCS2(const char
  */
 UT_Confidence_t IE_Imp_Text_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-  if (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text") || !UT_stricmp (szSuffix, ".doc"))
+  if (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text"))
     return UT_CONFIDENCE_PERFECT;
+  if (!UT_stricmp (szSuffix, ".doc"))
+    return UT_CONFIDENCE_POOR;
 
   return UT_CONFIDENCE_ZILCH;
 }
@@ -440,13 +499,19 @@ bool IE_Imp_Text_Sniffer::getDlgLabels(const char ** pszDesc,
 	return true;
 }
 
+IE_Imp_EncodedText_Sniffer::IE_Imp_EncodedText_Sniffer ()
+	: IE_ImpSniffer(IE_IMPEXPNAME_TEXTENC)
+{
+	// 
+}
+
 /*!
   Check if buffer contains data meant for this importer.
 
  We don't attempt to recognize.  User must specifically choose Encoded Text.
  */
 UT_Confidence_t IE_Imp_EncodedText_Sniffer::recognizeContents(const char * /* szBuf */,
-												   UT_uint32 /* iNumbytes */)
+															  UT_uint32 /* iNumbytes */)
 {
 	return UT_CONFIDENCE_ZILCH;
 }
@@ -463,7 +528,7 @@ UT_Confidence_t IE_Imp_EncodedText_Sniffer::recognizeSuffix(const char * szSuffi
 }
 
 UT_Error IE_Imp_EncodedText_Sniffer::constructImporter(PD_Document * pDocument,
-												IE_Imp ** ppie)
+													   IE_Imp ** ppie)
 {
 	IE_Imp_Text * p = new IE_Imp_Text(pDocument,true);
 	*ppie = p;
@@ -488,7 +553,7 @@ bool IE_Imp_EncodedText_Sniffer::getDlgLabels(const char ** pszDesc,
 /*
   Import data from a plain text file
  \param szFilename Name of file to import
-  
+
  Each line terminator is taken to be a paragraph break
 */
 UT_Error IE_Imp_Text::importFile(const char * szFilename)
@@ -500,7 +565,7 @@ UT_Error IE_Imp_Text::importFile(const char * szFilename)
 		UT_DEBUGMSG(("Could not open file %s\n",szFilename));
 		return UT_IE_FILENOTFOUND;
 	}
-	
+
 	ImportStream *pStream = 0;
 	UT_Error error;
 
@@ -509,12 +574,11 @@ UT_Error IE_Imp_Text::importFile(const char * szFilename)
 		getDoc()->setEncodingName(m_szEncoding);
 
 	// Call encoding dialog
-	if (!m_bIsEncoded || _doEncodingDialog(m_szEncoding))
+	if (!m_bIsEncoded || m_bExplicitlySetEncoding || _doEncodingDialog(m_szEncoding))
 	{
 		X_CleanupIfError(error,_constructStream(pStream,fp));
-		Inserter ins(getDoc());
 		X_CleanupIfError(error,_writeHeader(fp));
-		X_CleanupIfError(error,_parseStream(pStream,ins));
+		X_CleanupIfError(error,_parseStream(pStream));
 		error = UT_OK;
 	}
 	else
@@ -535,14 +599,20 @@ Cleanup:
   Construct text importer
  \param pDocument Document to import text into
  \param bEncoded True if we should show encoding dialog
-  
+
  Uses current document's encoding if it is set
 */
 IE_Imp_Text::IE_Imp_Text(PD_Document * pDocument, bool bEncoded)
-	: IE_Imp(pDocument)
+  : IE_Imp(pDocument),
+    m_szEncoding(0),
+    m_bExplicitlySetEncoding(false),
+    m_bIsEncoded(false),
+    m_bIs16Bit(false),
+    m_bUseBOM(false),
+	m_bBigEndian(false),
+	m_bBlockDirectionPending(true),
+	m_bFirstBlockData(true)
 {
-	UT_ASSERT(pDocument);
-
 	// Get encoding dialog prefs setting
 	bool bAlwaysPrompt;
 	getDoc()->getApp()->getPrefsValueBool(AP_PREF_KEY_AlwaysPromptEncoding, &bAlwaysPrompt);
@@ -556,6 +626,27 @@ IE_Imp_Text::IE_Imp_Text(PD_Document * pDocument, bool bEncoded)
 	_setEncoding(szEncodingName);
 }
 
+IE_Imp_Text::IE_Imp_Text(PD_Document * pDocument, const char * encoding)
+  : IE_Imp(pDocument),
+    m_szEncoding(0),
+    m_bExplicitlySetEncoding(false),
+    m_bIsEncoded(false),
+    m_bIs16Bit(false),
+    m_bUseBOM(false),
+	m_bBigEndian(false),
+	m_bBlockDirectionPending(true),
+	m_bFirstBlockData(true),
+	m_pBlock(0)
+{
+  m_bIsEncoded = ((encoding != NULL) && (strlen(encoding) > 0));
+  
+  if ( m_bIsEncoded )
+    {
+      m_bExplicitlySetEncoding = true ;
+      _setEncoding(encoding);
+    }
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -597,14 +688,13 @@ UT_Error IE_Imp_Text::_recognizeEncoding(const char *szBuf, UT_uint32 iNumbytes)
 		IE_Imp_Text_Sniffer::UCS2_Endian eUcs2 = IE_Imp_Text_Sniffer::UE_NotUCS;
 
 		eUcs2 = IE_Imp_Text_Sniffer::_recognizeUCS2(szBuf, iNumbytes, false);
-		
+
 		if (eUcs2 == IE_Imp_Text_Sniffer::UE_BigEnd)
 			_setEncoding(XAP_EncodingManager::get_instance()->getUCS2BEName());
 		else if (eUcs2 == IE_Imp_Text_Sniffer::UE_LittleEnd)
 			_setEncoding(XAP_EncodingManager::get_instance()->getUCS2LEName());
- 		else
+		else
 		  _setEncoding ("ISO-8859-1");
-
 	}
 
 	return UT_OK;
@@ -629,33 +719,46 @@ UT_Error IE_Imp_Text::_constructStream(ImportStream *& pStream, FILE * fp)
  */
 UT_Error IE_Imp_Text::_writeHeader(FILE * /* fp */)
 {
-  // text gets applied in the Normal style
-  const XML_Char * propsArray[3];
-  propsArray[0] = "style";
-  propsArray[1] = "Normal";
-  propsArray[2] = 0;
+	// text gets applied in the Normal style
+	const XML_Char * propsArray[3];
+	propsArray[0] = "style";
+	propsArray[1] = "Normal";
+	propsArray[2] = 0;
 
-  X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Section, NULL));
-  X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Block, (const XML_Char**)propsArray));
+	X_ReturnNoMemIfError(appendStrux(PTX_Section, NULL));
+	X_ReturnNoMemIfError(appendStrux(PTX_Block, static_cast<const XML_Char**>(&propsArray[0])));
 
-  return UT_OK;
+	pf_Frag * pf = getDoc()->getPieceTable()->getFragments().getLast();
+	UT_ASSERT( pf->getType() == pf_Frag::PFT_Strux );
+	m_pBlock = (pf_Frag_Strux *) pf;
+	UT_ASSERT( m_pBlock->getStruxType() == PTX_Block );
+  
+	return UT_OK;
 }
 
 /*!
   Parse stream contents into the document
  \param stream Stream to import from
- \param ins Inserter helper class
 
  This code is used for both files and the clipboard
  */
-UT_Error IE_Imp_Text::_parseStream(ImportStream * pStream, Inserter & ins)
+UT_Error IE_Imp_Text::_parseStream(ImportStream * pStream)
 {
-	UT_ASSERT(pStream);
+	UT_return_val_if_fail(pStream, UT_ERROR);
 
 	bool bFirstChar = true;
 	UT_GrowBuf gbBlock(1024);
 	UT_UCSChar c;
 
+	if (!m_bExplicitlySetEncoding) {
+		const UT_UTF8String * prop;
+
+		prop = getProperty ("encoding");
+		if (prop) {
+			_setEncoding (prop->utf8_str());
+		}
+	}
+
 	pStream->init(m_szEncoding);
 
 	while (pStream->getChar(c))
@@ -674,8 +777,8 @@ UT_Error IE_Imp_Text::_parseStream(ImportStream * pStream, Inserter & ins)
 
 			// flush out what we have
 			if (gbBlock.getLength() > 0)
-				X_ReturnNoMemIfError(ins.insertSpan(gbBlock));
-			X_ReturnNoMemIfError(ins.insertBlock());
+				X_ReturnNoMemIfError(_insertSpan(gbBlock));
+			X_ReturnNoMemIfError(_insertBlock());
 			break;
 
 		case UCS_BOM:
@@ -683,15 +786,47 @@ UT_Error IE_Imp_Text::_parseStream(ImportStream * pStream, Inserter & ins)
 			if (bFirstChar)
 				break;
 
+		// if we encounter any of the following characters we will
+		// return error code immediately, as these have no
+		// business in text files
+		case 0x0000:
+		case 0x0001:
+		case 0x0002:
+		case 0x0003:
+		case 0x0004:
+		case 0x0005:
+		case 0x0006:
+		case 0x0007:
+		case 0x0008:
+		case 0x000e:
+		case 0x000f:
+		case 0x0010:
+		case 0x0011:
+		case 0x0012:
+		case 0x0013:
+		case 0x0014:
+		case 0x0015:
+		case 0x0016:
+		case 0x0017:
+		case 0x0018:
+		case 0x0019:
+		case 0x001a:
+		case 0x001b:
+		case 0x001c:
+		case 0x001d:
+		case 0x001e:
+		case 0x001f:
+			return UT_ERROR;
+			
 		default:
-			X_ReturnNoMemIfError(gbBlock.append(&c,1));
+			X_ReturnNoMemIfError(gbBlock.append(reinterpret_cast<UT_GrowBufElement*>(&c),1));
 			break;
 		}
 		bFirstChar = false;
 	}
 
 	if (gbBlock.getLength() > 0)
-		X_ReturnNoMemIfError(ins.insertSpan(gbBlock));
+		X_ReturnNoMemIfError(_insertSpan(gbBlock));
 
 	return UT_OK;
 }
@@ -706,22 +841,22 @@ bool IE_Imp_Text::_doEncodingDialog(const char *szEncoding)
 	XAP_Dialog_Id id = XAP_DIALOG_ID_ENCODING;
 
 	XAP_DialogFactory * pDialogFactory
-		= (XAP_DialogFactory *)(getDoc()->getApp()->getDialogFactory());
+		= static_cast<XAP_DialogFactory *>(getDoc()->getApp()->getDialogFactory());
 
 	XAP_Dialog_Encoding * pDialog
-		= (XAP_Dialog_Encoding *)(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
+		= static_cast<XAP_Dialog_Encoding *>(pDialogFactory->requestDialog(id));
+	UT_return_val_if_fail(pDialog, false);
 
 	pDialog->setEncoding(szEncoding);
 
 	// run the dialog
 	XAP_Frame * pFrame = getDoc()->getApp()->getLastFocussedFrame();
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail(pFrame, false);
 
 	pDialog->runModal(pFrame);
 
 	// extract what they did
-	
+
 	bool bOK = (pDialog->getAnswer() == XAP_Dialog_Encoding::a_OK);
 
 	if (bOK)
@@ -730,10 +865,10 @@ bool IE_Imp_Text::_doEncodingDialog(const char *szEncoding)
 		static XML_Char szEnc[16];
 
 		s = pDialog->getEncoding();
-		UT_ASSERT (s);
+		UT_return_val_if_fail (s, false);
 
 		strcpy(szEnc,s);
-		_setEncoding((const char *)szEnc);
+		_setEncoding(static_cast<const char *>(szEnc));
 		getDoc()->setEncodingName(szEnc);
 	}
 
@@ -797,21 +932,20 @@ void IE_Imp_Text::_setEncoding(const char *szEncoding)
 // TODO always interpreted using the system default encoding which can be wrong.
 
 void IE_Imp_Text::pasteFromBuffer(PD_DocumentRange * pDocRange,
-								  unsigned char * pData, UT_uint32 lenData,
+								  const unsigned char * pData, UT_uint32 lenData,
 								  const char *szEncoding)
 {
-	UT_ASSERT(getDoc() == pDocRange->m_pDoc);
-	UT_ASSERT(pDocRange->m_pos1 == pDocRange->m_pos2);
+	UT_return_if_fail(getDoc() == pDocRange->m_pDoc);
+	UT_return_if_fail(pDocRange->m_pos1 == pDocRange->m_pos2);
 
 	// Attempt to guess whether we're pasting 8 bit or unicode text
 	if (szEncoding)
 		_setEncoding(szEncoding);
 	else
-		_recognizeEncoding((const char *)pData, lenData);
+		_recognizeEncoding(reinterpret_cast<const char *>(pData), lenData);
 
 	ImportStreamClipboard stream(pData, lenData);
-	Inserter ins(getDoc(), pDocRange->m_pos1);
-
-	_parseStream(&stream, ins);
+	setClipboard (pDocRange->m_pos1);
+	_parseStream(&stream);
 }
 
