@@ -1,7 +1,7 @@
 /* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
 
 /* AbiWord
- * Copyright (C) 1998-2002 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,7 +19,6 @@
  * 02111-1307, USA.
  */
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -43,13 +42,10 @@
 
 #include "ie_types.h"
 #include "ie_impGraphic.h"
+#include "ie_Table.h"
 #include "ie_impexp_HTML.h"
 #include "ie_imp_XHTML.h"
 
-#ifdef XHTML_MULTIPART_SUPPORTED
-#include "ie_imp_MHT.h"
-#endif
-
 typedef enum section_class
 {
 	sc_section = 0,
@@ -70,87 +66,6 @@ static UT_UTF8String s_parseCSStyle (const UT_UTF8String & style, UT_uint32 css_
 /*****************************************************************/
 /*****************************************************************/
 
-#if defined(ENABLE_PLUGINS) || defined(ABI_PLUGIN_SOURCE)
-
-// completely generic code to allow this to be a plugin
-
-#include "xap_Module.h"
-
-ABI_PLUGIN_DECLARE("HTML")
-
-class IE_Imp_MHT_Sniffer;
-
-// we use a reference-counted sniffer
-static IE_Imp_XHTML_Sniffer * m_sniffer = 0;
-static IE_Imp_MHT_Sniffer * m_mht_sniffer = 0;
-
-ABI_FAR_CALL
-int abi_plugin_register (XAP_ModuleInfo * mi)
-{
-	if (!m_sniffer)
-	{
-		m_sniffer = new IE_Imp_XHTML_Sniffer ();
-	}
-	else
-	{
-		m_sniffer->ref();
-	}
-#ifdef XHTML_MULTIPART_SUPPORTED
-	if (!m_mht_sniffer)
-	{
-		m_mht_sniffer = new IE_Imp_MHT_Sniffer ();
-	}
-	else
-	{
-		m_mht_sniffer->ref();
-	}
-#endif
-
-	mi->name = "XHTML Importer";
-	mi->desc = "Import XHTML Documents";
-	mi->version = ABI_VERSION_STRING;
-	mi->author = "Abi the Ant";
-	mi->usage = "No Usage";
-
-	IE_Imp::registerImporter (m_sniffer);
-#ifdef XHTML_MULTIPART_SUPPORTED
-	IE_Imp::registerImporter (m_mht_sniffer);
-#endif
-	return 1;
-}
-
-ABI_FAR_CALL
-int abi_plugin_unregister (XAP_ModuleInfo * mi)
-{
-	mi->name = 0;
-	mi->desc = 0;
-	mi->version = 0;
-	mi->author = 0;
-	mi->usage = 0;
-
-	UT_ASSERT (m_sniffer);
-
-	IE_Imp::unregisterImporter (m_sniffer);
-	if (!m_sniffer->unref())
-	{
-		m_sniffer = 0;
-	}
-
-	return 1;
-}
-
-ABI_FAR_CALL
-int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
-								 UT_uint32 release)
-{
-  return 1;
-}
-
-#endif
-
-/*****************************************************************/
-/*****************************************************************/
-
 IE_Imp_XHTML_Sniffer::IE_Imp_XHTML_Sniffer ()
 #ifdef XHTML_NAMED_CONSTRUCTORS
 	: IE_ImpSniffer("AbiXHTML::XHTML")
@@ -292,9 +207,15 @@ bool	IE_Imp_XHTML_Sniffer::getDlgLabels(const char ** pszDesc,
 
 IE_Imp_XHTML::IE_Imp_XHTML(PD_Document * pDocument) :
 	IE_Imp_XML(pDocument,false),
+#ifdef USE_IE_IMP_TABLEHELPER
+	m_TableHelperStack(new IE_Imp_TableHelperStack(pDocument)),
+#else
+	m_TableHelperStack(0),
+#endif
 	m_listType(L_NONE),
     m_iListID(0),
 	m_bFirstDiv(true),
+	m_bUseTidy(false),
 	m_iNewListID(0),
 	m_iNewImage(0),
 	m_szBookMarkName(NULL),
@@ -305,6 +226,9 @@ IE_Imp_XHTML::IE_Imp_XHTML(PD_Document * pDocument) :
 
 IE_Imp_XHTML::~IE_Imp_XHTML()
 {
+#ifdef USE_IE_IMP_TABLEHELPER
+	DELETEP(m_TableHelperStack);
+#endif
 	UT_VECTOR_PURGEALL(UT_UTF8String *,m_divStyles);
 }
 
@@ -329,8 +253,11 @@ static struct xmlToIdMapping s_Tokens[] =
 	{ "blockquote",	TT_BLOCKQUOTE	},
 	{ "body",		TT_BODY			},
 	{ "br",			TT_BR			},
+	{ "caption",	TT_CAPTION		},
 	{ "cite",		TT_CITE			},
 	{ "code",		TT_CODE			},
+	{ "col",		TT_COL			},
+	{ "colgroup",	TT_COLGROUP		},
 	{ "dd",			TT_DD			},
 	{ "dfn",		TT_DFN			},
 	{ "div",		TT_DIV			},
@@ -368,6 +295,12 @@ static struct xmlToIdMapping s_Tokens[] =
 	{ "style",		TT_STYLE		},
 	{ "sub",		TT_SUB			},
 	{ "sup",		TT_SUP			},
+	{ "table",		TT_TABLE		},
+	{ "tbody",		TT_TBODY		},
+	{ "td",			TT_TD			},
+	{ "tfoot",		TT_TFOOT		},
+	{ "th",			TT_TH			},
+	{ "thead",		TT_THEAD		},
 	{ "title",		TT_TITLE		},
 	{ "tr",			TT_TR			},	
 	{ "tt",			TT_TT			},
@@ -418,169 +351,226 @@ static SectionClass s_class_query (const char * class_value)
 	return sc;
 }
 
-static void convertFontFace(UT_String & szDest, const char *szFrom)
+static void s_append_font_family (UT_UTF8String & style, const char * face)
 {
-	// TODO: make me better
-	// TODO: and handle things like comma lists of font faces
-	char *newFont;
-	
-	// default...
-	if(szFrom == NULL)
-		newFont = "Times New Roman";
+	unsigned char u;
+
+	while (*face)
+		{
+			u = static_cast<unsigned char>(*face);
+			if (!isspace (static_cast<int>(u))) break;
+			++face;
+		}
+	if (*face == 0) return;
+
+	char quote = ',';
+	if ((*face == '"') || (*face == '\''))
+		{
+			quote = *face++;
+		}
+
+	char * value = UT_strdup (face);
+	if (value == 0) return;
+
+	char * ptr = value;
+	while (*ptr)
+		{
+			if (*ptr == quote)
+				{
+					*ptr = 0;
+					break;
+				}
+			++ptr;
+		}
+	if (quote == ',')
+		while (ptr > value)
+			{
+				--ptr;
+				u = static_cast<unsigned char>(*ptr);
+				if (!isspace (static_cast<int>(u))) break;
+				*ptr = 0;
+			}
+	if (strlen (value))
+		{
+			if (style.byteLength ())
+				{
+					style += "; ";
+				}
+			style += "font-family:";
+			style += value;
+		}
+	free (value);
+}
+
+static const char * s_font_size[7] = {
+	"8pt",
+	"10pt",
+	"11pt",
+	"12pt",
+	"16pt",
+	"24pt",
+	"32pt"
+};
+
+static void s_append_font_size (UT_UTF8String & style, const char * size)
+{
+	unsigned char u;
+
+	while (*size)
+		{
+			u = static_cast<unsigned char>(*size);
+			if (!isspace (static_cast<int>(u))) break;
+			++size;
+		}
+	if (*size == 0) return;
+
+	if (*size == '+')
+		{
+			int sz = atoi (size + 1);
+			if ((sz < 1) || (sz > 7)) return;
+			if (sz > 3) sz = 3;
+
+			if (style.byteLength ())
+				{
+					style += "; ";
+				}
+			style += "font-size:";
+			style += s_font_size[3+sz];
+		}
+	else if (*size == '-')
+		{
+			int sz = atoi (size + 1);
+			if ((sz < 1) || (sz > 7)) return;
+			if (sz > 3) sz = 3;
+
+			if (style.byteLength ())
+				{
+					style += "; ";
+				}
+			style += "font-size:";
+			style += s_font_size[3-sz];
+		}
 	else
-		newFont = (char*)szFrom;
-	
-	szDest = newFont;
+		{
+			int sz = atoi (size);
+			if (sz == 0) return;
+
+			if (style.byteLength ())
+				{
+					style += "; ";
+				}
+			style += "font-size:";
+
+			if (sz <= 7)
+				{
+					style += s_font_size[sz-1];
+				}
+			else
+				{
+					UT_String pt_size;
+					UT_String_sprintf(pt_size, "%2dpt", sz);
+					style += pt_size.c_str ();
+				}
+		}
 }
 
-static void convertFontSize(UT_String & szDest, const char *szFrom)
+static void s_append_color (UT_UTF8String & style, const char * color, const char * property)
 {
-	// if it starts with a +
-	// if it starts with a -
-	// if it is a number, clamp it
-	
-	int sz = 12;
-	
-	if(szFrom == NULL)
-    {
-		sz = 12;
-    }
-#ifdef XHTML_UCS4
-	else if(UT_UCS4_isdigit(*szFrom))
-#else
-	else if(UT_UCS_isdigit(*szFrom))
-#endif
-    {
-		sz = atoi(szFrom);
-    }
-	else if(*szFrom == '+')
-    {
-		switch(atoi(szFrom+1))
+	unsigned char u;
+
+	while (*color)
 		{
-		case 1:
-			sz = 10; break;
-		case 2:
-			sz = 12; break;
-		case 3:
-			sz = 16; break;
-		case 4:
-			sz = 20; break;
-		case 5:
-			sz = 24; break;
-		case 6:
-			sz = 36; break;
-		default:
-			sz = 12; break;
+			u = static_cast<unsigned char>(*color);
+			if (!isspace (static_cast<int>(u))) break;
+			++color;
 		}
-    }
-	else if(*szFrom == '-')
-    {
-		switch(atoi(szFrom+1))
+	if (*color == 0) return;
+
+	char * value = UT_strdup (color);
+	if (value == 0) return;
+
+	int length = 0;
+
+	bool bValid = true;
+	bool bHexal = true;
+
+	char * ptr = value;
+	if (*ptr == '#') ++ptr;
+	while (*ptr)
 		{
-		case 1:
-			sz = 9; break;
-		default:
-			sz = 8; break;
+			char c = *ptr;
+			u = static_cast<unsigned char>(c);
+			if (isspace (static_cast<int>(u)))
+				{
+					*ptr = 0;
+					break;
+				}
+			if (!isalnum (static_cast<int>(u)))
+				{
+					bValid = false;
+					break;
+				}
+			if (bHexal)
+				if (!isdigit (static_cast<int>(u)))
+					if (((c < 'a') && (c > 'f')) && ((c < 'A') && (c > 'F')))
+						bHexal = false;
+			++ptr;
+			++length;
 		}
-    }
-	
-	// clamp the value
-	if(sz > 36)
-		sz = 36;
-	else if(sz < 8)
-		sz = 8;
-	
-	UT_String_sprintf(szDest, "%2d", sz);
-}
+	if (!bValid)
+		{
+			free (value);
+			return;
+		}
+	if (*value == '#')
+		if (!bHexal || ((length != 3) && (length != 6)))
+			{
+				free (value);
+				return;
+			}
 
-static void convertColor(UT_String & szDest, const char *szFrom, int dfl = 0x000000)
-{
+	UT_HashColor hash_color;
+	UT_UTF8String hex_color;
 
-  // if it starts with a #, send it through
-  // if it starts with a number, send it through
-  // else convert color from values in XHTML DTD
+	if (*value == '#')
+		{
+			if (length == 3)
+				{
+					unsigned int rgb;
+					if (sscanf (value + 1, "%x", &rgb) == 1)
+						{
+							unsigned int uir = (rgb & 0x0f00) >> 8;
+							unsigned int uig = (rgb & 0x00f0) >> 4;
+							unsigned int uib = (rgb & 0x000f);
 
-	int col = dfl;
-	
-	if(szFrom == NULL)
-    {
-		col = dfl;
-    }
-	if(*szFrom == '#')
-    {
-		col = atoi(szFrom+1);
-    }
-#ifdef XHTML_UCS4
-	else if(UT_UCS4_isdigit(*szFrom))
-#else
-	else if(UT_UCS_isdigit(*szFrom))
-#endif
-    {
-		col = atoi(szFrom);
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "green"))
-    {
-		col = 0x008000;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "silver"))
-    {
-		col = 0xC0C0C0;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "lime"))
-    {
-		col = 0x00FF00;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "gray"))
-    {
-		col = 0x808080;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "olive"))
-    {
-		col = 0x808000;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "white"))
-    {
-		col = 0xFFFFFF;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "yellow"))
-    {
-		col = 0xFFFF00;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "maroon"))
-    {
-		col = 0x800000;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "navy"))
-    {
-		col = 0x000080;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "red"))
-    {
-		col = 0xFF0000;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "blue"))
-    {
-		col = 0x0000FF;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "purple"))
-    {
-		col = 0x800080;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "teal"))
-    {
-		col = 0x008080;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "fuchsia"))
-    {
-		col = 0xFF00FF;
-    }
-	else if(!UT_XML_stricmp((XML_Char *)szFrom, "aqua"))
-    {
-		col = 0x00FFFF;
-    }
-	
-	UT_String_sprintf(szDest, "%6x", col);
-	UT_DEBUGMSG(("DOM: color: %0xd (%s => %s)\n", col, szFrom, szDest.c_str()));
+							unsigned char r = static_cast<unsigned char>(uir|(uir<<4));
+							unsigned char g = static_cast<unsigned char>(uig|(uig<<4));
+							unsigned char b = static_cast<unsigned char>(uib|(uib<<4));
+
+							hex_color = hash_color.setColor (r, g, b) + 1;
+						}
+				}
+			else hex_color = value + 1;
+		}
+	else if (bHexal && (length == 6))
+		{
+			hex_color = value;
+		}
+	else
+		{
+			hex_color = hash_color.lookupNamedColor (color) + 1;
+		}
+	free (value);
+
+	if (hex_color.byteLength () == 0) return;
+
+	if (style.byteLength ())
+		{
+			style += "; ";
+		}
+	style += property;
+	style += ":";
+	style += hex_color;
 }
 
 /*****************************************************************/
@@ -604,6 +594,23 @@ UT_Error IE_Imp_XHTML::importFile(const char * szFilename)
 	return e;
 }
 
+void IE_Imp_XHTML::pasteFromBuffer(PD_DocumentRange * pDocRange,
+								   const unsigned char * pData, 
+								   UT_uint32 lenData, 
+								   const char * szEncoding)
+{
+	UT_return_if_fail(getDoc() == pDocRange->m_pDoc);
+	UT_return_if_fail(pDocRange->m_pos1 == pDocRange->m_pos2);
+
+	setClipboard (pDocRange->m_pos1);
+
+	UT_XML xml;
+	xml.setListener (this);
+	UT_ByteBuf buf (lenData);
+	buf.append (pData, lenData);
+	xml.parse (&buf);
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -652,7 +659,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			/* stack class attr. values if recognized;
 			 * NOTE: these are ptrs to static strings - don't alloc/free them
 			 */
-			const XML_Char * p_val = _getXMLPropValue ((const XML_Char *) "class", atts);
+			const XML_Char * p_val = _getXMLPropValue (static_cast<const XML_Char *>("class"), atts);
 			SectionClass sc = childOfSection () ? sc_other : s_class_query (p_val);
 			if (sc == sc_other)
 				m_divClasses.push_back (0);
@@ -674,7 +681,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 
 			if (style)
 				{
-					p_val = _getXMLPropValue ((const XML_Char *) "align", atts);
+					p_val = _getXMLPropValue (static_cast<const XML_Char *>("align"), atts);
 					if (p_val)
 						{
 							if (!UT_XML_strcmp (p_val, "right"))
@@ -688,7 +695,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 						}
 				}
 
-			p_val = _getXMLPropValue ((const XML_Char *) "style", atts);
+			p_val = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 			if (style && p_val)
 				{
 					*style += p_val;
@@ -748,29 +755,26 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	case TT_FONT:
 		UT_DEBUGMSG(("Font tag encountered\n"));
 		{
-			const XML_Char *p_val;
-			UT_String color, bgcolor, size, face;
-			UT_String output;
+			UT_UTF8String style;
 			
-			p_val = _getXMLPropValue((const XML_Char *)"color", atts);
-			convertColor(color, p_val, 0x000000);
+			const XML_Char * p_val = 0;
 
-			p_val = _getXMLPropValue((const XML_Char *)"background", atts);
-			if ( p_val )
-			  convertColor(bgcolor, p_val, 0xFFFFFF);
-			else
-			  bgcolor = "none";
+			p_val = _getXMLPropValue (static_cast<const XML_Char *>("color"), atts);
+			if (p_val) s_append_color (style, p_val, "color");
 
-			p_val = _getXMLPropValue((const XML_Char *)"size", atts);
-			convertFontSize(size, p_val);
+			p_val = _getXMLPropValue (static_cast<const XML_Char *>("background"), atts);
+			if (p_val) s_append_color (style, p_val, "bgcolor");
+
+			p_val = _getXMLPropValue (static_cast<const XML_Char *>("size"), atts);
+			if (p_val) s_append_font_size (style, p_val);
 			
-			p_val = _getXMLPropValue((const XML_Char *)"face", atts);
-			convertFontFace(face, p_val);
+			p_val = _getXMLPropValue (static_cast<const XML_Char *>("face"), atts);
+			if (p_val) s_append_font_family (style, p_val);
 			
-			UT_String_sprintf(output, "color:%s; bgcolor: %s; font-family:%s; size:%spt", color.c_str(), bgcolor.c_str(), face.c_str(), size.c_str());
-			UT_DEBUGMSG(("Font properties: %s\n", output.c_str()));
-			
-			X_CheckError(pushInline (output.c_str ()));
+			// UT_String_sprintf(output, "color:%s; bgcolor: %s; font-family:%s; size:%spt", color.c_str(), bgcolor.c_str(), face.c_str(), size.c_str());
+			UT_DEBUGMSG(("Font properties: %s\n", style.utf8_str()));
+
+			X_CheckError(pushInline (style.utf8_str ()));
 		}
 		return;
 
@@ -786,7 +790,6 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	case TT_BLOCKQUOTE:
 		// &...
 	case TT_P:
-	case TT_TR:
 	case TT_H4:
 	case TT_H5:
 	case TT_H6:
@@ -797,16 +800,16 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			if (_data_CharCount ())
 				{
 					UT_UCSChar ucs = UCS_LF;
-					X_CheckError(getDoc()->appendSpan (&ucs, 1));
+					X_CheckError(appendSpan (&ucs, 1));
 					_data_NewBlock ();
 				}
 		}
 		else
 		{
-			const XML_Char * style = _getXMLPropValue ((const XML_Char *) "style", atts);
-			const XML_Char * align = _getXMLPropValue ((const XML_Char *) "align", atts);
+			const XML_Char * style = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
+			const XML_Char * align = _getXMLPropValue (static_cast<const XML_Char *>("align"), atts);
 
-			const XML_Char * p_val = _getXMLPropValue ((const XML_Char *) "awml:style", atts);
+			const XML_Char * p_val = _getXMLPropValue (static_cast<const XML_Char *>("awml:style"), atts);
 
 			if (p_val)
 				{
@@ -892,7 +895,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		if (m_listType != L_NONE)
 		{
 			UT_uint16 thisID = m_iListID;
-			m_utsParents.viewTop((void**) &parentID);
+			m_utsParents.viewTop(reinterpret_cast<void**>(&parentID));
 
 			const XML_Char** listAtts;
 			listAtts = (m_listType == L_OL ? ol_p_atts : ul_p_atts);
@@ -921,10 +924,10 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			listAtts[IDpos] = szListID.c_str();
 			listAtts[parentIDpos] = szParentID.c_str();
 
-			XML_Char* temp = (XML_Char*) listAtts[propsPos];
+			const XML_Char* temp = static_cast<const XML_Char*>(listAtts[propsPos]);
 			listAtts[propsPos] = props.c_str();
 
-			X_CheckError(getDoc()->appendStrux(PTX_Block, listAtts));
+			X_CheckError(appendStrux(PTX_Block, listAtts));
 
 			listAtts[propsPos] = temp;
 
@@ -933,19 +936,19 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			new_atts[0] = sz;
 			UT_XML_cloneString(sz, "list_label");
 			new_atts[1] = sz;
-			X_CheckError(getDoc()->appendObject(PTO_Field, new_atts));
+			X_CheckError(appendObject(PTO_Field, new_atts));
 
 			// append the character run
 			UT_XML_cloneString(sz, "type");
 			new_atts[0] = sz;
 			UT_XML_cloneString(sz, "list_label");
 			new_atts[1] = sz;
-			X_CheckError(getDoc()->appendFmt(new_atts));
+			X_CheckError(appendFmt(new_atts));
 
 			/* warn XML charData() handler of new block, but insert a tab first
 			 */
 			UT_UCSChar ucs = UCS_TAB;
-			X_CheckError(getDoc()->appendSpan (&ucs, 1));
+			X_CheckError(appendSpan (&ucs, 1));
 			_data_NewBlock ();
 		}
 		return;
@@ -955,10 +958,10 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		{
 			UT_UTF8String utf8val;
 
-			const XML_Char * p_val = _getXMLPropValue ((const XML_Char *) "style", atts);
+			const XML_Char * p_val = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 			if (p_val)
 				{
-					utf8val = (const char *) p_val;
+					utf8val = static_cast<const char *>(p_val);
 					utf8val = s_parseCSStyle (utf8val, CSS_MASK_INLINE);
 					UT_DEBUGMSG(("CSS->Props (utf8val): [%s]\n",utf8val.utf8_str()));
 				}
@@ -971,15 +974,15 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		if(m_parseState == _PS_Block)
 		{
 			UT_UCSChar ucs = UCS_LF;
-			X_CheckError(getDoc()->appendSpan(&ucs,1));
+			X_CheckError(appendSpan(&ucs,1));
 		}
 		return;
 
 	case TT_A:
 	{
 		const XML_Char * p_val = 0;
-		p_val = _getXMLPropValue((const XML_Char *)"xlink:href", atts);
-		if (p_val == 0) p_val = _getXMLPropValue((const XML_Char *)"href", atts);
+		p_val = _getXMLPropValue(static_cast<const XML_Char *>("xlink:href"), atts);
+		if (p_val == 0) p_val = _getXMLPropValue(static_cast<const XML_Char *>("href"), atts);
 		if( p_val )
 		{
 			X_CheckError(requireBlock ());
@@ -988,12 +991,12 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	    	sz = NULL;
 		    UT_XML_cloneString(sz, p_val);
 		    new_atts[1] = sz;
-			X_CheckError(getDoc()->appendObject(PTO_Hyperlink,new_atts));
+			X_CheckError(appendObject(PTO_Hyperlink,new_atts));
 		}
 		else
 		{
-			p_val = _getXMLPropValue((const XML_Char *)"id", atts);
-			if (p_val == 0) p_val = _getXMLPropValue((const XML_Char *)"name", atts);
+			p_val = _getXMLPropValue(static_cast<const XML_Char *>("id"), atts);
+			if (p_val == 0) p_val = _getXMLPropValue(static_cast<const XML_Char *>("name"), atts);
 			if (p_val)
 			{
 				X_CheckError(requireBlock ());
@@ -1016,7 +1019,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			    bm_new_atts[3] = sz;
 				if (m_szBookMarkName)
 				{
-					X_CheckError(getDoc()->appendObject(PTO_Bookmark,bm_new_atts));
+					X_CheckError(appendObject(PTO_Bookmark,bm_new_atts));
 				}
 				else for (i = 0; i < 4; i++) FREEP(bm_new_atts[i]);
 
@@ -1034,7 +1037,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 					sz = NULL;
 					UT_XML_cloneString(sz, m_szBookMarkName);
 					bm_new_atts[3] = sz;
-					X_CheckError(getDoc()->appendObject(PTO_Bookmark,bm_new_atts));
+					X_CheckError(appendObject(PTO_Bookmark,bm_new_atts));
 
 					FREEP(m_szBookMarkName);
 					m_szBookMarkName = NULL;
@@ -1046,10 +1049,10 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 
 	case TT_IMG:
 		{
-		const XML_Char * szSrc    = _getXMLPropValue ((const XML_Char *) "src",    atts);
-		const XML_Char * szStyle  = _getXMLPropValue ((const XML_Char *) "style",  atts);
-		const XML_Char * szWidth  = _getXMLPropValue ((const XML_Char *) "width",  atts);
-		const XML_Char * szHeight = _getXMLPropValue ((const XML_Char *) "height", atts);
+		const XML_Char * szSrc    = _getXMLPropValue (static_cast<const XML_Char *>("src"),    atts);
+		const XML_Char * szStyle  = _getXMLPropValue (static_cast<const XML_Char *>("style"),  atts);
+		const XML_Char * szWidth  = _getXMLPropValue (static_cast<const XML_Char *>("width"),  atts);
+		const XML_Char * szHeight = _getXMLPropValue (static_cast<const XML_Char *>("height"), atts);
 
 		if ( szSrc == 0) break;
 		if (*szSrc == 0) break;
@@ -1057,10 +1060,9 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		FG_Graphic * pfg = 0;
 
 		if (strncmp (szSrc, "data:", 5) == 0) // data-URL - special case
-			{
 				pfg = importDataURLImage (szSrc + 5);
-			}
-		else pfg = importImage (szSrc);
+		else if (!isClipboard ())
+			pfg = importImage (szSrc);
 
 		if (pfg == 0) break;
 
@@ -1073,7 +1075,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		UT_UTF8String utf8val;
 		if (szStyle)
 			{
-				utf8val = (const char *) szStyle;
+				utf8val = static_cast<const char *>(szStyle);
 				utf8val = s_parseCSStyle (utf8val, CSS_MASK_IMAGE);
 				UT_DEBUGMSG(("CSS->Props (utf8val): [%s]\n",utf8val.utf8_str()));
 			}
@@ -1081,7 +1083,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			{
 				UT_Dimension units = UT_determineDimension (szWidth, DIM_PX);
 				double d = UT_convertDimensionless (szWidth);
-				float width = (float) UT_convertDimensions (d, units, DIM_IN);
+				float width = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 				UT_String tmp;
 				char * old_locale = setlocale (LC_NUMERIC, "C");
 				UT_String_sprintf (tmp, "%gin", width);
@@ -1097,7 +1099,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			{
 				UT_Dimension units = UT_determineDimension (szHeight, DIM_PX);
 				double d = UT_convertDimensionless (szHeight);
-				float height = (float) UT_convertDimensions (d, units, DIM_IN);
+				float height = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 				UT_String tmp;
 				char * old_locale = setlocale (LC_NUMERIC, "C");
 				UT_String_sprintf (tmp, "%gin", height);
@@ -1136,7 +1138,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		const XML_Char * api_atts[5];
 
 		UT_String dataid;
-		UT_String_sprintf (dataid, "image%u", (unsigned int) m_iNewImage++);
+		UT_String_sprintf (dataid, "image%u", static_cast<unsigned int>(m_iNewImage++));
 
 		sz = NULL;
 		UT_XML_cloneString (sz, PT_PROPS_ATTRIBUTE_NAME);
@@ -1162,13 +1164,79 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			}
 		UT_DEBUGMSG(("inserting `%s' as `%s' [%s]\n",szSrc,dataid.c_str(),utf8val.utf8_str()));
 
-		X_CheckError(getDoc()->appendObject (PTO_Image, api_atts));
-		X_CheckError(getDoc()->createDataItem (dataid.c_str(), false, pBB, (void*) mimetype, NULL));
+		X_CheckError(appendObject (PTO_Image, api_atts));
+		X_CheckError(getDoc()->createDataItem (dataid.c_str(), false, pBB, static_cast<void*>(mimetype), NULL));
 
 		UT_DEBUGMSG(("insertion successful\n"));
 		}
 		return;
+#ifdef USE_IE_IMP_TABLEHELPER
+	case TT_CAPTION:
+	case TT_COLGROUP:
+	case TT_COL:
+		// TODO
+		break;
+	case TT_TABLE:
+		{
+			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
+			X_CheckError(m_TableHelperStack->tableStart (static_cast<const char *>(szStyle)));
+		}
+		break;
+	case TT_THEAD:
+		{
+			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
+
+			m_TableHelperStack->theadStart (static_cast<const char *>(szStyle));
+		}
+		break;
+	case TT_TFOOT:
+		{
+			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
+
+			m_TableHelperStack->tfootStart (static_cast<const char *>(szStyle));
+		}
+		break;
+	case TT_TBODY:
+		{
+			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
+
+			m_TableHelperStack->tbodyStart (static_cast<const char *>(szStyle));
+		}
+		break;
+	case TT_TR:
+		{
+			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
+
+			m_TableHelperStack->trStart (static_cast<const char *>(szStyle));
+		}
+		break;
+	case TT_TH:
+	case TT_TD:
+		{
+			const XML_Char * szStyle   = _getXMLPropValue (static_cast<const XML_Char *>("style"),   atts);
+			const XML_Char * szColSpan = _getXMLPropValue (static_cast<const XML_Char *>("colspan"), atts);
+			const XML_Char * szRowSpan = _getXMLPropValue (static_cast<const XML_Char *>("rowspan"), atts);
+
+			UT_uint32 colspan = 1;
+			UT_uint32 rowspan = 1;
+
+			if (szColSpan)
+				{
+					int span = atoi (static_cast<const char *>(szColSpan));
+					if (span > 1)
+						colspan = static_cast<UT_uint32>(span);
+				}
+			if (szRowSpan)
+				{
+					int span = atoi (static_cast<const char *>(szRowSpan));
+					if (span > 1)
+						rowspan = static_cast<UT_uint32>(span);
+				}
+			m_TableHelperStack->tdStart (colspan, rowspan, static_cast<const char *>(szStyle));
+		}
+		break;
+#endif /* USE_IE_IMP_TABLEHELPER */
 	case TT_HEAD:
 	case TT_TITLE:
 	case TT_META:
@@ -1185,7 +1253,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		// <ruby> text.  The <rp> text will not be rendered but should
 		// be retained so it can be exported.
 		X_CheckError(requireBlock ());
-		X_CheckError(getDoc()->appendFmt(new_atts));
+		X_CheckError(appendFmt(new_atts));
 		return;
 
 	case TT_OTHER:
@@ -1250,7 +1318,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_DL:
 		UT_uint16 *temp;
 
-		if(m_utsParents.pop((void**) &temp))
+		if(m_utsParents.pop(reinterpret_cast<void**>(&temp)))
 		{
 			m_iListID = *temp;
 			DELETEP(temp);
@@ -1270,7 +1338,6 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		return;
 
 	case TT_P:
-	case TT_TR:
 	case TT_H1:
 	case TT_H2:
 	case TT_H3:
@@ -1311,7 +1378,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		_popInlineFmt ();
 		if (m_parseState == _PS_Block)
 			{
-				X_CheckError(getDoc()->appendFmt(&m_vecInlineFmt));
+				X_CheckError(appendFmt(&m_vecInlineFmt));
 			}
 		return;
 
@@ -1321,7 +1388,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		X_VerifyParseState(_PS_Block);
 		X_CheckDocument(_getInlineDepth()>0);
 		_popInlineFmt();
-		getDoc()->appendFmt(&m_vecInlineFmt);
+		appendFmt(&m_vecInlineFmt);
 		return;
 
 	case TT_BR:						// not a container, so we don't pop stack
@@ -1329,7 +1396,32 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		//UT_DEBUGMSG(("B %d\n", m_parseState));
 //		X_VerifyParseState(_PS_Block);
 		return;
-
+#ifdef USE_IE_IMP_TABLEHELPER
+	case TT_CAPTION:
+	case TT_COLGROUP:
+	case TT_COL:
+		// TODO
+		break;
+	case TT_TABLE:
+		{
+			m_TableHelperStack->tableEnd ();
+		}
+		break;
+	case TT_THEAD:
+	case TT_TFOOT:
+	case TT_TBODY:
+		{
+			m_TableHelperStack->tbodyStart ();
+		}
+		break;
+	case TT_TR:
+		// 
+		break;
+	case TT_TH:
+	case TT_TD:
+		// 
+		break;
+#endif /* USE_IE_IMP_TABLEHELPER */
 	case TT_HEAD:
 	case TT_TITLE:
 	case TT_META:
@@ -1354,7 +1446,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
     		sz = NULL;
 	    	UT_XML_cloneString(sz, m_szBookMarkName);
 		    bm_new_atts[3] = sz;
-			X_CheckError(getDoc()->appendObject(PTO_Bookmark,bm_new_atts));
+			X_CheckError(appendObject(PTO_Bookmark,bm_new_atts));
 			for(i = 0; i < 5; i++) FREEP(bm_new_atts[i]);
 			FREEP(m_szBookMarkName);
 			m_szBookMarkName = NULL;
@@ -1364,7 +1456,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 			/* if (m_parseState == _PS_Sec) then this is an anchor outside
 			 * of a block, not a hyperlink (see TT_A in startElement)
 			 */
- 			X_CheckError(getDoc()->appendObject(PTO_Hyperlink,0));
+ 			X_CheckError(appendObject(PTO_Hyperlink,0));
 		}
 		return;
 
@@ -1387,7 +1479,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		X_VerifyParseState(_PS_Block);
 		X_CheckDocument(_getInlineDepth()==0);
 		//_popInlineFmt();
-		X_CheckError(getDoc()->appendFmt(&m_vecInlineFmt));
+		X_CheckError(appendFmt(&m_vecInlineFmt));
 		return;
 
 	case TT_OTHER:
@@ -1446,10 +1538,10 @@ FG_Graphic * IE_Imp_XHTML::importDataURLImage (const XML_Char * szData)
 			UT_DEBUGMSG(("importDataURLImage: URL-embedded data does not appear to be an image...\n"));
 			return 0;
 		}
-	const char * b64bufptr = (const char *) szData;
+	const char * b64bufptr = static_cast<const char *>(szData);
 
 	while (*b64bufptr) if (*b64bufptr++ == ',') break;
-	size_t b64length = (size_t) strlen (b64bufptr);
+	size_t b64length = static_cast<size_t>(strlen (b64bufptr));
 	if (b64length == 0)
 		{
 			UT_DEBUGMSG(("importDataURLImage: URL-embedded data has no data?\n"));
@@ -1458,7 +1550,7 @@ FG_Graphic * IE_Imp_XHTML::importDataURLImage (const XML_Char * szData)
 
 	size_t binmaxlen = ((b64length >> 2) + 1) * 3;
 	size_t binlength = binmaxlen;
-	char * binbuffer = (char *) malloc (binmaxlen);
+	char * binbuffer = static_cast<char *>(malloc (binmaxlen));
 	if (binbuffer == 0)
 		{
 			UT_DEBUGMSG(("importDataURLImage: out of memory\n"));
@@ -1481,7 +1573,7 @@ FG_Graphic * IE_Imp_XHTML::importDataURLImage (const XML_Char * szData)
 			FREEP(binbuffer);
 			return 0;
 		}
-	pBB->ins (0, (const UT_Byte *) binbuffer, binlength);
+	pBB->ins (0, reinterpret_cast<const UT_Byte *>(binbuffer), binlength);
 	FREEP(binbuffer);
 
 	IE_ImpGraphic * pieg = 0;
@@ -1507,7 +1599,7 @@ FG_Graphic * IE_Imp_XHTML::importDataURLImage (const XML_Char * szData)
 
 FG_Graphic * IE_Imp_XHTML::importImage (const XML_Char * szSrc)
 {
-	const char * szFile = (const char *) szSrc;
+	const char * szFile = static_cast<const char *>(szSrc);
 
 	if (strncmp (szFile, "http://", 7) == 0)
 		{
@@ -1585,7 +1677,7 @@ bool IE_Imp_XHTML::pushInline (const char * props)
 	api_atts[2] = NULL;
 
 	_pushInlineFmt (api_atts);
-	return getDoc()->appendFmt (&m_vecInlineFmt);
+	return appendFmt (&m_vecInlineFmt);
 }
 
 bool IE_Imp_XHTML::newBlock (const char * style_name, const char * css_style, const char * align)
@@ -1651,7 +1743,7 @@ bool IE_Imp_XHTML::newBlock (const char * style_name, const char * css_style, co
 				return false;
 			api_atts[3] = sz;
 		}
-	if (getDoc()->appendStrux (PTX_Block, api_atts) == 0)
+	if (!appendStrux (PTX_Block, api_atts))
 		{
 			return false;
 		}
@@ -1682,7 +1774,7 @@ bool IE_Imp_XHTML::requireSection ()
 {
 	if (m_parseState == _PS_Sec) return true;
 
-	if (getDoc()->appendStrux (PTX_Section,NULL) == 0)
+	if (!appendStrux (PTX_Section,NULL))
 		{
 			return false;
 		}
@@ -1726,7 +1818,7 @@ static void s_pass_whitespace (const char *& csstr)
 				continue;
 			}
 		}
-		else if (isspace ((int) u))
+		else if (isspace (static_cast<int>(u)))
 		{
 			csstr++;
 			continue;
@@ -1760,7 +1852,7 @@ static const char * s_pass_name (const char *& csstr)
 			while (static_cast<unsigned char>(*++csstr) & 0x80) { }
 			continue;
 		}
-		else if ((isspace ((int) u)) || (*csstr == ':'))
+		else if ((isspace (static_cast<int>(u))) || (*csstr == ':'))
 		{
 			name_end = csstr;
 			break;
@@ -1809,7 +1901,7 @@ static const char * s_pass_value (const char *& csstr)
 				break;
 			}
 		}
-		else if (!bQuoted && isspace ((int) u)) bSpace = true;
+		else if (!bQuoted && isspace (static_cast<int>(u))) bSpace = true;
 
 		csstr++;
 		if (!bSpace) value_end = csstr;
@@ -1825,12 +1917,12 @@ static bool s_pass_number (char *& ptr, bool & bIsPercent)
 			ptr++;
 		}
 	unsigned char u = static_cast<unsigned char>(*ptr);
-	if (!isdigit ((int) u)) return false;
+	if (!isdigit (static_cast<int>(u))) return false;
 
 	while (*ptr)
 		{
 			u = static_cast<unsigned char>(*ptr);
-			if (!isdigit ((int) u)) break;
+			if (!isdigit (static_cast<int>(u))) break;
 			ptr++;
 		}
 	if (*ptr == '%')
@@ -2023,7 +2115,7 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 							while (*ptr)
 								{
 									unsigned char u = static_cast<unsigned char>(*ptr);
-									if ((isdigit ((int) u)) || (*ptr == '%'))
+									if ((isdigit (static_cast<int>(u))) || (*ptr == '%'))
 										{
 											ptr++;
 											continue;
@@ -2084,7 +2176,7 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 				{
 					UT_Dimension units = UT_determineDimension (value, DIM_PX);
 					double d = UT_convertDimensionless (value);
-					float dim = (float) UT_convertDimensions (d, units, DIM_IN);
+					float dim = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 					UT_String tmp;
 					char * old_locale = setlocale (LC_NUMERIC, "C");
 					UT_String_sprintf (tmp, "%gin", dim);
@@ -2148,13 +2240,13 @@ static UT_UTF8String s_parseCSStyle (const UT_UTF8String & style, UT_uint32 css_
 		/* unfortunately there's no easy way to turn these two sequences into strings :-(
 		 * atm, anyway
 		 */
-		char * name = (char *) malloc (name_end - name_start + 1);
+		char * name = static_cast<char *>(malloc (name_end - name_start + 1));
 		if (name)
 		{
 			strncpy (name, name_start, name_end - name_start);
 			name[name_end - name_start] = 0;
 		}
-		char * value = (char *) malloc (value_end - value_start + 1);
+		char * value = static_cast<char *>(malloc (value_end - value_start + 1));
 		if (value)
 		{
 			strncpy (value, value_start, value_end - value_start);
